/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core;

import modesteditor.core.modest.ModestEditorMessages;
import modesteditor.core.stepsimulation.SimulationAction;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.editors.text.TextEditorActionContributor;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.texteditor.RetargetTextEditorAction;
import org.eclipse.ui.texteditor.TextEditorAction;

/**
 * Contributes MoDeST actions to the desktop's Edit menu and the toolbar.
 */
public class ModestActionContributor extends TextEditorActionContributor {

	protected RetargetTextEditorAction fContentAssistProposal;
	protected RetargetTextEditorAction fContentAssistTip;
	protected TextEditorAction simulationAction;

	/**
	 * Default constructor. Creates the action which does the content assistance.
	 */
	public ModestActionContributor() {
		super();
		fContentAssistProposal= new RetargetTextEditorAction(ModestEditorMessages.getResourceBundle(), "ContentAssistProposal.");
		fContentAssistProposal.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS); 
		fContentAssistTip= new RetargetTextEditorAction(ModestEditorMessages.getResourceBundle(), "ContentAssistTip."); 
		fContentAssistTip.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_CONTEXT_INFORMATION);
		simulationAction= new SimulationAction();
		
	}
	
	/*
	 * @see IEditorActionBarContributor#init(IActionBars)
	 */
	public void init(IActionBars bars) {
		super.init(bars);
		
		IMenuManager menuManager= bars.getMenuManager();
		IMenuManager editMenu= menuManager.findMenuUsingPath(IWorkbenchActionConstants.M_EDIT);
		if (editMenu != null) {
			editMenu.add(new Separator());
			editMenu.add(fContentAssistProposal);
			editMenu.add(fContentAssistTip);
		}	
		
		// Simulation launcher (hopefully a button)
		IToolBarManager toolBarManager= bars.getToolBarManager();
		if (toolBarManager != null) {
			toolBarManager.add(new Separator());
			toolBarManager.add(simulationAction);
		}
	}
	
	/**
	 * Sets the editor which gets the action contributions.
	 */
	private void doSetActiveEditor(IEditorPart part) {
		super.setActiveEditor(part);

		ITextEditor editor= null;
		if (part instanceof ITextEditor)
			editor= (ITextEditor) part;

		fContentAssistProposal.setAction(getAction(editor, "ContentAssistProposal"));
		fContentAssistTip.setAction(getAction(editor, "ContentAssistTip")); 

		//fTogglePresentation.setEditor(editor);
		//fTogglePresentation.update();
		
		simulationAction.setEditor(editor);
		simulationAction.update();
	}
	
	/*
	 * @see IEditorActionBarContributor#setActiveEditor(IEditorPart)
	 */
	public void setActiveEditor(IEditorPart part) {
		super.setActiveEditor(part);
		doSetActiveEditor(part);
	}
	
	/**
	 * Removes the active editor and calls dispose from the superclass.
	 */
	public void dispose() {
		doSetActiveEditor(null);
		super.dispose();
	}
	
}
