/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core;


import modesteditor.core.model.IModestSimulationListener;
import modesteditor.core.model.ModestSimulation;
import modesteditor.core.modest.ModestEditorMessages;
import modesteditor.core.stepsimulation.ISimulationListener;
import modesteditor.core.stepsimulation.ISimulationViewListener;
import modesteditor.core.stepsimulation.SimulationNode;
import modesteditor.core.stepsimulation.SimulationRoot;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.ide.ResourceUtil;
import org.eclipse.ui.internal.editors.text.EditorsPlugin;
import org.eclipse.ui.texteditor.AbstractDecoratedTextEditor;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.eclipse.ui.texteditor.DefaultRangeIndicator;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.texteditor.MarkerUtilities;
import org.eclipse.ui.texteditor.TextOperationAction;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;




/**
 * MoDeST specific text editor.
 */
public class ModestEditor extends AbstractDecoratedTextEditor implements IModestSimulationListener, ISimulationViewListener, ISimulationListener {
	
	/*private class DefineFoldingRegionAction extends TextEditorAction {

		public DefineFoldingRegionAction(ResourceBundle bundle, String prefix, ITextEditor editor) {
			super(bundle, prefix, editor);
		}
		
		private IAnnotationModel getAnnotationModel(ITextEditor editor) {
			return (IAnnotationModel) editor.getAdapter(ProjectionAnnotationModel.class);
		}
				
		
		 * @see org.eclipse.jface.action.Action#run()
		 
		public void run() {
			ITextEditor editor= getTextEditor();
			ISelection selection= editor.getSelectionProvider().getSelection();
			if (selection instanceof ITextSelection) {
				ITextSelection textSelection= (ITextSelection) selection;
				if (!textSelection.isEmpty()) {
					IAnnotationModel model= getAnnotationModel(editor);
					if (model != null) {
						
						int start= textSelection.getStartLine();
						int end= textSelection.getEndLine();
						
						try {
							IDocument document= editor.getDocumentProvider().getDocument(editor.getEditorInput());
							int offset= document.getLineOffset(start);
							int endOffset= document.getLineOffset(end + 1);
							Position position= new Position(offset, endOffset - offset);
							model.addAnnotation(new ProjectionAnnotation(), position);
						} catch (BadLocationException x) {
							// ignore
						}
					}
				}
			}
		}
	}*/
	
	

	/** The outline page */
	private ModestContentOutlinePage fOutlinePage;
	/** The projection support */
	// TODO check where it comes from.
	//private ProjectionSupport fProjectionSupport;
	
	/** The high-level model of the program in the editor. */
	private ModestSimulation sim;
	
	private static ModestEditor editor;
	
	//protected boolean dirty = false;
	
	private ArrayList<IMarker> mySimMarkers;
	
	public ModestEditor() {
		super();
		editor=this;
	}
	
	/** Called by Eclipse to initialize the editor. Sets the editor to use
	 * the Modest plug-in's preference store, and configures its syntax
	 * highlighting, etc. See ModestSourceViewerConfiguration. */
	protected void initializeEditor () {
		super.initializeEditor();
		setPreferenceStore(new ChainedPreferenceStore(new IPreferenceStore[] {
				ModestEditorPlugin.getDefault().getPreferenceStore(),
				EditorsPlugin.getDefault().getPreferenceStore()}));
        setSourceViewerConfiguration(new ModestSourceViewerConfiguration(this));
        setRangeIndicator(new DefaultRangeIndicator());
        showOverviewRuler();
        //System.err.println("DocProvider nach init: " + getDocumentProvider());
	}
	
	/** Returns this editor's outline page if requested. (This is how
	 * Eclipse gets the outline from the editor.) If the request is for
	 * something else, passes it on to the superclass. */
	public Object getAdapter (Class adaptTo) {
        if (adaptTo.equals(IContentOutlinePage.class)) {
            if (fOutlinePage == null)
            	fOutlinePage = new ModestContentOutlinePage(this);
            return fOutlinePage;
        }
        return super.getAdapter(adaptTo);
    }
	
	/** The <code>ModestEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method extend the 
	 * actions to add those specific to the receiver
	 */
	protected void createActions() {
		super.createActions();
		
		IAction a= new TextOperationAction(ModestEditorMessages.getResourceBundle(), "ContentAssistProposal.", this, ISourceViewer.CONTENTASSIST_PROPOSALS);
		a.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS);
		setAction("ContentAssistProposal", a); 
		
		/*a= new TextOperationAction(ModestEditorMessages.getResourceBundle(), "ContentAssistTip.", this, ISourceViewer.CONTENTASSIST_CONTEXT_INFORMATION); 
		a.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_CONTEXT_INFORMATION);
		setAction("ContentAssistTip", a); 
		
		a= new DefineFoldingRegionAction(ModestEditorMessages.getResourceBundle(), "DefineFoldingRegion.", this); 
		setAction("DefineFoldingRegion", a);*/
	}
	
	
	
	/** Called by Eclipse when creating the editor. Adds bracket matching. *//*
	protected void configureSourceViewerDecorationSupport (SourceViewerDecorationSupport support) {
		super.configureSourceViewerDecorationSupport(support);
		support.setCharacterPairMatcher(new SmlBracketMatcher());
		support.setMatchingCharacterPainterPreferenceKeys(SmlUiPlugin.SML_BRACKET_MATCHING_ENABLED, SmlUiPlugin.SML_BRACKET_MATCHING_COLOR);
	}*/
	
	/** Returns the ModestSimulation representing the document in this editor. */
	public ModestSimulation getSimulation () {
		if (sim == null) {
			IDocument document = getDocumentProvider().getDocument(getEditorInput());
			sim = new ModestSimulation(document);
			sim.addListener(this);
		}
//		System.err.println("DocProvider: " + getDocumentProvider());
//		System.err.println("EditorInput: " + getEditorInput());
//		System.err.println("IDocument: " + getDocumentProvider().getDocument(getEditorInput()));
		
		return sim;
	}
	
	
	/** Called when the ModestSimulation is updated. 
	 * Does nothing for the time being.
	 * Removes existing error markers,
	 * and optionally adds markers for the errors in the updated program. */
	/* We always remove all markers to avoid any markers getting left over
	 * after the user disables error marking. */
	public void simulationChanged (ModestSimulation sim) {
		removeErrorMarkers();
		if (ModestEditorPlugin.getDefault().getPluginPreferences().getBoolean(ModestEditorPlugin.MODEST_MARK_ERRORS)){
			addErrorMarkers(sim);
			addWarningMarkers(sim);
		}
	}
	
	/**
	 * When the SimulationRoot changes after a step in the simulation, remove the highlighting.
	 * @param sim
	 */
	public void simulationChanged(SimulationRoot sim) {
		deleteMarkers();		
	}
	
	public static ModestEditor getEditor() {
		return editor;
	}
	
	
	/** Removes all error markers from the editor's document. */
	private void removeErrorMarkers () {
		//if (!(getEditorInput() instanceof IFileEditorInput)) return;
		//IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		IResource resource = ResourceUtil.getResource(getEditorInput());
		if (resource != null){ 
		try {
			//file.deleteMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
			if (resource.findMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE) != null)
				resource.deleteMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
		} catch (CoreException e) {e.printStackTrace();}
		}
		
	}
	
	/** Adds markers for all errors in the given program. */
	private void addErrorMarkers (ModestSimulation sim) {
		//if (!(getEditorInput() instanceof IFileEditorInput)) return;
		//IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		IResource resource = ResourceUtil.getResource(getEditorInput());
		IRegion[] problems = sim.getProblems();
		if (resource != null){
		try {
			for (int i = 0; i < problems.length; i++) {
				Map attributes = new HashMap();
				attributes.put(IMarker.MESSAGE, "Syntax error");
				attributes.put(IMarker.CHAR_START, new Integer(problems[i].getOffset()));
				attributes.put(IMarker.CHAR_END, new Integer(problems[i].getOffset()+problems[i].getLength()));
				attributes.put(IMarker.SEVERITY, new Integer(IMarker.SEVERITY_ERROR));
				attributes.put(IMarker.TRANSIENT, Boolean.TRUE);
				//MarkerUtilities.createMarker(file, attributes, IMarker.PROBLEM);
				MarkerUtilities.createMarker(resource, attributes, IMarker.PROBLEM);
			}
		} catch (CoreException e) {e.printStackTrace();}
		}
	}
	
	/** Adds markers for all unused variables in the given program. */
	private void addWarningMarkers (ModestSimulation sim) {
		//if (!(getEditorInput() instanceof IFileEditorInput)) return;
		//IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		IResource resource = ResourceUtil.getResource(getEditorInput());
		IRegion[] problems = sim.getWarnings();
		if (resource != null){
		try {
			for (int i = 0; i < problems.length; i++) {
				Map attributes = new HashMap();
				attributes.put(IMarker.MESSAGE, "Unused variable");
				attributes.put(IMarker.CHAR_START, new Integer(problems[i].getOffset()));
				attributes.put(IMarker.CHAR_END, new Integer(problems[i].getOffset()+problems[i].getLength()));
				attributes.put(IMarker.SEVERITY, new Integer(IMarker.SEVERITY_WARNING));
				attributes.put(IMarker.TRANSIENT, Boolean.TRUE);
				//MarkerUtilities.createMarker(file, attributes, IMarker.PROBLEM);
				MarkerUtilities.createMarker(resource, attributes, IMarker.PROBLEM);
			}
		} catch (CoreException e) {e.printStackTrace();}
		}
	}
	
	public void simulationViewChanged(Object selection) {
		// TODO Auto-generated method stub
		
			deleteMarkers();

			IDocument document = getDocumentProvider().getDocument(getEditorInput());
			IResource resource = ResourceUtil.getResource(getEditorInput());
			if (selection instanceof SimulationNode && resource != null){
				/*int line = ((SimulationNode)selection).getLine()-1;
				int column = ((SimulationNode)selection).getColumn();
				int offset = document.getLineOffset(line) + column -1;
				int length = ((SimulationNode)selection).getName().length();
				//editor.setHighlightRange(offset, length, false);
				//editor.showHighlightRangeOnly(true);
				
				
				IMarker marker = resource.createMarker(IMarker.PROBLEM);//("modesteditor.core.simulationmarker");//IMarker.TEXT
				marker.setAttribute(IMarker.MESSAGE, "active part of the simulation");
				marker.setAttribute(IMarker.CHAR_START, offset);
				marker.setAttribute(IMarker.CHAR_END, offset+length);
				//marker.setAttribute("isParallel", false);
				mySimMarker = marker;*/
				
				mySimMarkers = ((SimulationNode)selection).createSimHighlighting(resource, document, false);

			}
		

	}
	
	public void deleteMarkers(){
		try{
			if (mySimMarkers != null)
				for (int i = 0; i < mySimMarkers.size(); i++) {
					mySimMarkers.get(i).delete();
				}
		} catch (CoreException e) {
			System.err.println("war wohl nix mit dem Marker");
		}
	}
	
	/**
	 * Allow the Save As ... command.
	 */
	@Override
	public boolean isSaveAsAllowed() {
		return true;
		//return super.isSaveAsAllowed();
	}
	
	protected void performSaveAs(IProgressMonitor progressMonitor) {
		 IPreferenceStore store= EditorsUI.getPreferenceStore();
		 String key= getEditorSite().getId() + ".internal.delegateSaveAs";
		 store.setValue(key, true);
		 super.performSaveAs(progressMonitor);
		}
	
	
	/** Called when the cursor position changes. Selects the binding at that
	 * position in the outline page. *//*
	protected void handleCursorPositionChanged () {
		super.handleCursorPositionChanged();
		ISourceViewer sourceViewer = getSourceViewer();
		int offset = widgetOffset2ModelOffset(sourceViewer, sourceViewer.getTextWidget().getCaretOffset());
		Model model = sim.getModel(offset);
		if (binding != null)
			outlinePage.select(binding);
		else
			outlinePage.deselect();
	}*/
	
	
	
	
	
	
	
	/*
	 * 
	 * Old code before leaning onto the SML example 
	 * 
	*//**
	 * Default constructor
	 *
	 *//*
	public ModestEditor() {
		super();
		//setDocumentProvider(ModestEditorPlugin.getDefault().getDocumentProvider());
		this.fOutlinePage = new ModestContentOutlinePage(this);
		System.err.println("Der documentProvider von ModestEditor. " + getDocumentProvider());
	}
	
	*//** The <code>ModestEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method extend the 
	 * actions to add those specific to the receiver
	 *//*
	protected void createActions() {
		super.createActions();
		
		IAction a= new TextOperationAction(ModestEditorMessages.getResourceBundle(), "ContentAssistProposal.", this, ISourceViewer.CONTENTASSIST_PROPOSALS);
		a.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS);
		setAction("ContentAssistProposal", a); 
		
		a= new TextOperationAction(ModestEditorMessages.getResourceBundle(), "ContentAssistTip.", this, ISourceViewer.CONTENTASSIST_CONTEXT_INFORMATION); 
		a.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_CONTEXT_INFORMATION);
		setAction("ContentAssistTip", a); 
		
		a= new DefineFoldingRegionAction(ModestEditorMessages.getResourceBundle(), "DefineFoldingRegion.", this); 
		setAction("DefineFoldingRegion", a);
	}
	
	*//** The <code>ModestEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs any extra 
	 * disposal actions required by the modest editor.
	 *//*
	public void dispose() {
		if (fOutlinePage != null)
			fOutlinePage.setInput(null);
		super.dispose();
	}
	
	*//** The <code>ModestEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs any extra 
	 * revert behavior required by the modest editor.
	 *//*
	public void doRevertToSaved() {
		super.doRevertToSaved();
		if (fOutlinePage != null)
			fOutlinePage.update();
	}
	
	*//** The <code>ModestEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs any extra 
	 * save behavior required by the modest editor.
	 * 
	 * @param monitor the progress monitor
	 *//*
	public void doSave(IProgressMonitor monitor) {
		super.doSave(monitor);
		if (fOutlinePage != null)
			fOutlinePage.update();
		System.err.println("DocProvider bei save: " + getDocumentProvider());
	}
	
	*//** The <code>ModestEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs any extra 
	 * save as behavior required by the modest editor.
	 *//*
	public void doSaveAs() {
		super.doSaveAs();
		if (fOutlinePage != null)
			fOutlinePage.update();
	}
	
	*//** The <code>ModestEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs sets the 
	 * input of the outline page after AbstractTextEditor has set input.
	 * 
	 * @param input the editor input
	 * @throws CoreException in case the input can not be set
	 *//* 
	public void doSetInput(IEditorInput input) throws CoreException {
		super.doSetInput(input);
		if (fOutlinePage != null)
			fOutlinePage.setInput(input);
	}
	
	
	 * @see org.eclipse.ui.texteditor.ExtendedTextEditor#editorContextMenuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 
	*//**
	 * TODO see if we need all three actions
	 *//*
	protected void editorContextMenuAboutToShow(IMenuManager menu) {
		super.editorContextMenuAboutToShow(menu);
		addAction(menu, "ContentAssistProposal"); 
		addAction(menu, "ContentAssistTip"); 
		addAction(menu, "DefineFoldingRegion");
	}
	
	*//** The <code>ModestEditor</code> implementation of this 
	 * <code>AbstractTextEditor</code> method performs gets
	 * the modest content outline page if request is for a an 
	 * outline page.
	 * 
	 * @param required the required type
	 * @return an adapter for the required type or <code>null</code>
	 *//* 
	public Object getAdapter(Class required) {
		if (IContentOutlinePage.class.equals(required)) {
			if (fOutlinePage == null) {
				fOutlinePage= new ModestContentOutlinePage(getDocumentProvider(), this);
				if (getEditorInput() != null)
					fOutlinePage.setInput(getEditorInput());
			}
			return fOutlinePage;
		}
		
		if (fProjectionSupport != null) {
			Object adapter= fProjectionSupport.getAdapter(getSourceViewer(), required);
			if (adapter != null)
				return adapter;
		}
		
		return super.getAdapter(required);
	}
	
	 (non-Javadoc)
	 * Method declared on AbstractTextEditor
	 
	protected void initializeEditor() {
		super.initializeEditor();
		setSourceViewerConfiguration(new ModestSourceViewerConfiguration());
		System.err.println("Der documentProvider von ModestEditor nach init. " + getDocumentProvider());
	}
	
	
	 * @see org.eclipse.ui.texteditor.ExtendedTextEditor#createSourceViewer(org.eclipse.swt.widgets.Composite, org.eclipse.jface.text.source.IVerticalRuler, int)
	 
	protected ISourceViewer createSourceViewer(Composite parent, IVerticalRuler ruler, int styles) {
		
		fAnnotationAccess= createAnnotationAccess();
		fOverviewRuler= createOverviewRuler(getSharedColors());
		
		ISourceViewer viewer= new ProjectionViewer(parent, ruler, getOverviewRuler(), isOverviewRulerVisible(), styles);
		// ensure decoration support has been created and configured.
		getSourceViewerDecorationSupport(viewer);
		
		return viewer;
	}
	
	
	 * @see org.eclipse.ui.texteditor.ExtendedTextEditor#createPartControl(org.eclipse.swt.widgets.Composite)
	 
	public void createPartControl(Composite parent) {
		super.createPartControl(parent);
		ProjectionViewer viewer= (ProjectionViewer) getSourceViewer();
		
		// So no folding can be done.
		viewer.disableProjection();
		fProjectionSupport= new ProjectionSupport(viewer, getAnnotationAccess(), getSharedColors());
		fProjectionSupport.addSummarizableAnnotationType("org.eclipse.ui.workbench.texteditor.error");
		fProjectionSupport.addSummarizableAnnotationType("org.eclipse.ui.workbench.texteditor.warning");
		fProjectionSupport.install();
		viewer.doOperation(ProjectionViewer.TOGGLE);
	}
	
	
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#adjustHighlightRange(int, int)
	 
	protected void adjustHighlightRange(int offset, int length) {
		ISourceViewer viewer= getSourceViewer();
		if (viewer instanceof ITextViewerExtension5) {
			ITextViewerExtension5 extension= (ITextViewerExtension5) viewer;
			extension.exposeModelRange(new Region(offset, length));
		}
	}
	
	
	
	public IDocumentProvider getDocumentProvider() {
		if (provider==null) provider = new FileDocumentProvider();
		return provider;
	}*/
	
}
