/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core;

import modesteditor.core.model.Model;
import modesteditor.core.model.ModestAction;
import modesteditor.core.model.ModestException;
import modesteditor.core.model.ModestType;
import modesteditor.core.model.ModestVariable;
import modesteditor.core.model.Process;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;


/**
 * A provider for the MoDeST models in the outline. Gives each model a 
 * label and an image.
 */
public class ModestLabelProvider extends LabelProvider {
	private Map imageCache;
	public ModestLabelProvider() {
		// TODO needs probably not 11 elements in the map.
		imageCache = new HashMap(11);
	}

	/**
	 * Returns an image to a given Object, the Object has to be a Model.
	 */
	public Image getImage(Object element) {
		ImageDescriptor descriptor = null;
		//System.err.println("Getting descriptor of: " + ((Model)element).getName());
		//if (element instanceof Process) System.err.println("Process: " + element.toString());
		   if (element instanceof Process) {
		       descriptor = ModestEditorPlugin.getImageDescriptor("process.gif");
		   } else if (element instanceof ModestAction) {
			   //descriptor = ModestEditorPlugin.getImageDescriptor("circle_blue.gif");
		       descriptor = ModestEditorPlugin.getImageDescriptor("action.gif");
		   } else if (element instanceof ModestType) {
			   //descriptor = ModestEditorPlugin.getImageDescriptor("circle_blue.gif");
		       descriptor = ModestEditorPlugin.getImageDescriptor("struct.gif");
		   } else if (element instanceof ModestException) {
			   //descriptor = ModestEditorPlugin.getImageDescriptor("circle_blue.gif");
		       descriptor = ModestEditorPlugin.getImageDescriptor("exception.gif");
		   } else if (element instanceof ModestVariable) {
			   //descriptor = ModestEditorPlugin.getImageDescriptor("circle_blue.gif");
		       descriptor = ModestEditorPlugin.getImageDescriptor("variable.gif");
		   } else {
		       throw unknownElement(element);
		   }

		   if (descriptor==null) 
			   descriptor = ModestEditorPlugin.getImageDescriptor("diamond_blue.gif");//return null;
		   //obtain the cached image corresponding to the descriptor
		   Image image = (Image)imageCache.get(descriptor);
		   if (image == null) {
		       image = descriptor.createImage();
		       imageCache.put(descriptor, image);
		   }
		   return image;
	}
	
	protected RuntimeException unknownElement(Object element) {
		return new RuntimeException("Unknown type of element in tree of type " + element.getClass().getName());
	}

	/**
	 * Returns the text (label) displayed in the outline.
	 */
	public String getText(Object element) {
		return ((Model)element).getName();
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose() {
		for (Iterator i = imageCache.values().iterator(); i.hasNext();) {
			((Image) i.next()).dispose();
		}
		imageCache.clear();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
	 */
	public boolean isLabelProperty(Object element, String property) {
		// TODO Auto-generated method stub
		return false;
	}

	

}
