/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */

package modesteditor.core;

import org.eclipse.jface.text.rules.*;

import java.util.ArrayList;
import java.util.List;

/**
 * This scanner recognizes the MoDeST multi line comments
 */
public class ModestPartitionScanner extends RuleBasedPartitionScanner {
	public final static String MODEST_MULTILINE_COMMENT= "__modest_multiline_comment"; //$NON-NLS-1$
	public final static String MODEST_SINGLELINE_COMMENT= "__modest_singleline_comment"; //$NON-NLS-1$
	public final static String MODEST_DEFAULT = "__modest_default";
	/** The partition types in a MoDeST document. */
	public final static String[] MODEST_PARTITION_TYPES= new String[] { MODEST_SINGLELINE_COMMENT, MODEST_MULTILINE_COMMENT };
	

	/**
	 * Detector for empty comments.
	 */
	static class EmptyCommentDetector implements IWordDetector {

		/* (non-Javadoc)
		* Method declared on IWordDetector
	 	*/
		public boolean isWordStart(char c) {
			return (c == '/');
		}

		/* (non-Javadoc)
		* Method declared on IWordDetector
	 	*/
		public boolean isWordPart(char c) {
			return (c == '*' || c == '/');
		}
	}
	
	/**
	 * 
	 */
	static class WordPredicateRule extends WordRule implements IPredicateRule {
		
		private IToken fSuccessToken;
		
		public WordPredicateRule(IToken successToken) {
			super(new EmptyCommentDetector());
			fSuccessToken= successToken;
			addWord("/**/", fSuccessToken); //$NON-NLS-1$
		}
		
		/*
		 * @see org.eclipse.jface.text.rules.IPredicateRule#evaluate(ICharacterScanner, boolean)
		 */
		public IToken evaluate(ICharacterScanner scanner, boolean resume) {
			return super.evaluate(scanner);
		}

		/*
		 * @see org.eclipse.jface.text.rules.IPredicateRule#getSuccessToken()
		 */
		public IToken getSuccessToken() {
			return fSuccessToken;
		}
	}
	
	/**
	 * Creates the partitioner and sets up the appropriate rules.
	 */
	public ModestPartitionScanner() {
		super();

		IToken comment= new Token(MODEST_MULTILINE_COMMENT);
		IToken scomment = new Token(MODEST_SINGLELINE_COMMENT);

		List rules= new ArrayList();

		// Add rule for single line comments.
		rules.add(new EndOfLineRule("//", scomment)); //$NON-NLS-1$

		// Add rule for strings and character constants.
		//rules.add(new SingleLineRule("\"", "\"", Token.UNDEFINED, '\\')); //$NON-NLS-2$ //$NON-NLS-1$
		//rules.add(new SingleLineRule("'", "'", Token.UNDEFINED, '\\')); //$NON-NLS-2$ //$NON-NLS-1$

		// Add special case word rule.
		rules.add(new WordPredicateRule(comment));

		// Add rule for multi-line comments.
		rules.add(new MultiLineRule("/*", "*/", comment, (char) 0, true)); //$NON-NLS-1$ //$NON-NLS-2$

		IPredicateRule[] result= new IPredicateRule[rules.size()];
		rules.toArray(result);
		setPredicateRules(result);
	}
	
	
	
}
