package modesteditor.core;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.reconciler.DirtyRegion;
import org.eclipse.jface.text.reconciler.IReconcilingStrategy;

/** Updates the ModestSimulation representing a document open in the workspace,
 * whenever the document is changed. 
 */
public class ModestReconcilingStrategy implements IReconcilingStrategy {

	/** The editor whose document this instance maintains. */
	private ModestEditor editor;
	
	/** Creates a reconciling strategy to update the given editor's model
	 * when the document changes. It should only be used with this editor. */
	public ModestReconcilingStrategy(ModestEditor editor) {
		this.editor = editor;
	}
	
	public void reconcile(IRegion partition) {
		reconcile();
	}

	public void reconcile(DirtyRegion dirtyRegion, IRegion subRegion) {
		reconcile();
	}

	/** Tells this reconciling strategy on which document it will work.
	 * However, since the containing editor has already been given in the
	 * constructor, this method doesn't do anything. */
	public void setDocument(IDocument document) {}

	/** Blindly updates the entire simulation. */
	private void reconcile () {
		editor.getSimulation().update();
	}
	
}
