/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.antlrparser;

import antlr.CommonAST;
import antlr.Token;
import antlr.collections.AST;

/**
 * This is the AST class that will be used in the AST such that
 * the TreeWalker has access to line and column information.
 * 
 * CommonAST(default) from Antlr does not store line information.
 * So we extend it, so use setASTNodeClass() to set it up in parser.
 */
public class ASTWithOffsetInfos extends CommonAST {
	
	private static final long serialVersionUID = 1L;
	private int line = 0;
	private int column = 0;

	public void initialize(Token tok) {
		super.initialize(tok);
		line = tok.getLine();
		column = tok.getColumn();
	}

	public void initialize (AST ast) {
		super.initialize(ast);
		if (ast instanceof ASTWithOffsetInfos){
			line = ((ASTWithOffsetInfos)ast).getLine();
			column = ((ASTWithOffsetInfos)ast).getColumn();
		}
	}

	public int getLine() {
		return line;
	}
	
	public int getColumn() {
		return column;
	}
	
	/*public String getName() {
		return "modesteditor.core.antlrparser.ASTWithOffsetInfos";
	}*/

}
