/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.antlrparser;

import antlr.CommonAST;
import antlr.Token;
import antlr.collections.AST;

/**
 * This is the AST class that will be used in the AST such that
 * the TreeWalker has access to line and column information.
 * 
 * CommonAST(default) from Antlr does not store line information.
 * So we extend it, so use setASTNodeClass() to set it up in parser.
 *
 */
public class ModestAST extends CommonAST {

	private static final long serialVersionUID = 1L;
	private int line = 0;
	private int column = 0;
	private Symbol symbol;
	private String symbolType;
	private boolean guardUrgent;
	private double weight;
	private String typeKind;
	private String procName;
	
	public void initialize(Token tok) {
		super.initialize(tok);
		line = tok.getLine();
		column = tok.getColumn();
	}

	public void initialize (AST ast) {
		super.initialize(ast);
		if (ast instanceof ASTWithOffsetInfos){
			line = ((ASTWithOffsetInfos)ast).getLine();
			column = ((ASTWithOffsetInfos)ast).getColumn();
		}
	}

	public int getLine() {
		return line;
	}
	
	public int getColumn() {
		return column;
	}
	
	public Symbol getSymbol() {
		return symbol;
	}
	
	public void setSymbol(Symbol sym) {
		symbol = sym;
	}
	
	public String getSymbolType() {
		return symbolType;
	}
	
	public void setSymbolType(String st) {
		symbolType = st;
	}
	
	public void setLine(int line) {
		this.line = line;
	}
	
	public void setColumn(int column) {
		this.column = column;
	}
	
	/** Set in the case of a GUARD_AST urgent flag */
	public void setGuardType(boolean urgent)
	{
		guardUrgent = urgent;
	}
	
	/// returns for GUARD_AST's wether it's urgent.
	public boolean isUrgent()
	{
		return guardUrgent;
	}
	
	/**
	 * @return the weight
	 */
	public double getWeight() {
		return weight;
	}

	/**
	 * @param weight the weight to set
	 */
	public void setWeight(double weight) {
		this.weight = weight;
	}

	/**
	 * In case of a process definition, return the kind (either set, range or struct)
	 * @return the typeKind
	 */
	public String getTypeKind() {
		return typeKind;
	}

	/**
	 * @param typeKind the typeKind to set
	 */
	public void setTypeKind(String typeKind) {
		this.typeKind = typeKind;
	}

	/**
	 * @return the procName
	 */
	public String getProcName() {
		return procName;
	}

	/**
	 * @param procName the procName to set
	 */
	public void setProcName(String procName) {
		this.procName = procName;
	}
	
	
}
