header {
package modesteditor.core.antlrparser;

import modesteditor.core.ModestEditor;
    
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream; 
import java.util.ArrayList;

import org.eclipse.jface.text.IDocument; 
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.Region;

import antlr.CommonAST;  
}

//Keywords "extern" and "const" work on their own. But if you combine them "extern const int a = 10;" produces a syntax error.
//This is important for the varying the values of some external constants to be used by Studies in Mobius.

class ModestParser extends Parser;
options {
	buildAST=true;
	defaultErrorHandler = true;      // Don't generate parser error handlers
	//defaultErrorHandler = false;
	k=4;
	ASTLabelType="ASTWithOffsetInfos";
}

// Define some methods and variables to use in the generated parser.
{
  // Define a main
  public static void main(String[] args) {
    // Use a try/catch block for parser exceptions
    try {
      // if we have at least one command-line argument
      if (args.length > 0 ) {
        //System.err.println("Parsing...");

        // for each directory/file specified on the command line
        for(int i=0; i< args.length;i++){
        	//System.err.println("Doing file: " + args[i]);
        	doFile(new File(args[i])); // parse it
        	
          	}
      }
      else
        System.err.println("Usage: java ModestParser <directory name>");

    }
    catch(Exception e) {
      System.err.println("exception: "+e);
      e.printStackTrace(System.err);   // so we can get stack trace
    }
  }


  // This method decides what action to take based on the type of
  //   file we are looking at
  public static void doFile(File f) throws Exception {
    // If this is a directory, walk each file/dir in that directory
    if (f.isDirectory()) {
      String files[] = f.list();
      for(int i=0; i < files.length; i++)
        doFile(new File(f, files[i]));
    }

    // otherwise, if this is a MoDeST file, parse it!
    else {
      //System.err.println("-------------------------------------------");
      //System.err.println(f.getAbsolutePath());
      parseFile(new FileInputStream(f));
    }
  }

  // Here's where we do the real work...
  public static void parseFile(InputStream s) throws Exception {
    try {
      // Create a scanner that reads from the input stream passed to us
      ModestLexer lexer = new ModestLexer(s);

      // Create a parser that reads from the scanner
      ModestParser parser = new ModestParser(lexer);

      // start parsing at the compilationUnit rule
      parser.simulation();
    }
    catch (EndOfFileException e) {
    	//System.err.println("Finished parsing!");
    	
    }
    catch (Exception e) {
      System.err.println("parser exception: "+e);
      e.printStackTrace();   // so we can get stack trace		
    }
  }
  
  public ArrayList<IRegion> problems;
  private int actualBreakLine;
  private int actualDocOffset;
  
  public void setActualDocumentOffset(int offset){
	  actualDocOffset=offset;
  }
  
  public void reportError(RecognitionException ex){
	
	try {
		ModestEditor editor = ModestEditor.getEditor();
		IDocument document = editor.getDocumentProvider().getDocument(editor.getEditorInput());
		
		int actualLine = document.getLineOfOffset(actualDocOffset);
		System.err.println("Syntax error at line " + (ex.getLine()+actualLine) + ": " + ex.getMessage());
		
		
		int lineOffset = document.getLineOffset(ex.getLine()-1+actualLine);
		int nextLineOffset = document.getLineOffset(ex.getLine()+actualLine);
		problems.add(new Region(lineOffset,nextLineOffset-lineOffset-1));
	} catch (BadLocationException e) {
		System.err.println("Bad line offset for the erroneous element.");
	}
  }
  
  public int getBreakLine() {
	  return actualBreakLine;
  }
}

// the following tag is used to find the start of the rules section for
//   automated chunk-grabbing when displaying the page


simulation
{astFactory.setASTNodeClass(ASTWithOffsetInfos.class);
problems = new ArrayList<IRegion>();}
	: ( defaultsSection! )? (declaration)* sequentialProcess!//expression!		
		{actualBreakLine = LT(1).getLine();}
		// end of file
	;
	
	
/** Rule to take care of defaults section
 */
defaultsSection
	: "defaults" LBRACE
	( "actions" EQUAL ("patient" | "impatient") SEMI
	| "guards" EQUAL ("urgent" | "not_urgent") SEMI
	)+
	/*( setting:ID "=" value:ID ";"
 	{ 
		if( setting.getText() == "actions" )
		{
			if( value.getText() == "patient" )
				//def_actions_patient = true;
				return;
			else if( value.getText() == "impatient" )
				//def_actions_patient = false;
				return;
			else
			{
				System.err.println("Illegal value specified for default '" + setting.getText() + "'");
				throw new SemanticException("Illegal value specified for default '" + setting.getText() + "'", 
					this.getFilename(), setting.getLine(), setting.getColumn());
			}
		}
		else if( setting.getText() == "guards" )
		{
			if( value.getText() == "urgent" )
				//def_guards_urgent = true;
				return;
			else if( value.getText() == "not_urgent" )
				//def_guards_urgent = false;
				return;
			else
			{
				System.err.println("Illegal value specified for default '" + setting.getText() + "'");
				throw new SemanticException("Illegal value specified for default '" + setting.getText() + "'", 
					this.getFilename(), setting.getLine(), setting.getColumn());
			}
		}
		else
		{
			System.err.println("Unknown default: '" + setting.getText() + "'");
			throw new SemanticException("Unknown default: '" + setting.getText() + "'", 
					this.getFilename(), setting.getLine(), setting.getColumn());
		}
		
	}
	)+*/ RBRACE
;
	
declaration
	: actionDeclaration
	  | variableDeclaration
	  | exceptionDeclaration
	  | structDeclaration
	  | processDeclaration
	;
	
actionDeclaration
//{System.err.println("Action declaration in parser found"); }
	: "action" idList SEMI //{ System.err.println("Action declaration found: " + i.get(0)); }//+ i.getText()); }
	| "patient" "action" idList SEMI
	| "impatient" "action" idList SEMI
	;

idList //returns ArrayList<String>
	//{ ArrayList<String> myL = new ArrayList<String>(0); }
	
	: ID (COMMA ID)*
	//{ myL.add(i.getText()); return myL; }
	;
	
variableDeclaration
//{System.err.println("Var declaration in parser found"); }
	: (modifier)? qType arrayDeclaration SEMI
	| (modifier)? "float" ID (EQUAL FLOAT)? (COMMA ID (EQUAL FLOAT)? )* SEMI
	| (modifier)? "int" ID (EQUAL INT)? (COMMA ID (EQUAL INT)? )* SEMI
	| (modifier)? "bool" ID (EQUAL ("true"|"false"))? (COMMA ID (EQUAL ("true"|"false"))? )* SEMI
	| (modifier)? "clock" ID (EQUAL INT)? (COMMA ID (EQUAL INT)? )* SEMI
	;
	
modifier
	: "const" | "extern"
	;

arrayDeclaration
	: ID (LBRACKET (expression!) RBRACKET)+ (COMMA ID (LBRACKET (expression)! RBRACKET)+ )*
	//{ System.err.println("In parser arrayDeclaration"); }
	;
	
exceptionDeclaration
	: "exception" idList SEMI
	;
	
structDeclaration
	: "typedef" "struct" LBRACE (variableDeclaration!)+ RBRACE ID SEMI
	| "typedef" "set" LBRACE (idList!) RBRACE ID SEMI
	| "typedef" "range" LBRACKET INT DOTDOT INT RBRACKET ID SEMI
	;
	
processDeclaration
	: "process" ID LPAR (arguments!) RPAR LBRACE body RBRACE
	;
	
arguments
	: (type ID)*
	;
	
type
	: "int"
	| "float"
	| "clock"
	| "action"
	| "bool"
	| ID
	;

body
	: ( sequentialProcess!
	| declaration )*
	;

expression
	: orExpr
	| orExpr QUESTION orExpr COLON orExpr
	;

orExpr
	: andExpr
	| andExpr OROR orExpr
	;
	
andExpr
	: bitOrExpr
	| bitOrExpr ANDAND andExpr
	;	

bitOrExpr
	: bitExclOrExpr
	| bitExclOrExpr OR bitOrExpr
	;
	
bitExclOrExpr
	: bitAndExpr
	| bitAndExpr XOR bitExclOrExpr
	;
	
bitAndExpr
	: equalityExpr
	| equalityExpr AND bitAndExpr
	;
	
equalityExpr
	: relExpr
	| relExpr EQUALEQUAL equalityExpr
	| relExpr NOT_EQUAL equalityExpr
	;
	
relExpr
	: shiftExpr
	| shiftExpr GT relExpr
	| shiftExpr LT relExpr
	| shiftExpr GTE relExpr
	| shiftExpr LTE relExpr
	;
	
shiftExpr
	: additiveExpr
	| additiveExpr LEFTSHIFT shiftExpr
	| additiveExpr RIGHTSHIFT shiftExpr
	| additiveExpr ULEFTSHIFT shiftExpr
	| additiveExpr URIGHTSHIFT shiftExpr
	;

additiveExpr
	: multExpr 
	| multExpr PLUS additiveExpr
	| multExpr MINUS additiveExpr
	;
	
multExpr
	: unaryExpr 
	| unaryExpr TIMES multExpr
	| unaryExpr DIV multExpr
	| unaryExpr REMAINDER multExpr
	;
	
unaryExpr
	: PLUS baseExpr
	| MINUS baseExpr
	| NOT baseExpr
	| baseExpr
	;
	
baseExpr
	: INT
	| FLOAT
	| LPAR expression RPAR
	| set
	| range
	| "true"
	| "false"
	| ID
	;

set
	: LBRACE setElement (COMMA setElement)* RBRACE
	;
	
setElement
	: INT
	| ID
	| range
	| qType
	| LBRACE qID RBRACE
	;
	
qType
	: "int"
	| "float"
	| "clock"
	| "action"
	| ID
	;
	
qID
	: ID ( DOT ID 
		 | LBRACKET expression RBRACKET
		 )*
	;

range
	: LBRACKET ( qType  
			   | additiveExpr DOTDOT additiveExpr
			   ) RBRACKET
	;
	
tryBlock
	: "try" LBRACE body RBRACE ( "catch" LPAR ID RPAR LBRACE body RBRACE )+
	;
	
guardedProcess
	: "when" LPAR expression RPAR localProcess
	| "urgent" LPAR expression RPAR localProcess
	;
	
altStatement
	: "alt" LBRACE alternative RBRACE
	;
	
alternative
	: ( COLON COLON sequentialProcess )+
	;
	
loopStatement
	: "do" LBRACE alternative RBRACE
	;
	
parallelComposition
	: "par" LBRACE alternative RBRACE
	;
	
labelModification
	: "hide" label
	| "relabel" label
	| "extend" label
	| "prefix" label
	;
	
label
	: set
	| ID
	;
	
labelsByLabels
	: label "by" label (COMMA label "by" label)*
	;
	
assignement
	: ID assiOp expression
	;
	
assiOp
	: EQUAL | PLUS_EQUAL | MINUS_EQUAL | TIME_EQUAL | DIV_EQUAL 
	| REMAINDER_EQUAL | LSHIFT_EQUAL | RSHIFT_EQUAL | AND_EQUAL
	| OR_EQUAL | XOR_EQUAL | ULSHIFT_EQUAL | URSHIFT_EQUAL
	;

paramObject
	: ID LPAR params RPAR
	;
	
params
	: expression (COMMA expression)*
	|
	;
	
performAction
	: ID
	| ID paltStatement
	| ID assiBlock
	;
	
paltStatement
	: "palt" LBRACE (pAlternative)* RBRACE
	;
	
pAlternative
	: weight assiBlock SEMI sequentialProcess
	| weight assiBlock
	| weight sequentialProcess
	;
	
assiBlock
	: LBRACE EQUAL assignement (COMMA assignement)* EQUAL RBRACE
	;
	
weight
	: COLON COLON
	| COLON additiveExpr COLON
	;
	
sequentialProcess
	: localProcess //SEMI //TODO hier ohne SEMI ???
	| localProcess SEMI sequentialProcess
	;
	
localProcess
	: "break"
	| "return" expression
	| "throw" LPAR ID RPAR
	| tryBlock
	| altStatement
	| loopStatement
	| parallelComposition
	| labelModification localProcess
	| guardedProcess
	| LBRACE localProcess RBRACE
	| assignement
	| paramObject
	| performAction
	;
	
	
// --------------------------------------------------------------------
// Lexer begins here!
// --------------------------------------------------------------------

//Floating point variable: a float can only be initialized by number of the following forms:
//"10.0", "0.25". If I put something like "10" or "25" or "2.5e3", "2.5e-3", or "2.5E3", "2.5E-3" 
//or even "-10.0", it does not work. 

class ModestLexer extends Lexer;
options {
    k=4;// In case k=1 doesn't suffice
    charVocabulary = '\0'..'\377';
    testLiterals = false;
}

// Define Methods for the Lexer
{
	public void uponEOF() throws EndOfFileException {
		EndOfFileException exc = new EndOfFileException();
		//throw exc;
	}	
}



/*INT 
options {
  paraphrase = "an integer value";
}
: ('0'..'9')+ ;

FLOAT
options {
  paraphrase = "a floating point value";
}
: ('0'..'9')+ '.' ('0'..'9')+ ;*/

EXPONENT_PART:	( 'e' | 'E' ) ( '+' | '-' )? ('0'..'9')+ ;
FLOAT_SUFFIX: ('F'|'f'|'L'|'l') ;

NUMERIC:
	('0'..'9')+	{ $setType(INT); }
	(
	|	{ LA(2) >= '0' && LA(2) <= '9' }? '.' ('0'..'9')+ (EXPONENT_PART)? (FLOAT_SUFFIX)? { $setType(FLOAT); }
	|	EXPONENT_PART (FLOAT_SUFFIX)?  { $setType(FLOAT); }
	|	FLOAT_SUFFIX  { $setType(FLOAT); }
	)
;


// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
ID options { testLiterals=true; paraphrase = "an identifier";} 
    : ('a'..'z'|'A'..'Z') ('a'..'z'|'A'..'Z'|'0'..'9'|'_'|'$'|'-')*
	;

WS
options {
  paraphrase = "a whitespace character";
}
  : ( ' ' | '\t' | '\f'
    // handle newlines
    | ( "\r\n"  // DOS/Windows
      | '\r'    // Macintosh
      | '\n'    // Unix
      )
      // increment the line count in the scanner
      { newline(); }
    )
    { $setType(Token.SKIP); }
  ;
  
COMMENT
  : "//" (~('\n'|'\r'))*
    { $setType(Token.SKIP); }
  ;
  
// multiple-line comments
ML_COMMENT
  : "/*"
    (               /* '\r' '\n' can be matched in one alternative or by matching
                       '\r' in one iteration and '\n' in another. I am trying to
                       handle any flavor of newline that comes in, but the language
                       that allows both "\r\n" and "\r" and "\n" to all be valid
                       newline is ambiguous. Consequently, the resulting grammar
                       must be ambiguous. I'm shutting this warning off.
                    */
      options {
        generateAmbigWarnings=false;
      }
      :  { LA(2)!='/' }? '*'
      | '\r' '\n' {newline();}
      | '\r' {newline();}
      | '\n' {newline();}
      | ~('*'|'\n'|'\r')
    )*
    "*/"
    {$setType(Token.SKIP);}
  ;
  
CHAR
  : '\''! . '\''!
  ;

STRING
  : '"'!
    ( ~('"'|'\n'|'\r')
    )*
    ( '"'!
    | // nothing -- write error message
    )
  ;

DOT options { paraphrase = "'.'"; }	: '.'   ;
COLON options { paraphrase = "':'"; }	: ':'   ;
SEMI options { paraphrase = "';'"; }	: ';'   ;
COMMA options { paraphrase = "','"; }   : ','   ;
EQUAL options { paraphrase = "'='"; }   : '='   ;
LBRACKET options { paraphrase = "'['"; }: '['   ;
RBRACKET options { paraphrase = "']'"; }: ']'   ;
DOTDOT options { paraphrase = "'..'"; } : ".."  ;
LPAR options { paraphrase = "'('"; }    : '('   ;
RPAR options { paraphrase = "')'"; }    : ')'   ;
LBRACE options { paraphrase = "'{'"; }  : '{'   ;
RBRACE options { paraphrase = "'}'"; }  : '}'   ;
NOT_EQUAL options { paraphrase = "'!='"; }: "!="  ;
LT options { paraphrase = "'<'"; }      : '<'   ;
LTE options { paraphrase = "'<='"; }    : "<="  ;
GT options { paraphrase = "'>'"; }      : '>'   ;
GTE options { paraphrase = "'>='"; }    : ">="  ;
PLUS options { paraphrase = "'+'"; }    : '+'   ;
MINUS options { paraphrase = "'-'"; }   : '-'   ;
TIMES options { paraphrase = "'*'"; }   : '*'   ;
DIV options { paraphrase = "'/'"; }     : '/'   ;
NOT	options { paraphrase = "'!'"; }	: '!'   ;
XOR	options { paraphrase = "'^'"; }	: '^'   ;
REMAINDER options { paraphrase = "'%'"; }: '%'   ;
TWIDDLE options { paraphrase = "'~'"; } : '~'   ;
OR options { paraphrase = "'|'"; }      : '|'   ;
QUESTION options { paraphrase = "'?'"; }: '?'   ;
MINUSMINUS options { paraphrase = "'--'"; }: "--"  ;
PLUSPLUS options { paraphrase = "'++'"; }: "++"  ;
EQUALEQUAL options { paraphrase = "'=='"; }: "==";
LEFTSHIFT  : "<<"  ;
RIGHTSHIFT : ">>"  ;
ULEFTSHIFT : "<<<" ;
URIGHTSHIFT: ">>>" ;
PLUS_EQUAL : "+="  ;
MINUS_EQUAL: "-="  ;
TIMES_EQUAL: "*="  ;
DIV_EQUAL  : "/="  ;
AND_EQUAL  : "&="  ;
OR_EQUAL   : "|="  ;
XOR_EQUAL  : "^="  ;
REMAINDER_EQUAL: "%=" ;
LSHIFT_EQUAL: "<<=" ;
RSHIFT_EQUAL: ">>=" ;
ULSHIFT_EQUAL: "<<<=" ;
URSHIFT_EQUAL: ">>>=" ;
OROR options { paraphrase = "'||'"; }   : "||"  ;
ANDAND options { paraphrase = "'&&'"; } : "&&"  ;

{import modesteditor.core.model.Process;
import modesteditor.core.model.*;
import java.util.ArrayList;
import java.util.Vector;
}
class ModestTreeWalker extends TreeParser;
/*options {
	ASTLabelType="ASTWithOffsetInfos";
}*/


simulation returns [Process root = new Process("root",0,0)]
	{ //ArrayList<ArrayList<Model>> decl; 
		ArrayList<Model> decl;
	}
: (decl = declaration 
	{  //System.err.println("in Simulation");
		/*root.addActions(decl.get(0));
		root.addExceptions(decl.get(1));
       root.addVariables(decl.get(2));       
       root.addTypes(decl.get(3));
       root.addProcesses(decl.get(4)); */
		root.addModels(decl);
	} )*
;

declaration returns //[ArrayList<ArrayList<Model>> decl = new ArrayList<ArrayList<Model>>(5)]
[ArrayList<Model> decl = new ArrayList<Model>()]
//	{ System.err.println("in Declaration"); 
	{ //for (int i=0;i<5;i++) decl.add(new ArrayList<Model>());
	  ArrayList<Model> actions;
	  ArrayList<Model> excs;
	  ArrayList<Model> vars;
  	  ArrayList<Model> structs;
  	  ArrayList<Model> procs;
	}
: actions = actionDeclaration
	{ //decl.get(0).addAll(actions); 
		decl.addAll(actions);
		//System.err.println("Actions size: " + actions.size());
	}
| vars = variableDeclaration
	{ //decl.get(2).addAll(vars);
		decl.addAll(vars); 
	//System.err.println("Vars size: " + vars.size());
	}
| excs = exceptionDeclaration
	{ //decl.get(1).addAll(excs);
		decl.addAll(excs);
		//System.err.println("Excs size: " + excs.size());
	}
| structs = structDeclaration
	{ //decl.get(3).addAll(structs);
		decl.addAll(structs);
		//System.err.println("Structs size: " + structs.size());
	}
| procs = processDeclaration
	{ //decl.get(4).addAll(procs);
		decl.addAll(procs);
		//System.err.println("Procs size: " + procs.size());
	}
;

/*actionDeclaration returns [ArrayList<Model> acts = new ArrayList<Model>()]
	{ ArrayList<Vector> i; }
	//System.err.println("In ActionDeclaration");}
: "action" i=idList SEMI { for (int j=0; j < i.size(); j++)
						 acts.add(new ModestAction((String)i.get(j).get(0), (int)i.get(j).get(1), (int)i.get(j).get(2)));
						 //System.err.println("Name: " + i.get(j));
						 //System.err.println("Line: " + a.getLine());
						 }
| "patient" "action" i=idList SEMI { for (int j=0; j < i.size(); j++)
						 acts.add(new ModestAction((String)i.get(j).get(0), (int)i.get(j).get(1), (int)i.get(j).get(2), true));
						 //System.err.println("Name: " + i.get(j));
						 //System.err.println("Line: " + a1.getLine());
						 }
| "impatient" "action" i=idList SEMI { for (int j=0; j < i.size(); j++)
						 acts.add(new ModestAction((String)i.get(j).get(0), (int)i.get(j).get(1), (int)i.get(j).get(2), false));
						 //System.err.println("Name: " + i.get(j));
						 //System.err.println("Line: " + a2.getLine());
						 }						 
;*/

actionDeclaration returns [ArrayList<Model> acts = new ArrayList<Model>()]
: "action" i:ID {acts.add(new ModestAction(i.getText(), i.getLine(), i.getColumn()));
	/*System.err.println("action name: " + i.getText());
	System.err.println("action line: " + i.getLine());
	System.err.println("action column: " + i.getColumn()); */}
	(COMMA i2:ID {acts.add(new ModestAction(i2.getText(), i2.getLine(), i2.getColumn()));})* SEMI
| "patient" "action" i3:ID {acts.add(new ModestAction(i3.getText(), i3.getLine(), i3.getColumn()));}
	(COMMA i4:ID {acts.add(new ModestAction(i4.getText(), i4.getLine(), i4.getColumn(), true));})* SEMI
| "impatient" "action" i5:ID {acts.add(new ModestAction(i5.getText(), i5.getLine(), i5.getColumn()));}
	(COMMA i6:ID {acts.add(new ModestAction(i6.getText(), i6.getLine(), i6.getColumn(), false));})* SEMI
;

/*idList returns [ArrayList<Vector> s = new ArrayList<Vector>()]
: i:ID { Vector idVec = new Vector();
	idVec.add(i.getText());
	idVec.add(i.getLine());
	idVec.add(i.getColumn());
	s.add(idVec); } 
	(COMMA i2:ID { Vector idVec = new Vector();
	idVec.add(i2.getText());
	idVec.add(i2.getLine());
	idVec.add(i2.getColumn());
	s.add(idVec); })*
;*/

variableDeclaration returns [ArrayList<Model> vars = new ArrayList<Model>()]
{ //ArrayList<String> i = new ArrayList<String>(); 
	String q = ""; //int m = 0; 
	//ArrayList<String> a = new ArrayList<String>(); 
}
//	System.err.println("In VariableDeclaration");}
: (modifier)? tf:"float" myid:ID {vars.add(new ModestVariable(myid.getText(), myid.getLine(), myid.getColumn(), "float"));} (EQUAL FLOAT)? 
	(COMMA myid2:ID {vars.add(new ModestVariable(myid2.getText(), myid2.getLine(), myid2.getColumn(), "float"));} (EQUAL FLOAT)?)* SEMI 
	/*{ for (int j=0; j < i.size(); j++)
		vars.add(new ModestVariable(i.get(j), tf.getLine(),"float"));
	}*/
| (modifier)? ti:"int" myid3:ID {vars.add(new ModestVariable(myid3.getText(), myid3.getLine(), myid3.getColumn(), "int"));} (EQUAL INT)? 
	(COMMA myid4:ID {vars.add(new ModestVariable(myid4.getText(), myid4.getLine(), myid4.getColumn(), "int"));} (EQUAL INT)?)* SEMI 
	/*{ 	//System.err.println("In IntDeclaration");
		for (int j=0; j < i.size(); j++)
			vars.add(new ModestVariable(i.get(j), ti.getLine(),"int")); 
	}*/
| (modifier)? tb:"bool" myid5:ID {vars.add(new ModestVariable(myid5.getText(), myid5.getLine(), myid5.getColumn(), "bool"));} (EQUAL ("true"|"false"))? 
	(COMMA myid6:ID {vars.add(new ModestVariable(myid6.getText(), myid6.getLine(), myid6.getColumn(), "bool"));} (EQUAL ("true"|"false"))? )* SEMI 
	/*{ for (int j=0; j < i.size(); j++)
		vars.add(new ModestVariable(i.get(j), tb.getLine(),"bool")); 
	}*/
| (modifier)? tc:"clock" myid7:ID {vars.add(new ModestVariable(myid7.getText(), myid7.getLine(), myid7.getColumn(), "clock"));} (EQUAL INT)? 
	(COMMA myid8:ID {vars.add(new ModestVariable(myid8.getText(), myid8.getLine(), myid8.getColumn(), "clock"));} (EQUAL INT)?)* SEMI 
	/*{ for (int j=0; j < i.size(); j++)
		vars.add(new ModestVariable(i.get(j), tc.getLine(),"clock")); 
	}*/
| (modifier)? q=qType i:ID LBRACKET RBRACKET { vars.add(new ModestVariable(i.getText(), i.getLine(), i.getColumn(), q + "_array"));} 
	(COMMA i2:ID LBRACKET RBRACKET { vars.add(new ModestVariable(i2.getText(), i2.getLine(), i2.getColumn(), q + "_array")); } )* SEMI 
	/*{ for (int j=0; j < a.size(); j++)
		vars.add(new ModestVariable(a.get(j), m, q + "_array")); 
	}*/
;

// See if we need the modifier. Not used as of today.
modifier returns [int line = 0]
: c:"const" { line = c.getLine(); } | e:"extern" { line = e.getLine(); }
;

qType returns [String s = ""]
: "int" { s = "int"; }
| "float" { s = "float"; }
| "clock" { s = "clock"; }
| "action" { s = "action"; }
| i:ID { s = i.getText(); }
;

/*arrayDeclaration returns [ArrayList<Model> s = new ArrayList<Model>()]
//{ 	  System.err.println("In arrayDeclaration TreeWalker"); }
: i:ID LBRACKET RBRACKET { s.add(i.getText()); } (COMMA i2:ID LBRACKET RBRACKET { s.add(i2.getText()); } )*
;*/

exceptionDeclaration returns [ArrayList<Model> excs = new ArrayList<Model>()]
//	{ ArrayList<Vector> i; }
//	System.err.println("In ExceptionDeclaration");}
/*: e:"exception" i=idList SEMI { for (int j=0; j < i.size(); j++)
						 excs.add(new ModestException((Srring)i.get(j).get(0), (int)i.get(j).get(1), (int)i.get(j).get(2)));
						 //System.err.println("Name: " + i.get(j));
						 //System.err.println("Line: " + e.getLine());
						  }
;*/
: "exception" i:ID { excs.add(new ModestException(i.getText(), i.getLine(), i.getColumn())); }
	(COMMA i2:ID { excs.add(new ModestException(i2.getText(), i2.getLine(), i2.getColumn())); })* SEMI
;

structDeclaration returns [ArrayList<Model> structs = new ArrayList<Model>()]
//	{ System.err.println("In StructDeclaration"); }
: "typedef" ( "struct" LBRACE RBRACE i:ID SEMI 
	{ structs.add(new ModestType(i.getText(), i.getLine(), i.getColumn(), "struct"));
		 //System.err.println("Name: " + i.getText());
		 //System.err.println("Line: " + i.getLine());
	}
| "set" LBRACE RBRACE i2:ID SEMI
	{ structs.add(new ModestType(i2.getText(), i2.getLine(), i.getColumn(), "set"));
		 //System.err.println("Name: " + i2.getText());
		 //System.err.println("Line: " + i2.getLine());
	}
| "range" LBRACKET INT DOTDOT INT RBRACKET i3:ID SEMI
	{ structs.add(new ModestType(i3.getText(), i3.getLine(), i.getColumn(), "range"));
		 //System.err.println("Name: " + i3.getText());
		 //System.err.println("Line: " + i3.getLine());
	} )
;

processDeclaration returns [ArrayList<Model> procs = new ArrayList<Model>()]
	{ //System.err.println("In ProcessDeclaration"); 
	//ArrayList<ArrayList<Model>> b = new ArrayList<ArrayList<Model>>();
	ArrayList<Model> b = new ArrayList<Model>();
	}
: "process" i:ID LPAR RPAR LBRACE b=body RBRACE
	{ Process actProc = new Process(i.getText(),i.getLine(), i.getColumn());
		/*actProc.addActions(b.get(0)); 
		actProc.addExceptions(b.get(1));
		actProc.addVariables(b.get(2));
		actProc.addTypes(b.get(3));
		actProc.addProcesses(b.get(4));*/
		actProc.addModels(b);
		procs.add(actProc);
	}
;

body returns //[ArrayList<ArrayList<Model>> b = new ArrayList<ArrayList<Model>>()]
[ArrayList<Model> b = new ArrayList<Model>()]
	{ //ArrayList<ArrayList<Model>> decl = new ArrayList<ArrayList<Model>>(); 
		ArrayList<Model> decl = new ArrayList<Model>();
	//for (int i=0;i<5;i++) b.add(new ArrayList<Model>());
	}
: (decl = declaration 
	{
		/*b.get(0).addAll(decl.get(0));
		b.get(1).addAll(decl.get(1));
		b.get(2).addAll(decl.get(2));
		b.get(3).addAll(decl.get(3));
		b.get(4).addAll(decl.get(4));*/
		b.addAll(decl);
	})*
;