header {
package modesteditor.core.antlrparser;

import modesteditor.core.ModestEditor;
    
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream; 
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.jface.text.IDocument; 
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.Region;

import antlr.CommonAST;  
import antlr.RecognitionException;
}


// --------------------------------------------------------------------
// Lexer begins here!
// --------------------------------------------------------------------

/* TODOs:
- fill token types 
- setLine conversion from String to int
*/

class ModestLexerNew extends Lexer;
options {
    k=2;// In case k=1 doesn't suffice
    charVocabulary = '\u0000' .. '\u00FF';

    // Settings for literal matching
	caseSensitiveLiterals = false;	// case matters!
    testLiterals = false;
    
    exportVocab=lexerModestVocab_;
    
    //defaultErrorHandler=true;
    
}

tokens {
	/*DOT;
	DOTDOT;	
	INT;
	FLOAT;
	DIV;
	SHIFTR;
	SHIFTL;
	SR_ASSIGN;
	SL_ASSIGN;
	HASH;*/	
	
	
	TYPEDEF="typedef";
	CLASS="class";
	MCONST="const";
	MEXTERN="extern";
	RANGE="range";
	ENUM="enum";
	STRUCT="struct";
	
	// My addition
	SET="set";

	PATIENT="patient";
	IMPATIENT="impatient";

	STOP="STOP";
	ERROR="ERROR";

	FORALL="forall";

	ALT="alt";
	PALT="palt";
	WHEN="when";
	URGENT="urgent";

	DO="do";
	BREAK="break";

	IF="if";
	ELSE="else";
	RETURN="return";

	PAR="par";

	TRY="try";
	THROW="throw";
	CATCH="catch";

	PROCESS="process";

	RELABEL="relabel";
	BY="by";
	HIDE="hide";
	EXPOSE="expose";
	EXTEND="extend";
	REWRITE="rewrite";
	PREFIX="prefix";

	PACKAGE="package";
	IMPORT="import";
	EXPORT="export";

	PRIORITY="priority";
	DEFAULTS="defaults";

	BOR="|";
	AMPERSAND="&";
	EOR="^";
	SHIFTL="<<";
	SHIFTR=">>";
	NOT_OP="!";
	AND_OP="&&";
	OR_OP="||";
	EQ_OP="==";
	NE_OP="!=";
	LT_OP="<";
	GT_OP=">";
	LE_OP="<=";
	GE_OP=">=";

	PLUS="+";
	MINUS="-";
	MULT="*";
	DIV="/";
	MOD="%";

	LBRACE="(";
	RBRACE=")";
	LBRACKET="[";
	RBRACKET="]";
	LCURL="{";
	RCURL="}";
	ASSIGN="=";
	PLUS_ASSIGN="+=";
	MINUS_ASSIGN="-=";
	MULT_ASSIGN="*=";
	DIV_ASSIGN="/=";
	MOD_ASSIGN="%=";
	SR_ASSIGN=">>=";
	SL_ASSIGN="<<=";
	BAND_ASSIGN="&=";
	BXOR_ASSIGN="^=";
	BOR_ASSIGN="|=";
	ASSIGN_START="{=";
	ASSIGN_END="=}";
	DOTDOT="..";
	DOT=".";
	COMMA=",";
	QUESTION="?";
	DCOLON="::";
	COLON=":";
	SEMICOLON=";";
	AT="@";
	HASH="#";
	INT;
	FLOAT;
	STRING;
	IDENTIFIER;
		
}


// Define Methods for the Lexer
{
	/*public void uponEOF() throws EndOfFileException {
		EndOfFileException exc = new EndOfFileException();
		//throw exc;
	}*/	
	/** Sets Tab to one char as seen by Eclipse */
	public void tab() {
		int t = 1;
		int c = getColumn();
		int nc = (((c-1)/t)+1)*t+1;
		setColumn( nc );
	}
	
	
}


protected EXPONENT_PART:	( 'e' | 'E' ) ( '+' | '-' )? ('0'..'9')+ ;
protected FLOAT_SUFFIX: ('F'|'f'|'L'|'l') ;

DOT_OR_DOTDOT:
	".."	{ $setType(DOTDOT); }
|	'.'	{ $setType(DOT); }
;

NUMERIC:
	('0'..'9')+	{ $setType(INT); }
	(
	|	{ LA(2) >= '0' && LA(2) <= '9' }? '.' ('0'..'9')+ (EXPONENT_PART)? (FLOAT_SUFFIX)? { $setType(FLOAT); }
	|	EXPONENT_PART (FLOAT_SUFFIX)?  { $setType(FLOAT); }
	|	FLOAT_SUFFIX  { $setType(FLOAT); }
	)
;

IDENTIFIER options { testLiterals = true; paraphrase = "an identifier";}:
	( 'a'..'z' | 'A'..'Z' | '_' )
	( 'a'..'z' | 'A'..'Z' | '0'..'9' | '_' | '#' )*
;

/// skip whitespace
WS options {
  paraphrase = "a whitespace character";
} /* Rics version( ' ' | '\t' { tab(); } | '\n' { newline(); } | '\r' )+
	{
		// Skip whitespace...
		$setType(Token.SKIP);
	}
;*/
: ( ' ' | '\t' /*{ tab(); }*/ | '\f' // tab() seems to get called by consume so no need to call it again
    // handle newlines
    | ( "\r\n"  // DOS/Windows
      | '\r'    // Macintosh
      | '\n'    // Unix
      )
      // increment the line count in the scanner
      { newline(); }
    )
    { $setType(Token.SKIP); }
;

C_COMMENT options { paraphrase = "'C-comment'"; }
{
	int nstarts = 0;		// keep track of nesting level..
}:
	'/' 	{ $setType(DIV); }
	( '*' { $setType(Token.SKIP); nstarts++; }
		(  //options { greedy = false; }
	 		{
				if( LA(1) == EOF || LA(2) == EOF )
					throw new RecognitionException(
									 "Unclosed comment",
									 getFilename(),
									 inputState.getTokenStartLine(),
									 inputState.getTokenStartColumn() );
			}:
	 		{ (nstarts > 1) || (LA(2) != '/') }? '*'
	 		{
				if(LA(1) == '/')
					nstarts--;
			}
		| '\n' { newline(); }
		| '\r' { newline(); }
		| '\r' '\n' { newline(); }
		| '\t' //{ tab(); }
		| "/*" { nstarts++; }
		| ~('*'|'\n'|'\r'|'\t')
		)*
	"*/"
	)?
;

CPP_COMMENT: "//" (~'\n')* ( '\n' { newline(); } )?
	{ $setType(Token.SKIP); }
;

/// Rule for strings with simple escapes
STRING: '"' ( ESCAPE | ~('"'|'\\') )* '"';
protected ESCAPE: '\\' ( 'n' { $setText("\n"); }
							  | 'r' { $setText("\r"); }
							  | 't' { $setText("\t"); }
							  | '"' { $setText("\""); }
							  )
;




NOT_OP:		"!";
QUESTION:	"?";
AND_OP:		"&&";
OR_OP:		"||";
EQ_OP:		"==";
NE_OP:		"!=";
LT_OP:		"<";
GT_OP:		">";
LE_OP:		"<=";
GE_OP:		">=";
PLUS:			"+";
MINUS:		"-";
MULT:			"*";
AMPERSAND:	"&";
BOR:			"|";
EOR:			"^";
MOD:			"%";

SHIFTR_OR_ASGN:	">>" { $setType(SHIFTR); } ( "=" { $setType(SR_ASSIGN); } )?;
SHIFTL_OR_ASGN:	"<<" { $setType(SHIFTL); } ( "=" { $setType(SL_ASSIGN); } )?;

COMMA options { paraphrase = "','"; }:			",";
ASSIGN options { paraphrase = "'='"; }:			"=";
PLUS_ASSIGN:	"+=";
MINUS_ASSIGN:	"-=";
MULT_ASSIGN:	"*=";
DIV_ASSIGN:		"/=";
MOD_ASSIGN:		"%=";
BAND_ASSIGN:	"&=";
BXOR_ASSIGN:	"^=";
BOR_ASSIGN:		"|=";
ASSIGN_START:	"{=";
ASSIGN_END:		"=}";
LBRACE options { paraphrase = "'('"; }:			"(";
RBRACE options { paraphrase = "')'"; }:			")";
LCURL options { paraphrase = "'{'"; }:			"{";
RCURL options { paraphrase = "'}'"; }:			"}";
LBRACKET options { paraphrase = "'['"; }:		"[";
RBRACKET options { paraphrase = "']'"; }:		"]";
DCOLON:			"::";
COLON options { paraphrase = "':'"; }:			":";
SEMICOLON options { paraphrase = "';'"; }:		";";
AT:				"@";

INTNUMERIC: ('0'..'9')+ ;

HASH_OR_LINENR :		'#' 	{ $setType(HASH); }
	( "line" 
          (' '|'\t')+  i:INTNUMERIC { setLine((new Integer(i.getText()))-1);   }
          (' '|'\t')+ '"' ( ( ~('"'|'\\') )* {setFilename($getText);} ) '"' 
        { $setType(Token.SKIP);  } )? 
;        

// --------------------------------------------------------------------
// Parser begins here!
// --------------------------------------------------------------------

class ModestParserNew extends Parser;
options {
	buildAST=true;
	defaultErrorHandler = true;      // Don't generate parser error handlers
	//defaultErrorHandler = false;
	k=3;
	ASTLabelType="ModestAST";
	importVocab= lexerModestVocab_;
	exportVocab = parserModestVocab_;		// and export the lexer's and ours as this vocab
}
tokens { 
	// Imaginary tokens used for AST construction
	IDENTIFIER_AST;				///< preprocessed identifier reference
  	TYPE_AST;						///< preprocess type reference
	VARIABLE_DECLARATION_AST;
	FUNCTION_DEFINITION_AST;
	PARAMETER_LIST_AST;			///< tag for parameterlists
	PROCESS_DEFINITION_AST;
	CATCH_DEFINITION_AST;
	TYPE_DEFINITION_AST;
	FUNCTION_CALL_AST;
	PROCESS_INSTANTIATION_AST;
	WEIGHT_AST;						///< weights on palts
	GUARD_AST;						///< when or urgent
	TYPE_CAST;						///< a type cast tagged with the type
		
}

{
	// Define some methods and variables to use in the generated parser.

  // Define a main
  public static void main(String[] args) {
    // Use a try/catch block for parser exceptions
    try {
      // if we have at least one command-line argument
      if (args.length > 0 ) {
        //System.err.println("Parsing...");

        // for each directory/file specified on the command line
        for(int i=0; i< args.length;i++){
        	//System.err.println("Doing file: " + args[i]);
        	doFile(new File(args[i])); // parse it
        	
          	}
      }
      else
        System.err.println("Usage: java ModestParser <directory name>");

    }
    catch(Exception e) {
      System.err.println("exception: "+e);
      e.printStackTrace(System.err);   // so we can get stack trace
    }
  }


  // This method decides what action to take based on the type of
  //   file we are looking at
  public static void doFile(File f) throws Exception {
    // If this is a directory, walk each file/dir in that directory
    if (f.isDirectory()) {
      String files[] = f.list();
      for(int i=0; i < files.length; i++)
        doFile(new File(f, files[i]));
    }

    // otherwise, if this is a MoDeST file, parse it!
    else {
      //System.err.println("-------------------------------------------");
      //System.err.println(f.getAbsolutePath());
      parseFile(new FileInputStream(f));
    }
  }

  // Here's where we do the real work...
  public static void parseFile(InputStream s) throws Exception {
    try {
      // Create a scanner that reads from the input stream passed to us
      ModestLexerNew lexer = new ModestLexerNew(s);

      // Create a parser that reads from the scanner
      ModestParserNew parser = new ModestParserNew(lexer);

      // start parsing at the compilationUnit rule
      parser.start();
      System.out.println(parser.getAST().toStringList());
    }
    catch (EndOfFileException e) {
    	//System.err.println("Finished parsing!");
    	
    }
    catch (Exception e) {
      System.err.println("parser exception: "+e);
      e.printStackTrace();   // so we can get stack trace		
    }
  }

	protected long numTryCatch = 0;
	protected int numErrors = 0;
 	protected int procLevel = 0;
	protected boolean defActionsPatient;
	protected boolean defGuardsUrgent;
	protected static final int MOD_NONE = 0;
	protected static final int MOD_EXTERN = 1;
	protected static final int MOD_CONST = 2;
	protected static final int MOD_PATIENT = 4;
	protected static final int MOD_IMPATIENT = 8;
	private String[] builtInTypes =  { "type", // used to define types
		"int",
		"bool",
		"action",
		"clock",
		"float",
		"exception",
		"process",
		"set",
		"struct" };
	protected ArrayList<String> types = new ArrayList<String>();
	protected ArrayList<Symbol> vars = new ArrayList<Symbol>();
	protected ArrayList<Symbol> excs = new ArrayList<Symbol>();
	protected ArrayList<String> processes = new ArrayList<String>();
	protected HashMap<String, Symbol> unUsedVars = new HashMap<String, Symbol>();
	
	public HashMap<String, Symbol> getUnUsedVars(){
		return unUsedVars;
	}
	
	private void fillTypes() {
		for (int i = 0; i < builtInTypes.length; i++) {
			types.add(builtInTypes[i]);
		}
	}
	
	public ArrayList<String> getVars() {
		ArrayList<String> variables = new ArrayList<String>();
		for (int i = 0; i < vars.size(); i++) {
			variables.add(vars.get(i).getName());
		}
		for (int i = 0; i < excs.size(); i++) {
			variables.add(excs.get(i).getName());
		}	
		return variables;
	}
	
	public ArrayList<String> getProcesses() {
		return processes;
	}
	
	public int getErrorCount() {
			return numErrors;
	}
	
	public String findType(String s) {
		if (types.contains(s)) 
			return s;
		else return null;
	}
	
	public boolean addType(String s) {
		if (!types.contains(s))
			return types.add(s);
		else return false;
	}
	
	public ArrayList<IRegion> problems;
	
	/*public void setActualDocumentOffset(int offset){
	  actualDocOffset=offset;
  	}*/
  
  public void reportError(RecognitionException ex){
	
	try {
		ModestEditor editor = ModestEditor.getEditor();
		IDocument document = editor.getDocumentProvider().getDocument(editor.getEditorInput());
		
		/*int actualLine = document.getLineOfOffset(actualDocOffset);
		System.err.println("Syntax error at line " + (ex.getLine()+actualLine) + ": " + ex.getMessage());
		
		
		int lineOffset = document.getLineOffset(ex.getLine()-1+actualLine);
		int nextLineOffset = document.getLineOffset(ex.getLine()+actualLine);*/
		
		int lineOffset = document.getLineOffset(ex.getLine()-1);
		int nextLineOffset = document.getLineOffset(ex.getLine());
		
		problems.add(new Region(lineOffset,nextLineOffset-lineOffset-1));
	} catch (BadLocationException e) {
		System.err.println("Bad line offset for the erroneous element.");
	}
  }
	
	/** Utility to gerenerate a piece of parse tree with an IDENTIFIER_AST
 * referencing a tau.
 * @todo make the filename point to something sensible
 */
ModestAST create_tau_ast( int line, int column ) 
{ 
	ModestAST tau = /*ModestAST*/(#[IDENTIFIER,"tau"]); // maybe new
	tau.setLine(line);
	tau.setColumn(column);
	//tau->setSymbol(SymData->get_tau_symbol());
	tau.setSymbol(new Symbol("tau", "action"));

	return tau;
}
	
}

/** Start rule for grammar */
start //{ Symbol_process* s; } 
:
{
	astFactory.setASTNodeClass(ModestAST.class);
	fillTypes();
	problems = new ArrayList<IRegion>();
	Symbol tauAction = new Symbol("tau", "action");
	vars.add(tauAction);
	Symbol trueBoolean = new Symbol("true", "bool");
	vars.add(trueBoolean);
	Symbol falseBoolean = new Symbol("false", "bool");
	vars.add(falseBoolean);
	//System.err.println("Getting started");
}
	( defaults_section )?		// defaults for the parsed file..
	/*{
		// Directly add a root process symbol to the symbol table..
		// it allows us to treat some things more generically
		s = new Symbol_process(Symbol::bi_root_process,
									  SymData->get_builtin_type(SymbolInformation::TP_PROCESS).info,
									  this->getFilename().c_str(),
									  1, 1,
									  SymData->get_current_scope(),
									  SymData->get_current_namespace());
		s->inc_refcount();
		try {
			SymData->add(s);
		}
		catch ( SymbolTableException& ex )
		{
			throw antlr::SemanticException(ex.get_message(),
													 this->getFilename(),
													 ex.get_line(), ex.get_column() );
		}
		SymData->get_current_scope()->add_symbol(s);
	}*/
	d:description//[ s ]					// 'normal' description
	EOF!
;

/** Rule to take care of defaults section
 */
defaults_section
//{System.err.println("In defaults_section");}
:
	"defaults"^ LCURL!
	// label the subrule so that the ^ operator operates locally in the subrule
	( setting:IDENTIFIER ASSIGN^ value:IDENTIFIER SEMICOLON!
 	{ /*{{{*/
		if( setting.getText() == "actions" )
		{
			if( value.getText() == "patient" )
				defActionsPatient = true;
			else if( value.getText() == "impatient" )
				defActionsPatient = false;
			else
			{
				
				String error = "Illegal value specified for default '" + setting.getText() + "'";
				System.err.println(error);
				throw new SemanticException(error, this.getFilename(),
														 setting.getLine(), setting.getColumn());
			}
		}
		else if( setting.getText() == "guards" )
		{
			if( value.getText() == "urgent" )
				defGuardsUrgent = true;
			else if( value.getText() == "not_urgent" )
				defGuardsUrgent = false;
			else
			{
				String error = "Illegal value specified for default '" + setting.getText() + "'";
				System.err.println(error);
				throw new SemanticException(error, this.getFilename(),
														 setting.getLine(), setting.getColumn());
			}
		}
		else
		{
			String error = "Unknown default: '" + setting.getText() + "'";
			System.err.println(error);
			throw new SemanticException(error, this.getFilename(),
													 setting.getLine(), setting.getColumn());
		}
		/*}}}*/
	}
	)+ RCURL!
;

/** Grammar rule for descriptions
 * @todo rework description rule
 * - localprocess should be different for functions and processes
 * @param proc the process symbol the description belongs to.
 */
description /*! [Symbol_process* proc] {
	ASTVector declarations;
	ASTVector statements;
	ASTVector assignments;
} */
//{ System.err.println("In description");}
:
	( decl:declarative_stmnt //[assignments]	{ declarations.push_back(#decl); }
	)*
	stmnt:sequential_processes //				{ statements.push_back(#stmnt); }
/*{
	// output the declarations first in the AST then the statements
	ASTVector::const_iterator
		i = declarations.begin(),
		end = declarations.end();

	while( i != end )
	{
		antlr::RefAST t = antlr::RefAST(*i);
		## = #( antlr::nullAST, ##, t );
		++i;
	}

	// if there were assignments on initialization we generate an
	// additional palt with a assignment block.
	if( ! assignments.empty() )
	{
		// Build a palt with leading tau with the initializations
		RefModest_AST tau = create_tau_ast(0,0);
		RefModest_AST palt = RefModest_AST(#[PALT,"palt"]);
		tau->addChild(palt);
		RefModest_AST weight = RefModest_AST(#[WEIGHT_AST,"::"]);
		weight->setWeight(1);
		palt->addChild(weight);
		RefModest_AST assign = RefModest_AST(#[ASSIGN_START,"{="]);
		assign->setScope(SymData->get_current_scope());
		weight->addChild(assign);

		i = assignments.begin();
		end = assignments.end();

		while( i != end )
		{
			assign->addChild(*i);
			++i;
		}
		// glue it together...
		## = #( antlr::nullAST, ##, tau );
	}

	end = statements.end();
	i = statements.begin();

	while( i != end )
	{
		antlr::RefAST t = antlr::RefAST(*i);
		## = #( antlr::nullAST, ##, t );
		++i;
	}

	if( proc )
		proc->setAST( ## );

//	cout << "Description tree:" << endl;
//	print_tree<RefModest_AST,Modest_Parser> pr(*this);
//	pr.pr_tree(##);
}*/
;

/** Declarations of types/processes/functions/variables
 * @param assignments ASTVector used to collect initializations of variables
 */
declarative_stmnt//[ASTVector &assignments]
//{ System.err.println("in declarative_stmnt");}
:
	type_definition												// typedef ... ;
|	process_definition											// process proc(..) { }
/*|  ((modifier)? qualified_type IDENTIFIER param_definition[0] LCURL)
 		=> function_definition 									// int func(...) { }*/
|  ((modifier)? qualified_type )
 		=> variable_declaration[false] //[assignments]				// int x = ..;
;

/** Modifiers
 * used in variable declarations to specify externness, constness
 * and (im)patience.
 * @note no tree built for this rule.
 */
modifier returns [int mask = 0] 
:
	(	"extern" 		{ mask =(mask+1)%2; }//|= MOD_EXTERN; }
	|	"const" 		{ mask =(mask+2)%4; }//{ mask |= MOD_CONST; }
	|	"patient"		{ mask =(mask+4)%8; }//{ mask |= MOD_PATIENT; }
	|	"impatient" 	{ mask =(mask+8)%16; }//{ mask |= MOD_IMPATIENT; }
	)+
;

/** Qualified type
 * type with optional namespace qualification
 * @note The default errorhandler is turned of so that this rule behaves
 * as if it's one token. This prevents it returning type = 0 without throwing
 * an error (e.g. the erorr is handled inside this rule and a zero type is
 * returned and that's not quite what we want... no?)
 */
qualified_type returns [String type]//	returns [Symbol_type* type]
//	options { defaultErrorHandler = false; }
{ type = findType(LT(1).getText()); }
://!
	{ type!=null }? tp:IDENTIFIER
	{
		if( type == null )
		{
			String error = "invalid type '" + tp.getText() + "'";
			System.err.println(error);
			throw new SemanticException(error, this.getFilename(),
														 tp.getLine(), tp.getColumn());
		}
		// increase reference count..
		//type->inc_refcount();
		/* condense the complete AST for this rule into one handy imaginary AST node
		 * with all the info needed.
		 */
		## = #([TYPE_AST, type]);
		//##->setSymbolType(type);
		##.setLine(tp.getLine());
		##.setColumn(tp.getColumn());
	}
;


/** Type definition
 * define a new type
 * - a set (mostly implemented)
 * - a struct (mostly unimplemented)
 * - a range (mostly unimplemented)
 */
type_definition !
{
	//System.err.println("in type_definition");
	/*const Symbol_type *range_type = 0;
	ASTVector asgn;
	Symbol_struct* struct_sym; // we need this one early*/
	String rangeType = "";
	Symbol struct_sym;
	String typeKind = "";
}:
	td:"typedef"
	(	set_keyword:"set"/*IDENTIFIER*/ set:set { typeKind = "set"; }
	|  range_keyword:"range" rangeType=r:range { typeKind = "range"; }
	|	struct_keyword:"struct" { typeKind = "struct"; }
		{
			//the new symbol
			/*struct_sym =
			new Symbol( this.getFilename(),
										 struct_keyword.getLine(),
										 struct_keyword.getColumn());*/
				/*new Symbol_struct( this->getFilename().c_str(),
										 struct_keyword->getLine(),
										 struct_keyword->getColumn(),
										 SymData,
										 SymData->get_current_scope(),
										 SymData->get_current_namespace()
										 );*/
			// make the new scope introduced by this struct.
			// All new variables go in here.
			// this scope is owned by the struct
			//SymData->set_current_scope(SymData->get_current_scope()->add_struct(struct_sym));
		}
		s:astruct// [asgn]
		/*{
			if( !asgn.empty() )
			{
				//do not allow any assignments inside structure typedefinitions
				std::ostringstream error;
				error << "Initializations inside structures not supported";
				throw antlr::SemanticException(error.str(), this->getFilename(),
														 struct_keyword->getLine(), struct_keyword->getLine());
			}
		}*/
	)
	type_name:IDENTIFIER
	";"
	{ // Make symbol table entry, precalc some stuff & add TYPE_DEFINITION_AST as top 
		//Symbol_type *tp = 0;
		Symbol tp = null;
		/*if ( struct_keyword != null  )
		{	
			// the struct now knows which scope it owns and its name
			//struct_sym->setInfo(type_name->getText(),SymData->get_current_scope());
			// return to the original namespace
			//SymData->set_current_scope(SymData->get_current_scope()->get_parent());

			tp = struct_sym;
			
		}
		else if ( set_keyword != null )					// dealing with set definition?
		{ 
			if( set_keyword.getText() != "set" )
			{
				//std::ostringstream error;
				String error = "expecting set got '" + set_keyword.getText() + "'";
				throw new SemanticException(error, this.getFilename(),
									 set_keyword.getLine(), set_keyword.getLine());
			}
			//Symbol_symset *set_sym =
			Symbol set_sym =
				new Symbol(type_name.getText(),
										this.getFilename(),
										type_name.getLine(), type_name.getColumn());
										
				new Symbol_symset(type_name->getText(),
										this->getFilename().c_str(),
										type_name->getLine(), type_name->getColumn(),
										SymData,
										SymData->get_current_scope(),
										SymData->get_current_namespace() );
//			print_tree<RefModest_AST,Modest_Parser> pr(*this);
//			pr.pr_tree(#set);

			CalcSet walk( SymData, set_sym, false );
			walk.tree(#set);

			if( walk.OK() )
				tp = set_sym;
			else
			{
				std::ostringstream error;
				error << "set specification invalid in definiton of type '"
					<< type_name->getText() << "' (" << walk.error() << ")";
				throw antlr::SemanticException(error.str(), this->getFilename(),
														 type_name->getLine(), type_name->getColumn());
			}
			
			//tp = set_sym;
			
		}
		else*/ if( range_keyword != null)		// dealing with range definition?
		{ 
			if( rangeType != "int" )
			{
				//std::ostringstream error;
				String error = "range types of non-integers are not supported";
				throw new SemanticException(error, this.getFilename(),
														 range_keyword.getLine(), range_keyword.getLine());
			}
			// dissect range..
			/*CalcStaticRange walk( SymData, false );
			walk.tree(#r);

			if( ! walk.OK() )
			{
//				print_tree<RefModest_AST,Modest_Parser> pr(*this);
//				pr.pr_tree(#set);
				std::ostringstream error;
				error
					<< "definition of range in typedef is incorrect ("
					<< walk.error() << ")";
				throw antlr::SemanticException(error.str(), this->getFilename(),
														 range_keyword->getLine(),range_keyword->getColumn());
			}
			const Bound& range = walk.get_bounds();*/

			// things seem ok.. now make symbol...
			//Symbol_range *range_sym =
			/*Symbol range_sym = 
				new Symbol(type_name.getText(),type_name.getLine(), type_name.getColumn());*/
				/*new Symbol_range(type_name->getText(),
									  range,
									  range_type,
									  this->getFilename().c_str(),
									  type_name->getLine(), type_name->getColumn(),
									  SymData->get_current_scope(),
									  SymData->get_current_namespace() );*/
			//tp = range_sym;
			
		}

		/*if( tp == null )
		{
			//std::ostringstream error;
			String error = "could not define type '" + type_name.getText() + "'";
			throw SemanticException(error, this.getFilename(),
														 td.getLine(), td.getColumn());
		}*/
		//add_type( tp );
		addType(type_name.getText());

		#type_definition = #([TYPE_DEFINITION_AST, "typedef"], #type_definition);
		//#type_definition.setSymbolType(tp);
		#type_definition.setSymbolType(type_name.getText());
		#type_definition.setLine(type_name.getLine());
		#type_definition.setColumn(type_name.getColumn());
		#type_definition.setTypeKind(typeKind);
		
	}
;


vardecl_assign ! [int modifiers, /*Symbol *sym,*/ ModestAST id]
{
	//const Symbol_type* expr_type = 0;
	String exprType = "";
}
:
	asgn:"=" exprType=expr:expression
	/*{ //{{{ Sort base_expr out and if possible initialize stuff 
//		print_tree<RefModest_AST, Modest_Parser> pr(*this);
//		pr.pr_tree(#expr);

		#asgn->setScope( SymData->get_current_scope() );

		if( sym == 0 )	// sanity check..
		{
			throw antlr::SemanticException("cannot define symbol as constant",
													 this->getFilename(),
													 id ? id->getLine() : asgn->getLine(),
													 id ? id->getColumn() : asgn->getColumn() );
		}

		if( (modifiers & MOD_CONST) || (modifiers & MOD_EXTERN) )
		{
			if ( SymData->is_int(sym) || SymData->is_float(sym) )
			{ 
				Symbol_const *sc = dynamic_cast<Symbol_const*>(sym);

				if( ! sc )
				{
					std::ostringstream error;
					error << "cannot define symbol " << sym->get_name()
						<< " as constant";

					throw antlr::SemanticException(error.str(),
															 this->getFilename(),
															 sym->get_line(),
															 sym->get_column());
				}

//				print_tree<RefModest_AST, Modest_Parser> pr(*this);
//				pr.pr_tree(#expr);

				CalcStaticExpr walk( SymData, false );
				Value v = walk(#expr);

				if( v.is_float() && SymData->is_int(sym) )
				{
					std::ostringstream error;
					error << "expression in const definition of "
						<< sc->get_name() << " not valid can't assign float to int";
					throw antlr::SemanticException(error.str(), this->getFilename(),
															 sc->get_line(),
															 sc->get_column()
															 );
				}

				if( ! walk.OK() )
				{
					std::ostringstream error;
					error << "expression in const definition of "
						<< sc->get_name() << " not valid ("
						<< walk.error() << ")";
					throw antlr::SemanticException(error.str(), this->getFilename(),
															 sc->get_line(),
															 sc->get_column()
															 );
				}
				sc->setValue(v);
//				cout << "const set " << sym->get_name() << " "
//					<< walk.get_value() << std::endl;
				
			}
			else if ( SymData->is_set(sym) )
			{ 
				Symbol_symset *s = dynamic_cast<Symbol_symset*>(sym);

				if( ! s )
				{
					std::ostringstream error;
					error << "cannot define set " << sym->get_name();
					throw antlr::SemanticException(error.str(),
															 this->getFilename(),
															 sym->get_line(),
															 sym->get_column());
				}

				// print_tree<RefModest_AST, Modest_Parser> pr(*this);
				// pr.pr_tree(#expr);
				// add all the elements in #expr to the sym_set symbol we got
				// passed.
				CalcSet walk( SymData, s, false );
				walk.tree(#expr);
				
			}
			else
			{
				std::ostringstream error;
				error << "expression in const definition of "
					<< sym->get_name() << " unimplemented (invalid?) type";
				throw antlr::SemanticException(error.str(), this->getFilename(),
														 sym->get_line(),
														 sym->get_column() );
			}
			// in the case of a constant we don't return an assignment tree
			#vardecl_assign = antlr::nullAST;
		}
		else
		{
			if( ! #expr || !types_compatible_for_assignment( sym->get_type(), expr_type ) )
			{
				std::ostringstream error;
				error << "cannot initialize "
					<< sym->get_name() << " with given expression (invalid type?)";
				throw antlr::SemanticException(error.str(), this->getFilename(),
														 sym->get_line(),
														 sym->get_column() );
			}
			if( SymData->is_clock(sym) )
				check_clock_assignment( sym, #expr );
			// make a copy of id.. use clone to make a deep copy.
			RefModest_AST temp(id->clone());

			// temp->setSymbol(sym);
			// initializer -> sym is used
			sym->inc_refcount();

			// temp->setLine(id->getLine());
			// temp->setColumn(id->getColumn());

			## = #([ASSIGN,"="], temp, expr);
			##->setScope( SymData->get_current_scope() );
			##->setLine(asgn->getLine());
			##->setColumn(asgn->getColumn());
		}
//		std::cout << "Vardeclassign: " << std::endl;
//		print_tree<RefModest_AST, Modest_Parser> pr(*this);
//		pr.pr_tree(##);
		
	}*/
;

/** Variable declarations
 * define a variable for later use.
 * @todo verify that tree is built as declaration and split initialization
 *       out to separate assignment..
 * NOTE: a lot of the expressive power of previous versions was curbed
 */
variable_declaration! [boolean inStruct]//[ASTVector& assignments]
{
	int modifiers = 0;
	//Symbol_type *tp;
	Symbol sym;
	String tp;
}:
	( modifiers=modifier )?
	tp = type:qualified_type { ## = #type; }
	(id1:identifier_or_array_decl[tp,modifiers,inStruct]	//{ ##->addChild(#id1); }
		{ 
			##.addChild(#id1);
	      	//vars.add(new Symbol(id1.getText(), tp));//type.getText()); 
	      	//System.err.println("vars size: " + vars.size()+ " added: " + id1.getText());
	    }
		(! asn1:vardecl_assign[modifiers/*,sym*/,#id1]
			/*{
				// collect assignments for later...
				// fix de tree construction hier zie onder bij test.obj..
				if( #asn1 )
					assignments.push_back(#asn1);
			}*/
		)?	// optional initializer 
	)
	( COMMA!
		(/*sym =*/ id2:identifier_or_array_decl[tp,modifiers,inStruct]	//{ ##->addChild(#id2); }
		{ 
			##.addChild(#id2);
	      	//vars.add(new Symbol(id2.getText(), tp));
	      	//System.err.println("vars size: " + vars.size()+ " added: " + id2.getText()); 
	    }
		(! asn2:vardecl_assign[modifiers,/*sym,*/#id2]
			/*{ //{{{ collect assignments for later... 
				if( #asn2 )
					assignments.push_back(#asn2);
				//}}}
			}*/
		)? // optional initializer 
		)
	) *
	SEMICOLON!
	{ //{{{ Add top node and generate palt's for initializations 
		// add a VARIABLE_DECLARATION_AST node as top...
		## = #([VARIABLE_DECLARATION_AST,"var_decl"], #variable_declaration);
		##.setLine(#type.getLine());
		##.setColumn(#type.getColumn());
		//}}}
	}
;


/* ****************************************************************
* Parser Expressions start here!
******************************************************************/

/** sequential composition or action prefixing
 */
sequential_processes:
	local_process ( SEMICOLON! local_process ) *
;

/** Local process
 * work horse practically any statement.
 */
local_process //{ const Symbol_type *type = 0; }
//{ System.err.println("in local process"); }
:
	(/*	error:"ERROR"    					// process barfs
		{ #error->setSymbol(SymData->get_error_symbol()); }
	|*/  stop:"STOP"    					// process locks up
		//{ #stop->setSymbol(SymData->get_stop_symbol()); }
	|  brk:"break"
		//{ #brk->setSymbol(SymData->get_break_symbol()); }
	/*|  r:"return"^ type=e:expression	// function return
		{
			check_return_statement( r, #e, type );
		}
	*/|  t:"throw"^ id:IDENTIFIER		// exception
		{
			Symbol sym = null;
			for (Iterator iter = vars.iterator(); iter.hasNext();)
			{
				Symbol var = (Symbol)iter.next();
				if (id.getText().equals(var.getName()))
					sym = var;
			}
			for (Iterator iter = excs.iterator(); iter.hasNext();)
			{
				Symbol exc = (Symbol)iter.next();
				if (id.getText().equals(exc.getName()))
					sym = exc;
				
			}
			if( sym != null )
			{
				if( ! excs.contains(sym) )
				{
					//std::ostringstream error;
					String error = "can only throw exception typed identifiers";
					throw new SemanticException(error,
															 this.getFilename(),
															 #id.getLine(), #id.getColumn());
				}
				//#id->setSymbol(sym);
				//#t->setSymbol(sym);
				if (unUsedVars.containsValue(sym))
					unUsedVars.remove(sym.getName());
			}
			else
			{
				//std::ostringstream error;
				String error = "undefined symbol '" + id.getText() + "'";
				throw new SemanticException(error, this.getFilename(),
														 id.getLine(), id.getColumn());
			}
		}
	|  try_block						// (5) try ..throw.. catch ...
	|  if_stmnt							// if ...
	|  alt_stmnt						// alt ...
	|  loop_stmnt						// do ...
	|  parallel_composition			// par ... (10)
	|  label_modification			// hide ... etc..
	/*|  forall_stmnt					// forall ...
	*/|  guarded_process				// when ( ... ) ...
	|  process_body					// { ... }
	|  (assignment[true]) => assignment[true] // assignment (for in functions, in process rewritten to palt)
	|	(parameterized_object) => parameterized_object
	|  perform_action
	)
;

perform_action! 
{
	//const Symbol_type* tp = 0;
	String tp = null;
	//System.err.println("in perform_action");
}
:
	p1:palt_stmnt
	{
// This one triggers a bug in antlr's codegen.
//		#perform_action = #( create_tau_ast(#p1->getLine(),#p1->getColumn()), #p1 );
		// reportWarning(#p1, "inserting synthetic 'tau' for palt without leading action");
		ModestAST tau = create_tau_ast(#p1.getLine(),#p1.getColumn());
		## = #( tau, #p1 );
	}
|	(qualified_identifier[false] "palt") => tp=qi1:qualified_identifier[false] p2:palt_stmnt
	{
		/*if( !tp || !SymData->is_action(tp) )
		{
			std::ostringstream error;
			error << "identifier leading up to palt must be of type 'action'";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 #qi1->getLine(), #qi1->getColumn());
		}*/
		## = #( #qi1, #p2 );
	}
|	asgn1:assignment_block
	{
		ModestAST tau = create_tau_ast(#asgn1.getLine(),#asgn1.getColumn());
		ModestAST weight = (#[WEIGHT_AST,"::"]);
		weight.setWeight((double)1);

		## = #( tau, #(#[PALT,"palt"], #(weight, #asgn1 ) ) );
	}
|	(qualified_identifier[false] ASSIGN_START) => tp=qi2:qualified_identifier[false] asgn2:assignment_block
	{
		/*if( !tp || !SymData->is_action(tp) )
		{
			std::ostringstream error;
			error << "identifier leading up to palt must be of type 'action'";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 #qi2->getLine(), #qi2->getColumn());
		}*/
		ModestAST weight = (#[WEIGHT_AST,"::"]);
		weight.setWeight(1);

		## = #( #qi2, #(#[PALT,"palt"], #(weight, #asgn2 ) ) );
	}
|  tp=qi3:qualified_identifier[false]
	{
		if(!tp.equals("action"))//( !tp || !SymData->is_action(tp) )
		{
			String error = "Trying to use non action as action";
			System.err.println(error);
			throw new SemanticException(error, this.getFilename(),
										#qi3.getLine(), #qi3.getColumn());
		}
		## = #qi3;
	}
;

/** Try .. throw .. catch ..
 * exception handling
 * @todo check type checking on exceptions
 */
try_block //{	Symbol *sym = 0; }
:
	"try"^ process_body
	(! catcher:"catch" "("! id:IDENTIFIER ")"!
	{
		Symbol sym = null;
		for (Iterator iter = vars.iterator(); iter.hasNext();)
		{
			Symbol var = (Symbol)iter.next();
			if (id.getText().equals(var.getName()))
				sym = var;
		}
		for (Iterator iter = excs.iterator(); iter.hasNext();)
		{
			Symbol exc = (Symbol)iter.next();
			if (id.getText().equals(exc.getName()))
				sym = exc;
			
		}
		if( sym != null )
		{
			if( ! excs.contains(sym) )
			{
				//std::ostringstream error;
				String error = "can only throw exception typed identifiers";
				throw new SemanticException(error,
														 this.getFilename(),
														 #id.getLine(), #id.getColumn());
			}
			//#id->setSymbol(sym);
			//#t->setSymbol(sym);
		}
		else
		{
			//std::ostringstream error;
			String error = "undefined symbol '" + id.getText() + "'";
			throw new SemanticException(error, this.getFilename(),
													 id.getLine(), id.getColumn());
		}
	}
	pbody:process_body
	{
		// build AST
		//RefModest_AST cdef 
		ModestAST cdef = #([CATCH_DEFINITION_AST, "catch_def"], #id, #pbody );
		//cdef->setSymbol(sym);
		cdef.setLine(#id.getLine());
		cdef.setColumn(#id.getColumn());
		// and add it as a child..
		##.addChild(cdef);
	}
	)+
;

/** Labeling modifiers
 */
label_modification {
	/*RefModest_AST key;
	Relabeling::relabel_type type;
	const char* what;
	const Symbol_type* dummy;*/
	ModestAST key;
	String what;
	String dummy = "";
	
	//System.err.println("in label_modification"); 
} :
(
	h:"hide"^    { key = #h; /*type = Relabeling::HIDE;*/ what = "hide"; }
 		dummy=simple_expression
|	r:"relabel"^ { key = #r; /*type = Relabeling::RELABEL;*/ what = "relabel"; }
 		labels_by_labels //( ","! labels_by_labels ) *
|	e:"extend"^  { key = #e; /*type = Relabeling::EXTEND;*/ what = "extend"; }
 		dummy=expr1:simple_expression
/*|	p:"prefix"^  { key = #p; type = Relabeling::PREFIX; what = "prefix"; }
 		dummy=expr2:simple_expression*/
)
/*{
	CalcRelabeling relabeling( SymData, type, false );

	if( #expr1 )
	{
		// for the extend
		alphabet_set* labels = new alphabet_set;

		// Extension needs different handling
		relabeling.expression(#expr1,*labels);	// 'abuse' part of CalcRelabeling
		if( relabeling.OK() )
			key->setAlphabet(labels);
	}
	else
	{
		relabeling.start(##);

		if( relabeling.OK() )
			key->setRelabeling(Relabeling::factory(relabeling.getRelabeling()));
	}
	if( ! relabeling.OK() )
	{
		std::ostringstream error;
		error << what << " specification invalid - "	<< relabeling.error();
		throw antlr::SemanticException(error.str(), this->getFilename(),
												 ##->getLine(), ##->getColumn());
	}
}*/
	local_process
;

//	labels "by"^ labels
// FIXME: tree built here is prolly not practical
labels_by_labels
{
	//const Symbol_type *dummy = 0;
	String dummy;
}
:
	dummy=simple_expression "by"^ dummy=simple_expression
;



/** Guarded process
 */
guarded_process!
{
	//const Symbol_type *tp = 0;
	String tp = "";
	boolean isUrgent = false;
}:
	( w:"when" | u:"urgent" { isUrgent = true; } )
	"(" tp=expr:expression ")"
	lp:local_process
	{
		if( w == null )	// play a little dirty..
			w = u;	// set unused RefToken w == u

		/*if( !tp || ! SymData->is_bool(tp) )
		{
			std::ostringstream error;
			error << "guard expression must be boolean";
			// do not use #expr since it may be NULL
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 w->getLine(), w->getColumn());
		}*/

		#guarded_process = #([GUARD_AST,w.getText()], #expr, #lp);
		#guarded_process.setLine(w.getLine());
		#guarded_process.setColumn(w.getColumn());
		#guarded_process.setGuardType(isUrgent);
		//#guarded_process->setScope(SymData->get_current_scope());
	}
;

/** If statement
 * this is a static construct
 * NOTE: probably not used anymore due to disabling of forall
 */
if_stmnt:
   "if"^ "("! expression ")"!
			local_process
	( options { greedy=true; } : "else" local_process )?
;

/** normal variable or array declaration
 * @todo fix some range handling issues in this rule
 */
identifier_or_array_decl ! [/*Symbol_type *type,*/  String sType, int modifiers, boolean inStruct]
	//returns [Symbol *ret_sym = 0]
{
	//std::deque<Bound*> idxs;
	//std::string var_name;
	//String varName ="";
	//const Symbol_type *idx_type;
	String type = "";
	boolean isArray = false;
}
:
	id1:IDENTIFIER 		//{ varName=id1.getText(); }
	(
 		//{ idx_type = 0; }:
		(  "["! type=i:expression! "]"!	{ isArray = true; })
		
		/*{ //{{{ collect upper and lower bounds of array indices 
			// range_type ignored type handled by walk.tree below

			CalcStaticExpr walk( SymData, false );
			Value v = walk(#i);
			if( ! walk.OK() )
			{
				std::ostringstream error;
				error << "size for " << id1->getText() << " must be a constant integer";
				throw antlr::SemanticException(error.str(),this->getFilename(),
														 id1->getLine(),id1->getColumn());
			}
			// need to check type of idx here. Since we're faking a bit of tree
			if ( ! SymData->is_int(idx_type) || ! v.is_int() || (v.i <= 0) )
			{
				std::ostringstream error;
				error << "size for " << id1->getText() << " must be an integer > 0";
				throw antlr::SemanticException(error.str(),this->getFilename(),
														 id1->getLine(),id1->getColumn());
			}
			idxs.push_front( new BoundInt(idx_type, v.i) );
			
		}*/
	)*
	{ /*
		//{{{ Add the found variable declaration to the symbol table 
		// first check modifiers (int/float/action... )
		if( SymData->is_int(type) || SymData->is_float(type) )
		{
			check_modifiers_for_int( modifiers, var_name, id1->getLine(),
											 id1->getColumn() );
		}
		if( SymData->is_action(type) )
		{
			check_modifiers_for_action( modifiers, var_name, id1->getLine(),
												 id1->getColumn() );
		}
		/// @todo: modifier checks for other types

		if( idxs.size() != 0 && (modifiers & (MOD_CONST+MOD_EXTERN)))
		{
			const char* ext = (modifiers & MOD_EXTERN) ? "extern ":"";
			const char* cst = (modifiers & MOD_CONST) ? "constant ":"";

			std::ostringstream error;
			error << ext << cst << " variables are not supported for arrays in definition of '"
					<< var_name << "'";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 id1->getLine(),
													 id1->getColumn()
													 );
		}

		// handle const definition and model parameters
		if( modifiers & MOD_CONST )
		{
			if( (modifiers & MOD_CONST) || (modifiers & MOD_EXTERN) )
			{
				Symbol_const* t;
				if( SymData->is_float(type) )
					ret_sym = t = new Symbol_const(var_name,
															 type, 0.0,
															 this->getFilename().c_str(),
															 id1->getLine(), id1->getColumn(),
															 SymData->get_current_scope(),
															 SymData->get_current_namespace());
				else	// must be int
					ret_sym = t = new Symbol_const(var_name,
															 type, 0,
															 this->getFilename().c_str(),
															 id1->getLine(), id1->getColumn(),
															 SymData->get_current_scope(),
															 SymData->get_current_namespace());

				if( modifiers & MOD_EXTERN )
					t->setParameter(true);
			}
		}
		else  // not a constant or model parameter -> array or normal var
		{
			if( idxs.size() != 0 )
			{
				// we're dealing with an array take care of defining the
				 // types for it.
				 //
				if( ! type->supports_struct( STRUCT_ARRAY ) )
				{
					std::ostringstream error;
					error << "type '" << type->get_name()
						<< "' supports no arrays in definition of '"
						<< var_name << "'";
					throw antlr::SemanticException(error.str(),
															 this->getFilename(),
															 id1->getLine(),
															 id1->getColumn()
															 );
				}
				// the arrays are build using a linked list now.
				  //the real underlying type of an array is obtained by
				  //recursively doing a get_prototype on them untill you get to
				 // a prototype that's not symbol_array.
				 //
				 // int md[10][10]
				 //
				 // md.type = int[10][20]
				 // md[1].type = int[10]
				 // md[1][1].type = int
				 ///
				std::deque<Bound*>::reverse_iterator i = idxs.rbegin();
				const Symbol_type* cur_type = type;

				std::ostringstream typeName;
				typeName << cur_type->get_name();
				//					std::cerr << "Array type: " << typeName.str() << std::endl;
				while( i != idxs.rend() )
				{
					Bound* bound = *i;

					typeName << "[" << bound->get_size() << "]";

					// see if it's been declared ?
					const Symbol_array* array_type =
						dynamic_cast<const Symbol_array*>
						(SymData->find_type(typeName.str()));

					if( array_type == 0 )
					{
						// there was no type yet for this type of array
						// make it and insert into symbol table
						Symbol_array* new_arr = new
							Symbol_array( typeName.str().c_str(),
											  this->getFilename().c_str(),
											  id1->getLine(), id1->getColumn(),
											  SymData->get_current_scope(),
											  SymData->get_current_namespace()
											  );
						// set bound
						new_arr->set_bound(bound);
						// and the underlying type
						new_arr->set_prototype(cur_type);

						try   { SymData->add_type(new_arr); }
						catch ( SymbolTableException& ex ) {
							// rethrow
							std::ostringstream error;
							error << "Internal error failed to build prototype for array '"
								<< var_name << "' error: '" << ex.get_message() << "'";
							throw antlr::SemanticException(error.str(),
																	 this->getFilename(),
																	 ex.get_line(),
																	 ex.get_column() );
						}
						array_type = new_arr;
					}
					cur_type = array_type;
					++i;
				}
				// here all the array mess should be handled types and subtypes
				// should be inserted into symtab now define the symbol
				// with the newly constructed type (or old found one)
				//
				ret_sym = new Symbol( var_name,
											 cur_type,
											 this->getFilename().c_str(),
											 id1->getLine(), id1->getColumn(),
											 SymData->get_current_scope(),
											 SymData->get_current_namespace()
											 );
			}
			else // not an array...
			{
				if( SymData->is_action(type) )	// actions need different class.
				{
					bool patient = def_actions_patient;	// take default...
					if (modifiers & MOD_PATIENT)			// and update if requested..
						patient = true;
					if (modifiers & MOD_IMPATIENT)
						patient = false;

					if( SymData->get_current_scope() != SymData->get_global_scope() )
					{
						std::ostringstream error;
						error << "cannot declare action '" << var_name
							<< "' local (only global scope supported for actions)";
						throw antlr::SemanticException(error.str(),
																 this->getFilename(),
																 id1->getLine(),
																 id1->getColumn()
																 );
					}

					ret_sym = new Symbol_action(var_name,
														 type,
														 patient,
														 this->getFilename().c_str(),
														 id1->getLine(), id1->getColumn(),
														 SymData->get_current_scope(),
														 SymData->get_current_namespace());
				}
				else // make a generic symbol
					ret_sym = new Symbol( var_name, type,
												 this->getFilename().c_str(),
												 id1->getLine(), id1->getColumn(),
												 SymData->get_current_scope(),
												 SymData->get_current_namespace());
			}
		}

		try {
			// And finalyy add the thing to the symtab...
			SymData->add(ret_sym);
		}
		catch( SymbolTableException& ex )
		{
			// Rethrow the beast as something the parser understands...
			throw antlr::SemanticException(ex.get_message(),
													 this->getFilename(),
													 ex.get_line(), ex.get_column() );
		}
//		cout << "Defined symbol: " << *ret_sym << endl;
//		cout << "Defined symbol type: " << *(ret_sym->get_type()) << endl;

//		cout << "PARSER: current scope: " << SymData->get_current_namespace()
//			<< " sym: " << ret_sym->get_name()
//			<< endl;
		// and add symbol to the scope it's in
		SymData->get_current_scope()->add_symbol(ret_sym);
		
		*/
		if (isArray) sType = sType + "_array";
		Symbol varSymbol = new Symbol(id1.getText(), sType);
		varSymbol.setLine(id1.getLine());
		varSymbol.setColumn(id1.getColumn());
		if (!inStruct){
			if (sType.equals("exception"))
				excs.add(varSymbol);
			else vars.add(varSymbol);
			unUsedVars.put(id1.getText(), varSymbol);	
		}

		## = #([IDENTIFIER, id1.getText()]);
		##.setSymbol(new Symbol(id1.getText(), sType));
		##.setLine(id1.getLine());
		##.setColumn(id1.getColumn());
		
	}
;

/** Qualified identifier
 * 'normal' identifier reference with optional namespace specification
 * @return a Symbol_type * for the qualified id.
 * or throw semantic exception if id not found.
 * this rule is only used to referenced things that should be
 * defined on beforehand.
 * @note The default errorhandler is turned of so that this rule behaves
 * as if it's one thing. This prevents it returning sym = 0 without throwing
 * an error.
 */
qualified_identifier [boolean allow_single_id]
	returns [String type = null]//[const Symbol_type *type = 0]
//	options { defaultErrorHandler = false; }
	{
		//Namespace *ns = ns = SymData->get_current_namespace();
		boolean id_only = false;
		//std::string mem_parent;
		//Symbol *sym = 0;
		Symbol sym = null;
		//System.err.println("in qualified id");
	}
:
	base:IDENTIFIER
	{	
				
		/*{{{ look up the base name and do some preliminary sorting out... */

		// there's a catch...
		// in certain cases we need to match a single IDENTIFIER (for sets)
		// selecting between a defined symbol and a thing that should be added
		// to a set cannot be done with antlr guessing (since in guessing mode
		// no actions are executed).
		//
		// So now we change the return AST in a few cases.

		//sym = SymData->find(base->getText().c_str() );
		
		for (Iterator iter = vars.iterator(); iter.hasNext();)
			{
				Symbol var = (Symbol)iter.next();
				//System.err.println("Act Var: " + var.getName());
				//System.err.println("name of action: " + base.getText());
				if (base.getText().equals(var.getName()))
				//{
					//System.err.println("matched the action to a known declaration");
					sym = var;
				//}
			}
		if( sym == null )
		{
			int la = LA(1);

			if( allow_single_id && (la != LBRACE && la != DOT))
				id_only = true;
			else
			{
				String error = "undefined symbol '" + base.getText() + "'";
				//System.err.println(error);
				throw new SemanticException(error, this.getFilename(),
														 base.getLine(), base.getColumn());
			}
		}
		else
		{
			if (unUsedVars.containsValue(sym))
					unUsedVars.remove(sym.getName());
			
			
			type = sym.getType();
			#base.setSymbol(sym);
		}
		/*}}}*/
	}
	// Array indices and struct members
	(
		(
			dot:"."^ member:IDENTIFIER
	 		/*{
				if (SymData->is_struct(type))
				{
					// get the type
					const Symbol_struct* st = dynamic_cast<const Symbol_struct*>(type);
					assert(st);
					// before we 'forget' the struct, refcount it
					// (fields do not have to be refcounted, their structs are)
					sym->inc_refcount();
					// the new symbol is now this member of the symbol
					if( (sym = st->find_field(member->getText())) == 0 )
					{
						std::ostringstream error;
						error << member->getText() << " is not a member of struct type " << st->get_name();
						throw antlr::SemanticException( error.str(),
																  this->getFilename(),
																  base->getLine(),
																  base->getColumn() );
					}
					type = sym->get_type();
					#member->setSymbol(sym);
					#dot->setSymbol(sym);
				}
				else
				{
					std::ostringstream error;
					error << "Variable " << sym->get_name() << " is not a structure";
					throw antlr::SemanticException( error.str(),
															  this->getFilename(),
															  base->getLine(),
															  base->getColumn() );
				}
			}*/
		)
	|	( 	{ /*const Symbol_type *idx_type = 0;*/ String idx_type = ""; }:
			index:"["^ idx_type=expr:expression "]"!
			/*{
//				const char* tn = (type?type->get_name():"<null>");
//				cerr << "Index encountered - current tp = " << tn << endl;

				if( type && SymData->is_array(type) )
				{
					const Symbol_array* sa = dynamic_cast<const Symbol_array*>(type);
					assert(sa);
					// before we 'forget' the array, refcount it
					sym->inc_refcount();
					if (SymData->is_int(idx_type))
					{
						// get the underlying type after one indexing...
						const Symbol* s = sa->get_prototype();
						const Symbol_type *inner_type = dynamic_cast<const Symbol_type*>(s);
						#index->setSymbol(type);
						type = inner_type;

//						std::cerr << "Indexing of " << base->getText() << " type " << (type?type->get_name():"<null>") << std::endl;
					}
					else
					{
						std::ostringstream error;
						error << "Index for " << sym->get_name() << " must be an integer";
						throw antlr::SemanticException(error.str(),this->getFilename(),
							base->getLine(),base->getColumn());
					}
				}
				else
				{
					std::ostringstream error;
					error << "Trying to index non-array '" << sym->get_name() << "'";
					throw antlr::SemanticException(error.str(),this->getFilename(),
						base->getLine(),base->getColumn());
				}
			}*/
		)
	)*
	/*{
		mem_parent.assign(base->getText());
		//}}}

		if( sym == 0 && ! id_only )
		{
			std::ostringstream error;
			error << "this symbol did not make any sense to me '"
				<< base->getText() << "' guess I'm a stupid compiler :(";
			throw antlr::SemanticException(error.str(), this->getFilename(),
													 base->getLine(), base->getColumn());
		}

		sym->inc_refcount();

#if 0
		cout << "Qualified Identifier tree: >>>>>>>>>>>>>>>>>>>>>>>>>>>>>" << endl;
		print_tree<RefModest_AST,Modest_Parser> pr(*this);
		pr.pr_tree(##);
		cout << "Qualified Identifier tree: <<<<<<<<<<<<<<<<<<<<<<<<<<<<<" << endl;
#endif
	}*/
;

/** Process definition
 * Define a process.
 */
process_definition ! //{ Scope *prev_scope; Symbol_process* sp = 0; }
{
	ArrayList<Symbol> params = null;	
}
:
	{
		if( procLevel > 0 )
		{
			//std::ostringstream error;
			String error = "nested processes are not supported";
			throw new SemanticException(error,
													 this.getFilename(),
													 LT(1).getLine(), LT(1).getColumn());
		}
	}
	"process" name:IDENTIFIER
	{
		Symbol process = new Symbol(name.getText(), "process");
		process.setLine(name.getLine());
		process.setColumn(name.getColumn());
		unUsedVars.put(name.getText(), process);
		processes.add(name.getText());
	}
	
	/*{
		Symbol *sym = SymData->find( name->getText() );
		if( sym )
		{
			sp = dynamic_cast<Symbol_process*>(sym);
			if( !sp || (sp &&
				 sp->get_declaration_state() != Symbol_parameterized::FORWARD) )
			{
				std::ostringstream error;
				error << "symbol '" << name->getText() << "' doubly defined";
				throw antlr::SemanticException(error.str(),
														 this->getFilename(),
														 name->getLine(), name->getColumn());
			}
		}
		if( sp == 0 )	// don't have to allocate for forward declared one..
		{
			sp = new Symbol_process(name->getText(),
							  SymData->get_builtin_type(SymbolInformation::TP_PROCESS).info,
							  this->getFilename().c_str(),
							  name->getLine(), name->getColumn(),
							  SymData->get_current_scope(),
							  SymData->get_current_namespace());

			// if it already existed we don't have to add only this case..
			try {
				SymData->add(sp);
			}
			catch ( SymbolTableException& ex ) {
				throw antlr::SemanticException(ex.get_message(),
														 this->getFilename(),
														 ex.get_line(), ex.get_column() );
			}
		}
		// hmm in the case of forward declarations then what ??
		SymData->get_current_scope()->add_symbol(sp);

		// Add a new scope... (hmm for a forward declaration we create
		// a dummy scope like this..)
		prev_scope = SymData->get_current_scope();
		Scope *new_scope = SymData->get_current_scope()->add_child(sp);
		SymData->set_current_scope( new_scope );
		sp->setChildScope(new_scope);
	}*/
	params=pdef:param_definition[name.getText()]//[ sp ]
	{ vars.addAll(params); }
	( ";"
	| { procLevel++; } pbody:process_body { procLevel--; }
	)
	{
		// restore the scope..
		// cout << "Scope #" << SymData->get_current_scope()->get_id() << ": size = " << SymData->get_current_scope()->get_size() << endl;
		## = #([PROCESS_DEFINITION_AST, "proc_def"], #pdef, #pbody );
		//##->setSymbol(sp);
		//##->setScope(SymData->get_current_scope());
		##.setLine(name.getLine());
		##.setColumn(name.getColumn());
		// in the case of no pbody -> forward declaration
		/*sp->set_declaration_state( (#pbody == 0) ?
				Symbol_parameterized::FORWARD : Symbol_parameterized::COMPLETE );*/
		//sp->setAST( ## );
		//SymData->set_current_scope( prev_scope );
		##.setProcName(name.getText());
		for (int i = 0; i < params.size(); i++) {
			vars.remove(params.get(i));
		}
	}
;

/** Process body
 * used as process body and as function body.
 */
process_body:
	"{"! description/*[0]*/ "}"!
;


/** Parallel composition
 */
parallel_composition
//{ System.err.println("in parallel_composition"); }
:
	par:"par"^ LCURL!
		( alternative )+
	RCURL!
;

/** Alt statement
 */
alt_stmnt
//{ System.err.println("in alt_stmnt"); }
:
	alt:"alt"^ LCURL!
	( alternative )+
	RCURL!
;

/** Loop statement
 */
loop_stmnt
//{System.err.println("in loop_stmnt");}
:
	"do"^ LCURL!
	( alternative )+
	RCURL!
;

alternative
//{System.err.println("in alternative");}
:
	DCOLON^ sequential_processes
;



/** Normal assignment
 */
assignment_op returns [Token tok = new Token()]:
	a1:"=" { tok = a1; }
|	a2:"+=" { tok = a2; }
|	a3:"-=" { tok = a3; }
|	a4:"*=" { tok = a4; }
|	a5:"/=" { tok = a5; }
|	a6:"%=" { tok = a6; }
|	a7:">>=" { tok = a7; }
|	a8:"<<=" { tok = a8; }
|	a9:"&=" { tok = a9; }
|	a10:"^=" { tok = a10; }
|	a11:"|=" { tok = a11; }
;

/** Assignment rule. The parameter controls wether a 'tau PALT AST' needs to be
 * inserted in front of the assignment
 */
assignment ! [boolean need_palt]
{
	//const Symbol_type* expr_type = 0, *lhs_type = 0;
	String expr_type = "", lhs_type="";
	Token op;
}:
	lhs_type=id:qualified_identifier[false]
	op=oper:assignment_op
	expr_type=e:expression
	{
//		cout << "Assignment ------------\n";
//		print_tree<RefModest_AST,Modest_Parser> pr(*this);
//		pr.pr_tree(#id);
//		cout << "--\n";
//		pr.pr_tree(#oper);
//		cout << "--\n";
//		pr.pr_tree(#e);
//		cout << "Assignment ------------\n";

		// tag the local scope on the assignment operator
		/*#oper->setScope(SymData->get_current_scope());

		if( ! lhs_type )
		{
			std::ostringstream error;
			int line, column;

			if( op )
			{
				error << "invalid left hand side symbol in '" << op->getText()
					<< "' expression";
				line = op->getLine();
				column = op->getColumn();
			}
			else
			{
				error << "invalid left hand side symbol in assignment (near this line)";
				// these may be off some value due to guessing
				line = this->getLine();
				column = this->getColumn();
			}
			throw antlr::SemanticException(error.str(), this->getFilename(),
													 line, column);
		}
		if( ! #e || !types_compatible_for_assignment( lhs_type, expr_type ) )
		{
			std::ostringstream error;
			error << "incompatible types used in "
				<< op->getText() << " expression";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}
		// have to do special check on clocks, hope the id->getSymbol is ok..
		if( SymData->is_clock(lhs_type) )
			check_clock_assignment( #id->getSymbol(), #e );*/

		if( need_palt )
		{
			// reportWarning(op, "inserting synthetic tau palt { .. } for assignment");

			ModestAST tau = create_tau_ast(op.getLine(), op.getColumn());
			ModestAST assign = (#[ASSIGN_START,"{="]);
			//assign->setScope(SymData->get_current_scope());
		    ModestAST weight = (#[WEIGHT_AST,"::"]);
		    weight.setWeight((double)1);

			## = #(tau,
						#([PALT,"palt"],
							#(#weight,
								#(assign, #(oper, id, e))
							  )
						 )
					);
		}
		else
		{
			## = #(oper, id, e);
		}
	}
;
exception
	catch [ RecognitionException ex ]
	{
		// the default errorhandler has a tendency to eat one token too many
		// when recovering from an error. Make sure to eat up stuff till the
		// $FOLLOW set.
		if( inputState.guessing == 0 ) {
			reportError(ex);
			consumeUntil($FOLLOW);
		} else {
			//throw;
		}
	}

assignments:
	assignment[false] (","! assignment[false])*
;

/** Atomic assignment block
 * used in palt statement
 */
assignment_block:
	top:ASSIGN_START^ assignments ASSIGN_END!
	/*{
		#top->setScope(SymData->get_current_scope());
	}*/
;

/** Note: Alternative :: treated as :1:
 * @todo fix this for the change of weights to floats
 */
weight 
{
	//const Symbol_type *tp = 0;
	String tp = "";
}
:!
	(c:":"! ( tp=expr:simple_expression )? ":"!)
{
	if( #expr != null )
	{
// we do not forbid floats anymore                
        /*if( ! (SymData->is_int(tp) || SymData->is_float(tp)))
		{
			std::ostringstream error;
			error << "weight should be an integer or a float";
			throw antlr::SemanticException(error.str(), this->getFilename(),
													 #expr->getLine(),
													 #expr->getColumn() );
		}

		CalcStaticExpr walk( SymData, false );
		Value v = walk(#expr);

		if( ! walk.OK() || ! v.gt_zero() )
		{
			std::ostringstream error;
			error << "weight should be constant and >= 0 ("
				<< walk.error() << ")";
			throw antlr::SemanticException(error.str(), this->getFilename(),
													 c->getLine(),
													 c->getColumn() );
		}*/
		#weight = #([WEIGHT_AST,"::"]);
		/*if( v.is_int() )
			#weight->setWeight( v.i );
		else
			#weight->setWeight( v.f );*/
		#weight.setLine(c.getLine());
		#weight.setColumn(c.getColumn());
		#weight.setWeight(Double.valueOf(#expr.getText()));
	}
	else
	{
		#weight = #([WEIGHT_AST,"::"]);
		#weight.setWeight( (double)1 );
		#weight.setLine(c.getLine());
		#weight.setColumn(c.getColumn());
	}
}
;

palt_alternative!:
	w:weight	{ ## = #w;	}
	(	asgn:assignment_block { ##.addChild(#asgn); }
		(SEMICOLON! seq:sequential_processes { ##.addChild(#seq); } )?
	|	seq2:sequential_processes { ##.addChild(#seq2); }
	)
;

palt_stmnt!
//{System.err.println("in palt_stmnt");}
:
	p:"palt" LCURL				  { ## = #p; }
		( alt:palt_alternative { ##.addChild(#alt); }	)+
	RCURL
;

/** Parameter list for invocation of functions or instantiation of processes
 */
arguments//[ Symbol_parameterized* function ] 
{
	//Symbol_parameterized::const_iterator param_i;
	/*unsigned*/ int param_count = 0;
	//const Symbol_type *tp;
	String tp = "";
}
:
	arg_start:"("!
	( tp=expr1:expression
	{
		param_count++;
		/*param_i = function->param_begin();

		// make sure we have something to getLine/Column from
		if( !#expr1 )
			#expr1 = #arg_start;

		if( param_i == function->param_end() )
		{
			std::ostringstream error;
			error << "too many parameters passed to '" << function->get_name() << "'";
			throw antlr::SemanticException(error.str(), this->getFilename(),
												 #expr1->getLine(),
												 #expr1->getColumn());
		}

//		if( tp != (*param_i)->get_type() )
		if( !types_compatible_for_assignment( (*param_i)->get_type(), tp ) )
		{
			std::ostringstream error;
			error << "'" << function->get_name() << "' expects '"
				<< (*param_i)->get_type()->get_name() << "' parameter at position " << param_count;
			throw antlr::SemanticException(error.str(), this->getFilename(),
												 #expr1->getLine(),
												 #expr1->getColumn());
		}

		param_i++;*/
	}
	(comma:","! tp=expr2:expression
	{
		param_count++;

		// make sure we have something to getLine/Column from
		/*if( !#expr2 )
			#expr2 = #comma;

		if( param_i == function->param_end() )
		{
			std::ostringstream error;
			error << "too many parameters passed to '" << function->get_name() << "'";
			throw antlr::SemanticException(error.str(), this->getFilename(),
												 #expr2->getLine(),
												 #expr2->getColumn());
		}

//		if( tp != (*param_i)->get_type() )
		if( !types_compatible_for_assignment( (*param_i)->get_type(), tp ) )
		{
			std::ostringstream error;
			error << "'" << function->get_name() << "' expects '"
				<< (*param_i)->get_type()->get_name() << "' parameter at position " << param_count;
			throw antlr::SemanticException(error.str(), this->getFilename(),
												 #expr2->getLine(),
												 #expr2->getColumn());
		}

		param_i++;*/
	}
	)*
	)?
	")"!
	/*{
		if( param_count != function->param_count() )
		{
			std::ostringstream error;
			error << "'" << function->get_name() << "' expects "
				<< function->param_count() << " parameters";
			throw antlr::SemanticException(error.str(), this->getFilename(),
												 arg_start->getLine(),
												 arg_start->getColumn());
		}
	}*/
;

/** Parameter list for definition of functions/processes
 */
param_definition ! [String function]//[ Symbol_parameterized* function ] 
returns [ArrayList<Symbol> params = new ArrayList<Symbol>()]
{ /*unsigned*/ int cnt = 0;
	Symbol p = null;
	Symbol p2 = null;
} 
:
	t:"("
	{
		## = #([PARAMETER_LIST_AST, "params"]);
		##.setLine(t.getLine());
		##.setColumn(t.getColumn());
	}
		( { ++cnt; } p=p1:parameter[ function, cnt ]		{ ##.addChild(#p1); if (p!=null) params.add(p); }
		("," { ++cnt; } p2=pn:parameter [ function, cnt ]	{ ##.addChild(#pn); if (p2!=null) params.add(p2); }
		)*
		)?
	")"
	/*{
		// in the case of a forward declaration do extra check on param count
		if( function->get_declaration_state() == Symbol_parameterized::FORWARD &&
			 cnt < function->param_count() )
		{
			std::ostringstream error;
			error << "definition does not match prototype for '" << function->get_name() << "'";
			throw antlr::SemanticException(error.str(), this->getFilename(),
													 function->get_line(),
													 function->get_column());
		}
	}*/
;

parameter ! [ /*Symbol_parameterized* function,*/ String function, int cnt ] returns [Symbol param=null]
{
	int modifiers = MOD_NONE;
	String tp = "";
	/*Symbol_type *tp;
	Symbol *sym = 0;*/
}:
	( modifiers=modifier )? tp = type:qualified_type id:IDENTIFIER
{
	if( tp == "" )
	{
		//std::ostringstream error;
		String error = "invalid type in parameter " + cnt + " of '" + function + "'";
		throw new SemanticException(error, this.getFilename(),
												 id.getLine(),
												 id.getColumn());
	} else {
			param = new Symbol(id.getText(), tp);
	}
	// in the case of forward declarations we check the prototype against
	/*// what we got here..
	if( function->get_declaration_state() == Symbol_parameterized::FORWARD )
	{
		Symbol* checksym = function->get_param(cnt);

		if( checksym == 0 || (checksym && checksym->get_type() != tp))
		{
			std::ostringstream error;
			error << "definition does not match prototype for '" << function->get_name() << "' parameter " << cnt;
			throw antlr::SemanticException(error.str(), this->getFilename(),
													 function->get_line(),
													 function->get_column());
		}
		sym = checksym;
	}
	else
	{
		// in the other case we just define the mutha..
		if( modifiers & MOD_CONST )
		{
			std::ostringstream error;
			error << "const modifier supported for parameters in declaration of '" << id->getText() << "'";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 id->getLine(), id->getColumn() );
		}
		else
		{
			sym = new Symbol(id->getText().c_str(),
								  tp,
								  this->getFilename().c_str(),
								  id->getLine(), id->getColumn(),
								  SymData->get_current_scope(),
								  SymData->get_current_namespace());
		}
		try {
			SymData->add(sym);
		}
		catch ( SymbolTableException& ex ) {
			throw antlr::SemanticException(ex.get_message(),
													 this->getFilename(),
													 ex.get_line(), ex.get_column() );
		}

		// and add it to the parameter list of the function/process
		function->add_parameter( sym );
	}*/

	// and add symbol to the scope it's in
	//SymData->get_current_scope()->add_symbol(sym);

	// add a VARIABLE_DECLARATION_AST node as top...
	#parameter = #( [VARIABLE_DECLARATION_AST, id.getText()] );
/*#if 0
	// AST nodes do not have a symbol (anymore) but a symbol-reference.
	// In order to get a symbol make a symbolreference.
	if (sym)
		#parameter->setSymbolReference(sym->make_reference(this->getFilename(),id->getLine(),id->getColumn()));
	else
		#parameter->setSymbolReference(0);
#else
	#parameter->setSymbol(sym);
#endif*/
	#parameter.setLine(id.getLine());
	#parameter.setColumn(id.getColumn());
}
;

/* ***************************************************************************/
/* Expressions
 *****************************************************************************/

/* Note that simple expression does not support expressions enclosed in
 * parentheses
 */
simple_expression returns [String resultType = ""]
//{ System.err.println("in simple_expression"); }
://const Symbol_type* result_type = 0]:
   resultType = additive_expr
;

expression returns [String resultType = ""]//[const Symbol_type* result_type = 0]
{
	//const Symbol_type *opt1 = 0, *opt2 = 0;
	String opt1 = null, opt2 = null;
}
	:
   resultType=or_expr
	( op:"?"^ opt1=or_expr ":"! opt2=or_expr
	/*{
		if( !result_type || !SymData->is_bool(result_type) )
		{
			std::ostringstream error;
			error << "incompatible types in ? expression should be boolean";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}
		if( !opt1 || !opt2 || (opt1 != opt2) )
		{
			std::ostringstream error;
			error << "alternatives should have compatible type in ? .. : ..";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}
		result_type = opt1;
	}*/
	{ resultType = opt1; })?
;

or_expr returns [String resultType = ""]//[const Symbol_type* result_type = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
}:
   lhs=and_expr  { resultType = lhs; /* initialize result_type */ }
	( op:"||"^ rhs=and_expr
 		{
			/*if( (!lhs || !rhs) || (! SymData->is_bool(lhs) && !SymData->is_bool(rhs)) )
			{
				std::ostringstream error;
				error << "incompatible types in || expression";
				throw antlr::SemanticException(error.str(),
														 this->getFilename(),
														 op->getLine(), op->getColumn());
			}*/

			resultType = lhs;
			lhs = rhs;	// prev right hand becomes left hand side.
		}
	) *
;

and_expr returns [String resultType = ""]//[const Symbol_type* result_type = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
}:
   lhs=bitor_expr { resultType = lhs; /* initialize resultType */ }
	( op:"&&"^ rhs=bitor_expr
		{
			/*if( (!lhs || !rhs) || (! SymData->is_bool(lhs) && !SymData->is_bool(rhs)) )
			{
				std::ostringstream error;
				error << "incompatible types in && expression";
				throw antlr::SemanticException(error.str(),
														 this->getFilename(),
														 op->getLine(), op->getColumn());
			}*/

			resultType = lhs;
			lhs = rhs;	// prev right hand becomes left hand side.
		}
	) *
;
bitor_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
}:
	lhs=bitexclor_expr  { resultType = lhs; /* initialize resultType */ }
	( op:"|"^ rhs=bitexclor_expr
	{
		/*if( (!lhs || !rhs) || (! SymData->is_int(lhs) && !SymData->is_int(rhs)) )
		{
			std::ostringstream error;
			error << "incompatible types in | expression";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}*/

		resultType = lhs;
		lhs = rhs;	// prev right hand becomes left hand side.
	}
	) *
;

bitexclor_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
}:
	lhs=bitand_expr  { resultType = lhs; /* initialize resultType */ }
	( op:"^"^ rhs=bitand_expr
	{
		/*if( (!lhs || !rhs) || (! SymData->is_int(lhs) && !SymData->is_int(rhs)) )
		{
			std::ostringstream error;
			error << "incompatible types in ^ expression";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}*/
		resultType = lhs;
		lhs = rhs;	// prev right hand becomes left hand side.
	}
	) *
;

bitand_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
}:
	lhs=equality_expr  { resultType = lhs; /* initialize resultType */ }
	( op:"&"^ rhs=equality_expr
	{
		/*if( (!lhs || !rhs) || (! SymData->is_int(lhs) && !SymData->is_int(rhs)) )
		{
			std::ostringstream error;
			error << "incompatible types in & expression";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}*/
		resultType = lhs;
		lhs = rhs;	// prev right hand becomes left hand side.
	}
	) *
;

eq_op ! returns [Token tok = new Token()]:
	eq:"==" { tok = eq; }
|	ne:"!=" { tok = ne; }
;

equality_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
	Token op;
}:
	lhs=la:relational_expr  { resultType = lhs; /* initialize resultType */ }
	( op=oa:eq_op
  	  { astFactory.makeASTRoot(currentAST, astFactory.create(op)); }
	  rhs=ra:relational_expr
	{
		/*if( ! types_compatible_for_compare(lhs,rhs) )
		{
			std::ostringstream error;
			error << "incompatible types in " << op->getText() << " expression";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}*/

		resultType = "bool";//SymData->get_builtin(SymbolInformation::TP_BOOL);
		lhs = rhs;	// prev right hand becomes left hand side.
	}
	) *
;

comp_op ! returns [Token tok = new Token()]:
	lt:"<" { tok = lt; }
|	gt:">" { tok = gt; }
|	ge:">=" { tok = ge; }
|	le:"<=" { tok = le; }
;

relational_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
	Token op;
}:
   lhs=shift_expr  { resultType = lhs; /* initialize resultType */ }
	( op=comp_op
	  { astFactory.makeASTRoot(currentAST, astFactory.create(op)); }
  	  rhs=shift_expr
 	{
		/*if( ! types_compatible_for_compare(lhs,rhs) )
		{
			std::ostringstream error;
			error << "incompatible types in " << op->getText() << " expression";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}*/

		resultType = "bool"; //SymData->get_builtin(SymbolInformation::TP_BOOL);
		lhs = rhs;	// prev right hand becomes left hand side.
	}
	) *
;

shift_op ! returns [Token tok = new Token()]:
	l:"<<" { tok = l; } | r:">>" { tok = r; }
;

shift_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
	Token op;
}:
   lhs=additive_expr   { resultType = lhs; /* initialize resultType */ }
	( op=shift_op
	  { astFactory.makeASTRoot(currentAST, astFactory.create(op)); }
	  rhs=additive_expr
 	{
		/*if( (!lhs || !rhs) || (! SymData->is_int(lhs) && !SymData->is_int(rhs)) )
		{
			std::ostringstream error;
			error << "incompatible types in " << op->getText() << " expression";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}*/

		resultType = lhs;
		lhs = rhs;	// prev right hand becomes left hand side.
	}
	)*
;

additive_op ! returns [Token tok = new Token()]:
	p:"+" { tok = p; } | m:"-" { tok = m; }
;

additive_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
	Token op;
	
	//System.err.println("in additive_expr"); 
}:
   lhs=multiplicative_expr	{ resultType = lhs; /* initialize resultType */ }
	( op=additive_op
	  { astFactory.makeASTRoot(currentAST, astFactory.create(op)); }
	  rhs=multiplicative_expr
 	{
		/*if((resultType = get_resultType_for_op( op->getType(), lhs, rhs)) == 0 )
		{
			std::ostringstream error;
			error << "incompatible types in " << op->getText() << " expression";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn() );
		}*/

		resultType = rhs;
		lhs = rhs;	// prev right hand becomes left hand side.
	}
	)*
;

multiplicative_op !
	returns [Token tok = new Token()]
:
	t:"*" { tok = t; } | d:"/" { tok = d; } | r:"%" { tok = r; }
;

multiplicative_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0]
{
	//const Symbol_type *lhs = 0, *rhs = 0;
	String lhs = null, rhs = null;
	Token op;
	
	//System.err.println("in multiplicative_expr");
}:
   lhs=unary_expr  { resultType = lhs; /* initialize resultType */ }
	( op=multiplicative_op
	  { astFactory.makeASTRoot(currentAST, astFactory.create(op)); }
	  rhs=unary_expr
 	{
		/*if((resultType = get_resultType_for_op( op->getType(), lhs, rhs)) == 0 )
		{
			std::ostringstream error;
			error << "incompatible types in " << op->getText() << " expression";
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 op->getLine(), op->getColumn());
		}*/

		resultType = lhs;
		lhs = rhs;	// prev right hand becomes left hand side.
	}
	)*
;

unary_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0]
{
	//Symbol_type* cast_type = 0;
	String castType = "";
	
	//System.err.println("in unary_expr");
}
:
	up:"+"^ resultType=base_expr		// unary plus (pretty redundant)
		/*{
			if((resultType =
				 get_result_for_unary_op(up->getType(),resultType)) == 0 )
			{
				std::ostringstream error;
				error << "unary + expects integer argument";
				throw antlr::SemanticException(error.str(),
														 this->getFilename(),
														 up->getLine(), up->getColumn());
			}
		}*/
|	um:"-"^ resultType=base_expr		// unary minus
		/*{
			if((resultType =
				 get_result_for_unary_op(um->getType(),resultType)) == 0 )
			{
				std::ostringstream error;
				error << "unary - expects integer argument";
				throw antlr::SemanticException(error.str(),
														 this->getFilename(),
														 um->getLine(), um->getColumn());
			}
		}*/
|	un:"!"^ resultType=base_expr		// unary logical not
		/*{
			if((resultType =
				 get_result_for_unary_op(un->getType(),resultType)) == 0 )
			{
				std::ostringstream error;
				error << "unary ! expects boolean argument";
				throw antlr::SemanticException(error.str(),
														 this->getFilename(),
														 un->getLine(), un->getColumn());
			}
		}*/
|	uh:"#"^ resultType=base_expr		// cardinality operator (for e.g. sets)
		/*{
			if((resultType =
				 get_result_for_unary_op(uh->getType(),resultType)) == 0 )
			{
				std::ostringstream error;
				error << "set cardinality operator expects set argument";
				throw antlr::SemanticException(error.str(),
														 this->getFilename(),
														 uh->getLine(), uh->getColumn());
			}
		}*/
|!  ("(" qualified_type ) => uc:"(" castType = qualified_type ")" resultType=expr:base_expr
	{
		/*if( (resultType = get_cast_result( resultType, cast_type )) == 0 )
		{
			std::ostringstream error;
			error << "Invalid/unsupported cast from "
				<< resultType->get_name() << " to "
				<< cast_type->get_name();
			throw antlr::SemanticException(error.str(),
													 this->getFilename(),
													 uc->getLine(), uc->getColumn());
		}*/
		## = #([TYPE_CAST, castType], #expr );
		##.setSymbolType(castType);
	}
|	resultType=base_expr
;


base_expr returns [String resultType = ""]//[const Symbol_type* resultType = 0] 
//{System.err.println("in base_expr");}
:
   INT 	 
   	{ resultType = "int"; } 
   	//resultType = SymData->get_builtin(SymbolInformation::TP_INT); }
|  FLOAT  
	{ resultType = "float"; }
		//resultType = SymData->get_builtin(SymbolInformation::TP_FLOAT); }
//|	STRING { resultType = SymData->get_builtin(SymbolInformation::TP_STRING); }
// FIXME: implement at operator
//|  "@"! "("! simple_expression ","! expression ")"!
|  "("! resultType=expression ")"!
|  set1:set
//	{
//		print_tree<RefModest_AST, Modest_Parser> pr(*this);
//		pr.pr_tree(#set1);
//	}
|  resultType=range
|	( parameterized_object ) => resultType=parameterized_object		// function call
|  ( qualified_identifier[false] ) =>
	( resultType=qualified_identifier[false]	)
;

/** Function invocation or process instantiation
 */
parameterized_object! returns [String resultType =""]//returns [const Symbol_type* result_type = 0]
{
	//Symbol_process* proc = 0;
	//Symbol_function* func = 0;
	boolean process = false; //See if we want to check somewhere if it is a func or a proc
	boolean function = false;
}
:
	id:IDENTIFIER
	{ 
		if (id.getText().equals("Normal") || id.getText().equals("Uniform") || id.getText().equals("Exponential"))
			function = true;
		else process = true;
		if (unUsedVars.containsKey(id.getText()))
					unUsedVars.remove(id.getText());
		/*if( Symbol* sym = SymData->find( id->getText().c_str() ) )
		{
			// see if it's a function/process
			if( (proc = dynamic_cast<Symbol_process*>(sym)) == 0 )
				func = dynamic_cast<Symbol_function*>(sym);*/

			//if ( processMap.containsKey(id.getText()) ) process = true;

			// if not bomb out...
			//if( proc == 0 && func == 0 )
			/*if ( !process && !function )
			{
				//std::ostringstream error;
				String error = "attempting to call/instantiate non function/process";
				throw new SemanticException(error, this.getFilename(),
														 id.getLine(), id.getColumn());
			}*/
			//#id->setSymbol(sym);
		/*}
		else
		{
			std::ostringstream error;
			error << "undefined symbol '" << id->getText() << "'";
			throw antlr::SemanticException(error.str(), this->getFilename(),
													 id->getLine(), id->getColumn());
		}*/
		
	}
	args:arguments/*[proc ? static_cast<Symbol_parameterized*>(proc):
								 static_cast<Symbol_parameterized*>(func)]*/
	{ 
		if( process )
		{
			## = #( [PROCESS_INSTANTIATION_AST, id.getText()], #args);
			//##->setSymbol(proc);
			##.setLine(id.getLine());
			##.setColumn(id.getColumn());
			//##->setScope(SymData->get_current_scope());
		}
		else if ( function )
		{
			## = #([FUNCTION_CALL_AST, id.getText()], #args);
			//##->setSymbol(func);
			##.setLine(id.getLine());
			##.setColumn(id.getColumn());
			//##->setScope(SymData->get_current_scope());
			resultType = "float"; //func->get_type();
			// Check out how to find the result type of the function ;)
		}
		
	}
;


/* set of labels or numbers. This is a bonus over FSP at practically no cost
 * still have to look at the semantics.
 * Sets may appear as:
 * - typedef set { ... }    => definition of the bugger
 * - action a.{ a, b, c }   => definition of a.a a.b a.c (or withouth dot)
 *                             !!! was junked!
 * - forall id : { a, b }   => forall over set...
 * - priority { a, b }
 *                             !!!! junked!
 * - hide { x, y, z }		 => all hiding/expose/relabel things
 * - { a, b, c } + { d }    => calculations with sets..
 *
 * Issues:
 * - the set rule is always a definition of a named (typedefed) or anynomous
 *   set
 * - What is the most practical way to implement references to set members
 *   In general different behaviour is needed for a number of the uses
 *   * a typedef should not contain numbers
 *   * the relabeling/hiding operations should not either
 *   * inside relabeling/hiding it should all be named predefined actions or
 *     existing sets of those.
 *   *
 * Options:
 * -- introduce numerical sets and symbolic sets.
 * -- numerics handy for forall and
 *
 */
/** Set definition
 * @todo finish last bits of set implementation see comment before.
 */
set //{ Symbol *set; }
//{ System.err.println("in set"); }
: LCURL^ set_element/*[set]*/ ( ","! set_element/*[set]*/ )* RCURL!
;

set_element//[ Symbol * ] 
{
	//const Symbol_type *type = 0;
	String type = null;
}
:
	i:INT														// numerical sets
|  type=rng:range
|	(qualified_type) => type = qualified_type
|	type = qualified_identifier[true]				// include other set
																// or add symbolic name to set
;

/** specify a range [a_range_type] or [<numeral>..<numeral>]
 */
range returns [String returnType = ""]//[const Symbol_type* return_type = 0;]
{
	//const Symbol_type *type1 = 0, *type2 = 0;
	String type1 = "", type2 ="";
}:
	"["!
	 ( (qualified_type) => returnType=qualified_type
	 | ( type1=simple_expression dots:".."^ type2=simple_expression
		{
			if( type1 != type2 )
			{
				//std::ostringstream error;
				String error = "range expressions not compatible";
				throw new SemanticException(error, this.getFilename(),
														 dots.getLine(),
														 dots.getColumn());
			}
			/*if( !SymData->is_int(type1) || !SymData->is_int(type2) )
			{
				std::ostringstream error;
				error << "range expression(s) should be integers";
				throw antlr::SemanticException(error.str(), this->getFilename(),
														 dots->getLine(),
														 dots->getColumn());
			}*/
			// finaly set return type...
			returnType = type1;
		}
	  )
	)
	"]"!
;

/** Struct definition
 */
astruct//[ASTVector& asgn]
: 
	"{"^ ( variable_declaration[true]/*[asgn]*/ ) + "}"!
;




/* *************************************************************************
* Here begins the Outline Tree Walker!
***************************************************************************/

{import modesteditor.core.model.Process;
import modesteditor.core.model.*;
import java.util.ArrayList;
import java.util.Vector;
}

class ModestOutlineTreeWalker extends TreeParser;
options {
	ASTLabelType = "ModestAST";
	buildAST = false;
	importVocab = parserModestVocab_;
}

{ //Additional Methods
	protected String errorMessage = "";
	protected boolean valid = true;
	
	public boolean OK() {
		return valid;
	}
	
	public String error() {
		return errorMessage;
	}
}

start returns [Process root = new Process("root",0,0)]
	{ 
		ArrayList<Model> decl;
	}
:
	( #( "defaults" ( "=" IDENTIFIER IDENTIFIER ) * ) )?
	(	/*#( PACKAGE_AST pkg_name:IDENTIFIER description )
	|	*/decl = description 
	{  
		root.addModels(decl);
	}
	)
;
exception
	catch [RecognitionException ex]
	{
		errorMessage = ex.getMessage();
		this.valid = false;
	}
	
description returns [ArrayList<Model> decl = new ArrayList<Model>()]
{
	ArrayList<Model> stmnt;	
}
:
	( stmnt = declarative_stmnt { decl.addAll(stmnt); }) *
	seq_process
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#description_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}
	
declarative_stmnt returns [ArrayList<Model> stmnt = new ArrayList<Model>()]
{
	ArrayList<Model> procChildren = new ArrayList<Model>();	
	Process proc = null;
}
:
	/*#( FUNCTION_DEFINITION_AST param_definition ( description )? )
|*/	#( p:PROCESS_DEFINITION_AST param_definition ( procChildren=description )? ) 
	{ proc = new Process(p.getProcName(), p.getLine(), p.getColumn()); 
	proc.addModels(procChildren);
	stmnt.add(proc);
	}
|	ty:TYPE_DEFINITION_AST { stmnt.add(new ModestType(ty.getSymbolType(), ty.getLine(), ty.getColumn(), ty.getTypeKind())); }
|	#( VARIABLE_DECLARATION_AST
	  ( #(t:TYPE_AST (i:IDENTIFIER { if (t.getText().equals("action")) stmnt.add(new ModestAction(i.getText(), i.getLine(), i.getColumn()));
	  	else if (t.getText().equals("exception")) stmnt.add(new ModestException(i.getText(), i.getLine(), i.getColumn())); 
	  	else stmnt.add(new ModestVariable(i.getText(), i.getLine(), i.getColumn(), t.getText()));
	  })+ )
	  | #(e:"=" id2:IDENTIFIER expression
	  { stmnt.add(new ModestVariable(id2.getText(), id2.getLine(), id2.getColumn(), e.getSymbol().getType()));  }
	  )
	  )+
	 )
//|	#( CLASS_DEFINITION_AST class_name:IDENTIFIER description )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#declarative_stmnt_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}
	
param_definition:
	#(PARAMETER_LIST_AST ( VARIABLE_DECLARATION_AST )* )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#param_definition_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}
	
seq_process:
	( local_process )+
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#seq_process_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}

local_process:
(	/*"ERROR"
|*/	"STOP"
|	"break"/*
|	#( "return" expression )
*/|	#( "throw" IDENTIFIER )
|	#( "try"	description
	 	( #(CATCH_DEFINITION_AST IDENTIFIER description ) )+ )
|	#( "hide" expression seq_process )
|	#( "extend" expression seq_process )
/*|	#( "prefix" expression seq_process )*/
|	#( "relabel" ( #( "by" expression expression ) )/*+*/ seq_process )
|	#( GUARD_AST expression seq_process )
|	alt_stmnt
|	par_stmnt
|	do_stmnt
|  #(PROCESS_INSTANTIATION_AST (expression)* )
|	assignment
|	( #(IDENTIFIER "palt" ) ) => palt_stmnt
|  IDENTIFIER
)
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#local_process_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}

alt_stmnt:
	#( "alt" ( #(DCOLON seq_process ) )+ )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#alt_stmnt_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}

par_stmnt:
	#( "par" ( #(DCOLON seq_process ) )+ )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#par_stmnt_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}

do_stmnt:
	#( "do" ( #(DCOLON seq_process ) )+ )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#do_stmnt_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}
	
palt_stmnt:
	#( IDENTIFIER
		#( "palt"
		  (
			#(WEIGHT_AST ( #("{=" (assignment)+ ) )? ( seq_process )? )
		  )+
		 )
	 )

;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#palt_stmnt_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}

assignment:
	#( "=" expression expression )
|	#( "+=" expression expression )
|	#( "-=" expression expression )
|	#( "*=" expression expression )
|	#( "/=" expression expression )
|	#( "%=" expression expression )
|	#( ">>=" expression expression )
|	#( "<<=" expression expression )
|	#( "&=" expression expression )
|	#( "^=" expression expression )
|	#( "|=" expression expression )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#assignment_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}
	
	
expression:
(	(#("+" expression expression)) => #("+" expression expression)
|	(#("-" expression expression)) => #("-" expression expression)
|	#("*" expression expression)
|	#("/" expression expression)
|	#("%" expression expression)
// bitwise operators
|	#("&" expression expression)
|	#("|" expression expression)
|	#("^" expression expression)
|	#("<<" expression expression)
|	#(">>" expression expression)
// comparison operators..
|	#("<" expression expression)
|	#(">" expression expression)
|	#("<=" expression expression)
|	#(">=" expression expression)
|	#("==" expression expression)
|	#("!=" expression expression)
// logical operators
|	#("&&" expression expression)
|	#("||" expression expression)
// unary operators
|	#("+" expression)
|	#("-" expression)
|	#("!" expression)
|	#("#" expression)
//|	#(TYPE_CAST expression)
|  #("?" expression expression expression )
|	i:INT
|	f:FLOAT
|	tp:TYPE_AST
|  #(FUNCTION_CALL_AST (expression)* )
|  #("[" /* array */ expression /* index */ expression )
|  #("." /* struct */ expression /* field spec */ IDENTIFIER )
|  IDENTIFIER
|	set
)
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#expression_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}
	
range:
	#( ".." expression expression )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#range_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}

set:
	#("{" ( set_element )+ )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#set_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}

set_element:
	INT
|	IDENTIFIER
|	TYPE_AST
|	range
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#set_element_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}
