/* *************************************************************************
* Here begins the Simulation Tree Walker!
***************************************************************************/


header {
package modesteditor.core.antlrparser;

import modesteditor.core.ModestEditor;
import modesteditor.core.stepsimulation.*;
    
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream; 
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.jface.text.IDocument; 
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.Region;

import antlr.CommonAST;  
import antlr.RecognitionException;
}

class ModestSimulationTreeWalker extends TreeParser;
options {
	ASTLabelType = "ModestAST";
	buildAST = false;
	importVocab = parserModestVocab_;
	
}

{ //Additional Methods
	protected String errorMessage = "no error message";
	protected boolean valid = true;
	protected HashMap<String, ModestAST> processMap = new HashMap<String, ModestAST>();
	//private ModestSimulationTreeWalker bodyWalker = new ModestSimulationTreeWalker();
	
	private String processName = null;
	private int line = 0;
	private int column = 0;
	
	public boolean OK() {
		return valid;
	}
	
	public String error() {
		return errorMessage;
	}
	
	public void setProcessMap(HashMap<String, ModestAST> processMap){
		this.processMap = processMap;
	}
	
	public ModestSimulationTreeWalker(String processName, int line, int column) {
		this();
		this.processName = processName;
		this.line = line;
		this.column = column;
	}
}

start returns [SimulationNode start = new SequentialProcess(0,0,null)]
:
	( #( "defaults" ( "=" IDENTIFIER IDENTIFIER ) * ) )?
	(	/*#( PACKAGE_AST pkg_name:IDENTIFIER description )
	|	*/start = description[null]
	)
;
exception
	catch [RecognitionException ex]
	{
		errorMessage = "start " + ex.getMessage();
		this.valid = false;
	}
	
description[SimulationNode parent] returns [SimulationNode des = null]// = new SequentialProcess(0,0,parent)]
:
	( declarative_stmnt ) *
	des = seq_process [ parent, "" ]
	{ if (processName != null) ((SequentialProcess)des).setHighlightInfos(processName, line, column); }
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#description_in);*/

		errorMessage = "description " + ex.getMessage();
		this.valid = false;
	}
	
declarative_stmnt
{
	Symbol exp = new Symbol("","");
}
:
	/*#( FUNCTION_DEFINITION_AST param_definition ( description )? )
|*/	#( p:PROCESS_DEFINITION_AST param_definition ( d:description[null] { processMap.put(p.getProcName(), d); })? )
|	TYPE_DEFINITION_AST
|	#( VARIABLE_DECLARATION_AST
	  ( #(TYPE_AST (IDENTIFIER)+ )
	  | #("=" IDENTIFIER exp=expression)
	  )+
	 )
//|	#( CLASS_DEFINITION_AST class_name:IDENTIFIER description )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#declarative_stmnt_in);*/

		errorMessage = "declarative_stmnt " + ex.getMessage();
		this.valid = false;
	}
	
param_definition:
	#(PARAMETER_LIST_AST ( VARIABLE_DECLARATION_AST )* )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#param_definition_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}
	
seq_process [SimulationNode parent, String guard] returns [SimulationNode proc = new SequentialProcess(0,0,parent)]
//{ System.err.println("in SimTW seq_process");}
:
	local_process[proc, guard] ( local_process[proc, ""] )*
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#seq_process_in);*/

		errorMessage = "seq_process " + ex.getMessage();
		this.valid = false;
	}

local_process [SimulationNode parent, String guard]
{
	//{ System.err.println("in SimTW local_process");}
	SimulationNode proc = null;
	Symbol exp = new Symbol("","");
	SimulationNode catchTmp;
	ArrayList<String> labelExtension = new ArrayList<String>();
	ArrayList<String> relabel = new ArrayList<String>();
	ArrayList<String> labelby = new ArrayList<String>();
}
:
(	/*"ERROR"
|*/	s:"STOP" { proc = new ActionNode("stop", s.getLine(), s.getColumn(), parent, guard); }
|	b:"break" { proc = new BreakNode(b.getLine(), b.getColumn(), parent, guard); }/*
|	#( "return" expression )
*/|	#( th:"throw" iden:IDENTIFIER { proc = new ThrowNode(th.getLine(), th.getColumn(), iden.getText(), parent, guard); })
|	#( t:"try" { proc = new TryNode(t.getLine(), t.getColumn(), parent, guard); }	description[proc]
	 	( #(CATCH_DEFINITION_AST ide:IDENTIFIER catchTmp=description[null] { ((TryNode)proc).addCatch(catchTmp, ide.getText()); }) )+ )
|	#( "hide" /*expression*/ 
		(idhi:IDENTIFIER { labelExtension.add(idhi.getText()); }
		| labelExtension=set )proc=seq_process[parent, guard] )
		{ ((SequentialProcess)proc).hideAlphabet(labelExtension); }
|	#( "extend" /*expression*/ 
		(idex:IDENTIFIER { labelExtension.add(idex.getText()); }
		| labelExtension=set )  proc=seq_process[parent, guard] )
	 { ((SequentialProcess)proc).extendAlphabet(labelExtension); }
/*|	#( "prefix" expression seq_process )*/
|	#( "relabel" ( #( "by" /*expression expression*/ 
		(idre:IDENTIFIER { relabel.add(idre.getText()); } | relabel=set ) 
		(idby:IDENTIFIER { labelby.add(idby.getText()); } | labelby=set ) ) )/*+*/ 
			proc=seq_process[parent, guard] )
	{ if (relabel.size() != labelby.size()) {
		errorMessage = "size of relabel statements doesn't match";
		this.valid = false;
	  } else		
		((SequentialProcess)proc).addRelabeling(relabel, labelby); }
|	#( GUARD_AST exp=expression seq_process[parent, exp.getName()] )
|	alt_stmnt[parent, guard]
|	par_stmnt[parent, guard] 
|	do_stmnt[parent, guard] 
|  #(p:PROCESS_INSTANTIATION_AST (expression)* ) 
	{ 	
		ModestSimulationTreeWalker bodyWalker = new ModestSimulationTreeWalker(p.getText(), p.getLine(), p.getColumn());
		bodyWalker.setProcessMap(processMap);
		bodyWalker.description(processMap.get(p.getText()), parent); 
	}
|	assignment[parent, guard]
|	( #(id:IDENTIFIER "palt" ) ) => palt_stmnt[parent, guard]
|  i:IDENTIFIER { proc = new ActionNode(i.getText(), i.getLine(), i.getColumn(), parent, guard); }
)
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#local_process_in);*/

		errorMessage = "local_process " + ex.getMessage();
		this.valid = false;
	}

alt_stmnt[SimulationNode parent, String guard]
{
	SimulationNode alt = null;
	SimulationNode alternative = null;
	SimulationNode s;
}
:
	#( a:"alt" { alt = new AltNode(a.getLine(), a.getColumn(), parent); }
		( #(d:DCOLON { alternative = new Alternative(d.getLine(), d.getColumn(), alt); } 
				s = seq_process[alternative, guard] ) )+ )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#alt_stmnt_in);*/

		errorMessage = "alt_stmnt " + ex.getMessage();
		this.valid = false;
	}

par_stmnt[SimulationNode parent, String guard]
{
	SimulationNode par = null;
	SimulationNode alternative = null;
	SimulationNode s;
}
:
	#( p:"par" { par = new ParNode(p.getLine(), p.getColumn(), parent); }
		( #(d:DCOLON { alternative = new Alternative(d.getLine(), d.getColumn(), par); } 
				s = seq_process[alternative, guard] ) )+ )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#par_stmnt_in);*/

		errorMessage = "par_stmnt " + ex.getMessage();
		this.valid = false;
	}

do_stmnt[SimulationNode parent, String guard]
{
	SimulationNode doNode = null;
	SimulationNode alternative = null;
	SimulationNode s;
}
:
	#( dn:"do" { doNode = new DoNode(dn.getLine(), dn.getColumn(), parent); }
		( #(d:DCOLON { alternative = new Alternative(d.getLine(), d.getColumn(), doNode); } 
					s = seq_process[alternative, guard] ) )+ )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#do_stmnt_in);*/

		errorMessage = "do_stmnt " + ex.getMessage();
		this.valid = false;
	}
	
	
palt_stmnt[SimulationNode parent, String guard]
{
	ActionNode act = null;
	PaltNode palt = null;
	Alternative alt = null;
	boolean altExists = false;
	Alternative pseudoAlt = null;
	Symbol assi = null;
	String assignmentText = "";
}
:
	#( i:IDENTIFIER 
		#( p:"palt" { palt = new PaltNode(p.getLine(), p.getColumn(), parent); 
					  act = new ActionNode(i.getText(), i.getLine(), i.getColumn(), palt, guard);
					  altExists=false;
					}
		  (
			#(w:WEIGHT_AST 
				( #(a:ASSIGN_START (assi=assignment[alt, ""] { if (assignmentText != "") assignmentText += ", " + assi.getName(); else assignmentText += assi.getName(); })+ ) )? 
					( { altExists=true; alt = new Alternative(w.getLine(), w.getColumn(), palt, w.getWeight()); if (assi != null) alt.setAssignment(assignmentText); assi = null; assignmentText =""; } //Double.valueOf(string) 
																seq_process[alt, ""] )? 
					{ if (!altExists && assi!=null) //{act.setAssignment(assignmentText); assi = null;} }
						{ pseudoAlt = new Alternative(w.getLine(), w.getColumn(), palt, w.getWeight()); 
						  if (assi != null) pseudoAlt.setAssignment(assignmentText); 
						  assi = null; 
						  assignmentText = "";
						}
					}
					) // Put the alt somewhere else if we do something with the assignment
		  )+
		 )
	 )

;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#palt_stmnt_in);*/

		errorMessage = "palt_stmnt " + ex.getMessage();
		this.valid = false;
	}

assignment[SimulationNode parent, String guard] returns [Symbol assi = new Symbol("","")]
{
	Symbol lhs;
	Symbol rhs;
}
:
	#( "=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" = "+rhs.getName(), lhs.getType()); })
|	#( "+=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" += "+rhs.getName(), lhs.getType()); })
|	#( "-=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" -= "+rhs.getName(), lhs.getType()); })
|	#( "*=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" *= "+rhs.getName(), lhs.getType()); })
|	#( "/=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" /= "+rhs.getName(), lhs.getType()); })
|	#( "%=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" %= "+rhs.getName(), lhs.getType()); })
|	#( ">>=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" >>= "+rhs.getName(), lhs.getType()); })
|	#( "<<=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" <<= "+rhs.getName(), lhs.getType()); })
|	#( "&=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" &= "+rhs.getName(), lhs.getType()); })
|	#( "^=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" ^= "+rhs.getName(), lhs.getType()); })
|	#( "|=" lhs=expression rhs=expression { assi = new Symbol(lhs.getName()+" |= "+rhs.getName(), lhs.getType()); })
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#assignment_in);*/

		errorMessage = "assignment " + ex.getMessage();
		this.valid = false;
	}	


expression returns [Symbol exp = new Symbol("","")]
{
	Symbol e1 = new Symbol("","");
	Symbol e2 = new Symbol("","");
	Symbol e3 = new Symbol("","");
}
:
(	(#("+" expression expression)) => #("+" e1=expression e2=expression { exp = new Symbol(e1.getName()+" + "+e2.getName(),e1.getType()); })
|	(#("-" expression expression)) => #("-" e1=expression e2=expression { exp = new Symbol(e1.getName()+" - "+e2.getName(),e1.getType()); })
|	#("*" e1=expression e2=expression { exp = new Symbol(e1.getName()+" * "+e2.getName(),e1.getType()); })
|	#("/" e1=expression e2=expression { exp = new Symbol(e1.getName()+" / "+e2.getName(),e1.getType()); })
|	#("%" e1=expression e2=expression { exp = new Symbol(e1.getName()+" % "+e2.getName(),e1.getType()); })
// bitwise operators
|	#("&" e1=expression e2=expression { exp = new Symbol(e1.getName()+" & "+e2.getName(),e1.getType()); })
|	#("|" e1=expression e2=expression { exp = new Symbol(e1.getName()+" | "+e2.getName(),e1.getType()); })
|	#("^" e1=expression e2=expression { exp = new Symbol(e1.getName()+" ^ "+e2.getName(),e1.getType()); })
|	#("<<" e1=expression e2=expression { exp = new Symbol(e1.getName()+" << "+e2.getName(),e1.getType()); })
|	#(">>" e1=expression e2=expression { exp = new Symbol(e1.getName()+" >> "+e2.getName(),e1.getType()); })
// comparison operators..
|	#("<" e1=expression e2=expression { exp = new Symbol(e1.getName()+" < "+e2.getName(),e1.getType()); })
|	#(">" e1=expression e2=expression { exp = new Symbol(e1.getName()+" > "+e2.getName(),e1.getType()); })
|	#("<=" e1=expression e2=expression { exp = new Symbol(e1.getName()+" <= "+e2.getName(),e1.getType()); })
|	#(">=" e1=expression e2=expression { exp = new Symbol(e1.getName()+" >= "+e2.getName(),e1.getType()); })
|	#("==" e1=expression e2=expression { exp = new Symbol(e1.getName()+" == "+e2.getName(),e1.getType()); })
|	#("!=" e1=expression e2=expression { exp = new Symbol(e1.getName()+" != "+e2.getName(),e1.getType()); })
// logical operators
|	#("&&" e1=expression e2=expression { exp = new Symbol(e1.getName()+" && "+e2.getName(),e1.getType()); })
|	#("||" e1=expression e2=expression { exp = new Symbol(e1.getName()+" || "+e2.getName(),e1.getType()); })
// unary operators
|	#("+" e1=expression { exp = new Symbol(" +"+e1.getName(),e1.getType()); })
|	#("-" e1=expression { exp = new Symbol(" -"+e1.getName(),e1.getType()); })
|	#("!" e1=expression { exp = new Symbol(" !"+e1.getName(),e1.getType()); })
|	#("#" e1=expression { exp = new Symbol(" #"+e1.getName(),e1.getType()); })
//|	#(TYPE_CAST expression)
|  #("?" e1=expression e2=expression e3=expression { exp = new Symbol(e1.getName()+" ? "+e2.getName()+" : "+e3.getName(), e2.getType()); })
|	i:INT { exp = new Symbol(i.getText(), "int"); }
|	f:FLOAT { exp = new Symbol(f.getText(),"float"); }
|	tp:TYPE_AST { exp = new Symbol(tp.getText(), tp.getText()); }
|  #(fc:FUNCTION_CALL_AST (expression)* ) { exp = new Symbol(fc.getText(), "float"); }
|  #("[" /* array */ e1=expression /* index */ e2=expression { exp = new Symbol(e1.getName()+"["+e2.getName()+"]",e1.getType()); })
|  #("." /* struct */ e1=expression /* field spec */ ide:IDENTIFIER { exp = new Symbol(e1.getName()+"."+ide.getText(),e1.getType()); })
|  id:IDENTIFIER { exp = new Symbol(id.getText(),""); }
|	set
)
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#expression_in);*/

		errorMessage = "expression " + ex.getMessage();
		this.valid = false;
	}
	
range:
	#( ".." expression expression )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#range_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}

set returns [ArrayList<String> elems = new ArrayList<String>()]
	{ String el; }
:
	#("{" ( el=set_element { elems.add(el); } )+ )
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#set_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}

set_element returns [String el = ""]:
	i:INT { el = i.getText(); }
|	id:IDENTIFIER { el = id.getText(); }
|	TYPE_AST
|	range
;
exception
	catch [RecognitionException ex]
	{
		/*print_tree<RefModest_AST,BaseTreeParser> pr(*this);
		pr.pr_tree(#set_element_in);*/

		errorMessage = ex.getMessage();
		this.valid = false;
	}