/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.modest;

import modesteditor.core.util.ModestTextStyleProvider;
import modesteditor.core.util.ModestWhitespaceDetector;
import modesteditor.core.util.ModestWordDetector;

import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;

import java.util.ArrayList;
import java.util.List;

/**
 * A MoDeST code scanner
 */
public class ModestCodeScanner extends RuleBasedScanner {
	
	/** All keywords defined in the MoDeST specification */
	private static String[] fgKeywords = { 
		"alt", 
		"break",
		"by",
		"catch", 
		"const",
		"do", 
		"extend",
		"extern",
		"hide",
		"impatient",
		"palt", 
		"par", 
		"patient",
		"prefix",
		"try",
		"throw",
		"when", 
		"process", 
		"range", 
		"relabel",
		"return",
		"set", 
		"struct",
		"typedef",
		"urgent" };
	
	/** Predefined MoDeST types. */
	private static String[] fgTypes = {
		"int",
		"float",
		"bool",
		"action",
		"clock",
		"exception"
	};
	
	private static String[] fgConstants = { "false", "true" };
	
	/** Predefined ditributions. */
	private static String[] fgDistributions =  { "Exponential", "Normal", "Uniform"	};
	
	/**
	 * Constructor for the code scanner which gets a provider for the TextStyle 
	 * of the tokens. Sets the rules for the different tokens.
	 * @param provider
	 */
	public ModestCodeScanner(ModestTextStyleProvider provider) {
		IToken keyword= new Token(provider.getKeywordStyle());
		IToken type= new Token(provider.getTypeStyle());
		IToken dist= new Token(provider.getDistStyle());
		IToken comment= new Token(provider.getSingleLineCommentStyle());
		IToken other= new Token(provider.getDefaultStyle());

		List rules= new ArrayList();

		// Add rule for single line comments.
		rules.add(new EndOfLineRule("//", comment)); 

		// Add rule for strings. 
		//rules.add(new SingleLineRule("\"", "\"", string, '\\')); 
		//rules.add(new SingleLineRule("'", "'", string, '\\')); 

		// Add generic whitespace rule.
		rules.add(new WhitespaceRule(new ModestWhitespaceDetector()));

		// Add word rule for keywords, types, constants and distributions.
		WordRule wordRule= new WordRule(new ModestWordDetector(), other);
		for (int i= 0; i < fgKeywords.length; i++)
			wordRule.addWord(fgKeywords[i], keyword);
		for (int i= 0; i < fgTypes.length; i++)
			wordRule.addWord(fgTypes[i], type);
		for (int i= 0; i < fgConstants.length; i++)
			wordRule.addWord(fgConstants[i], type);
		for (int i= 0; i < fgDistributions.length; i++)
			wordRule.addWord(fgDistributions[i], dist);
		rules.add(wordRule);

		IRule[] result= new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}
}
