/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.preferences;

import modesteditor.core.ModestEditorPlugin;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;



/**
 * The main MoDeST page in the Preferences window. It is possible to set the different
 * colors for the syntax highlighting. 
 */
public class ModestEditorPreferencePage extends FieldEditorPreferencePage
		implements IWorkbenchPreferencePage {

	/** Creates an Modest preference page and sets it to save preferences in
	 * the ModestEditor plug-in's preference store. */
	public ModestEditorPreferencePage () {
		super(GRID);
		setPreferenceStore(ModestEditorPlugin.getDefault().getPreferenceStore());
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	/** Creates the page's fields. Since this class extends
	 * FieldEditorPreferencePage, all we have to do is create the field
	 * editors for all the required fields, and everything else will take
	 * care of itself. */
	protected void createFieldEditors() {
		addField(new ColorFieldEditor(ModestEditorPlugin.MULTI_LINE_COMMENT, "Multi line comment colour:", getFieldEditorParent()));
		addField(new ColorFieldEditor(ModestEditorPlugin.SINGLE_LINE_COMMENT, "Single line comment colour:", getFieldEditorParent()));
		addField(new ColorFieldEditor(ModestEditorPlugin.KEYWORD, "Keyword colour:", getFieldEditorParent()));
		addField(new ColorFieldEditor(ModestEditorPlugin.TYPE, "Type definition colour:", getFieldEditorParent()));
		addField(new ColorFieldEditor(ModestEditorPlugin.DIST, "Distribution colour:", getFieldEditorParent()));
		addField(new BooleanFieldEditor(ModestEditorPlugin.MODEST_MARK_ERRORS, "Mark syntax errors in the editor", getFieldEditorParent()));
		//addField(new FileFieldEditor(ModestEditorPlugin.COMPILER_LOCATION, "Path to the momodest binary:", getFieldEditorParent()));
		//addField(new DirectoryFieldEditor(ModestEditorPlugin.OUTPUT_FOLDER_LOCATION, "Path to the output folder:", getFieldEditorParent()));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	/** Nothing to initialize. */
	public void init(IWorkbench workbench) {
	}

}
