/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import java.util.ArrayList;
import java.util.List;

/**
 * The class representing an action in the MoDeST code.
 */
public class ActionNode extends SimulationNode {

	/** Name of the referenced action */
	private String name;
	
	/** Textual representation of an assignment following the action */
	private String assignment;
		
	/**
	 * Constructor that sets the parent node and the name
	 * @param name
	 * @param line
	 * @param column
	 * @param parent
	 */
	public ActionNode(String name, int line, int column, SimulationNode parent, String guard) {
		super(line,column);
		setParent(parent);
		this.name = name;
		this.guard = guard;
		assignment = "";
		length = name.length();
		parent.addChild(this);
	}
	
	
	
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	@Override
	public List<SimulationNode> collectTransitions() { 
		if (isActive()){ 
			ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>();
			trans.add(this);
			return ((List<SimulationNode>) trans);			
		} else return null;
	}
	
	@Override
	public List<ParTransition> collectParTransitions(){
		ArrayList<ParTransition> trans = new ArrayList<ParTransition>();
		if (isActive()){
			ParTransition parTrans = new ParTransition(parent, name);
			parTrans.addTransition(this);
			trans.add(parTrans);
		}
		return trans;
	}

	/**
	 * Puts the name of the action in the alphabet if it's not a STOP or a tau action.
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	@Override
	public void generateAlphabet() {
		if (alphabet!=null && alphabet.isEmpty() 
				&& !name.equals("tau") && !name.equals("stop")){
			alphabet.add(name);
			//System.err.println("Alphabet of ActionNode should add: " + name);
		}
	}
	
	@Override
	public void setActive() {
		super.setActive();
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void takeTransition(){
		//System.out.println("Action taken: " + name);
		if (isActive()){
			parent.notificationOfChildActivity(this);
			if (name == "stop")
				stopCalled();
			else isFinished();			
		}
	}
	
	@Override
	public void isFinished() {
		super.isFinished();
		parent.childIsFinished(this);
	}
	
	/**
	 * Never gets called since an action has no children.
	 */
	public void childIsFinished(SimulationNode child) {}
	
	@Override
	public void releaseTransitions() {
		isActive=false;
	}

	@Override
	public void notificationOfChildActivity(SimulationNode child) {}
	
	@Override
	public String printTransition() {
		/*if (parent instanceof SequentialProcess || parent instanceof PaltNode)
			return parent.printTransition();
		else return name;*/
		String label = name;
		if (!newLabel.equals("")) label = newLabel; 
		String toReturn = "["+guard+"] "+label;		
		if (!assignment.equals("")) toReturn += "\nAssignment(s): "+ assignment +"\n";
		return toReturn;
	}

	/**
	 * @return the name of the action
	 */
	public String getName() {
		return name;
	}



	/**
	 * @return the assignment in the case of an assignment block without a palt.
	 */
	public String getAssignment() {
		return assignment;
	}



	/**
	 * @param assignment the assignment to set
	 */
	public void setAssignment(String assignment) {
		this.assignment = assignment;
	}
	
	
	
	/**
	 * Method hides the action for printing.
	 */
	/*public void hide() {
		hidden = true;
	}*/
}
