/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import java.util.ArrayList;
import java.util.List;

/**
 * Class representing a break statement in a do loop.
 */
public class BreakNode extends SimulationNode {

	
	/** Textual representation of an assignment following the action */
	private String assignment;
	
	/** Flag to know whether to change the label in a tau or not */
	private boolean underADo;
	
	/**
	 * Default constructor.
	 * @param line
	 * @param column
	 * @param parent
	 * @param guard
	 */
	 public BreakNode(int line, int column, SimulationNode parent, String guard) {
		super(line, column);
		setParent(parent);
		this.guard = guard;
		parent.addChild(this);
		length = 5;
		assignment = "";
		underADo = false;
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void childIsFinished(SimulationNode child) {}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		if (isActive()){ 
			ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>();
			trans.add(this);
			return ((List<SimulationNode>) trans);			
		} else return null;
	}

	@Override
	public List<ParTransition> collectParTransitions(){
		ArrayList<ParTransition> trans = new ArrayList<ParTransition>();
		if (isActive()){
			ParTransition parTrans = new ParTransition(parent, "break");
			parTrans.addBreakTransition(this);
			trans.add(parTrans);
		}
		return trans;
	}
	
	/**
	 * Method that signals that the break in the transition 
	 * is in a do loop
	 */
	public void inDoSubtree(){
		underADo = true;
	}
	
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	/**
	 * Nothing to do because break is never in the common alphabet
	 */
	@Override
	public void generateAlphabet() {}

	/**
	 * Method calls the breakTransitionTaken method of the parent, to notify the parent of the occured break.
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition(java.util.List)
	 */
	@Override
	public void takeTransition() {
		if (isActive()){
			parent.notificationOfChildActivity(this);
			parent.breakTransitionTaken();
			isFinished();
		}
	}

	@Override
	public void isFinished() {
		super.isFinished();
		//parent.childIsFinished(this);
	}
	
	@Override
	public void releaseTransitions() {
		isActive=false;
	}

	/**
	 * Nothing happens, because a BreakNode is a leaf of the tree, so the method never gets called.
	 */
	@Override
	public void notificationOfChildActivity(SimulationNode child) {}
	
	@Override
	public String printTransition() {
		String label = underADo ? "tau" : "break";
		String toReturn = "["+guard+"] "+label;
			if (!assignment.equals("")) toReturn += "\nAssignment(s): "+ assignment +"\n";
		return toReturn;
	}
	
	/**
	 * @return break as the name of the Action (mainly used for printing puposes).
	 */
	public String getName() {
		return "break";
	}

	/**
	 * @return the assignment
	 */
	public String getAssignment() {
		return assignment;
	}

	/**
	 * @param assignment the assignment to set
	 */
	public void setAssignment(String assignment) {
		this.assignment = assignment;
	}
	
}
