/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;


/**
 * Class representing an do loop statement in the step simulation tree.
 *
 */
public class DoNode extends AltNode {

	
	/**
	 * Default constructor
	 * @param line
	 * @param column
	 * @param parent
	 */
	public DoNode(int line, int column, SimulationNode parent) {
		super(line, column, parent);
		length = 2;
	}

		
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	/**
	 * Method ensures that if a child is finished, all other children are set active again.
	 * @param child
	 */
	@Override
	public void childIsFinished(SimulationNode child) {
		if (child != activeChild) 
			System.err.println("Wrong argument to childIsFinished for DoNode: " + this.toString());
		else 
		{
			activeChild = null;
			setActive();
		}
	}

	
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition(java.util.List)
	 */
	// Keep synced with AltNode
	@Override
	public void takeTransition() {
		System.err.println("Cannot take transition in DoNode: "+this.toString());
	}

	/**
	 * If a break is taken in the scope of the do statement, all other statements 
	 * in the scope are released and the node is completed.
	 */
	@Override
	public void breakTransitionTaken() {
		releaseTransitions();
		isFinished();
	}
	
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	/**
	 * Method collects all children if not a child is already active
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		if (isActive()){
			
			
				ArrayList<SimulationNode> trans = new ArrayList<SimulationNode>();
				/*for (Iterator iter = children.iterator(); iter.hasNext();) {
					SimulationNode child = (SimulationNode) iter.next();
					trans.addAll(child.collectTransitions());
				}*/
				if (activeChild!=null)
					trans.addAll(activeChild.collectTransitions());
				else 
					for (int i = 0; i < children.size(); i++) {
						trans.addAll(children.get(i).collectTransitions());
					}
				for (int i = 0; i < trans.size(); i++) {
					SimulationNode actTrans = trans.get(i);
					if (actTrans instanceof BreakNode) 
						((BreakNode)actTrans).inDoSubtree();
				}
				return trans;
		}	
		else 
			return null;
	}

	
	@Override
	public List<ParTransition> collectParTransitions() {
		ArrayList<ParTransition> trans = new ArrayList<ParTransition>();
		if (isActive()){
			if (activeChild!=null){
				List<ParTransition> tmpTrans = activeChild.collectParTransitions();
				HashSet<String> containedLetters = new HashSet<String>();
				for (int i = 0; i < tmpTrans.size(); i++) {
					ParTransition actParTrans = tmpTrans.get(i);
					trans.add(actParTrans);
					String repLetter = actParTrans.getRepresentingLetter();
					if (repLetter!="break" && repLetter!="tau" && repLetter!="stop") 
						containedLetters.add(repLetter);
					else if (repLetter.equals("break")) actParTrans.inDoSubtree(); 
				}
				HashSet<String> myAlphabet = new HashSet<String>();
				myAlphabet.addAll((ArrayList<String>)getAlphabet().clone());
				for (Iterator iter = myAlphabet.iterator(); iter.hasNext();) {
					String element = (String) iter.next();
					/* TODO */
					// Don't know if this works since the transitions list is empty
					if (!containedLetters.contains(element))
						trans.add(new ParTransition(this.parent, element));
				}
			} else {
				for (int i = 0; i < children.size(); i++) {
					trans.addAll(children.get(i).collectParTransitions());
					/*if (!children.get(i).collectParTransitions().isEmpty())
						System.err.println("ParTrans from " +this.toString() + " " 
								+children.get(i).collectParTransitions().get(0).printTransition());*/
				}
			}
		}
		return trans;
	}
	
}
