/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.text.IDocument;

import java.util.ArrayList;
import java.util.List;

/**
 * This class represents a PaltTransition which is taken
 * after the action guarding the palt was taken.
 */
public class PaltTransition extends SimulationNode {

	private ArrayList<SimulationNode> pAlternatives;
	
	private double probability;

	/**
	 * Constructor for setting line and column parameters.
	 * @param line
	 * @param column
	 */
	public PaltTransition(int line, int column) {
		super(line, column);
		pAlternatives = new ArrayList<SimulationNode>();
		probability = 0;
	}
	
	/**
	 * Default constructor
	 * @param line
	 * @param column
	 * @param prob probability of the transition
	 * @param alt alternatives that can be taken in the transition
	 * (can be more than one in a parallel case).
	 */
	public PaltTransition(int line, int column, double prob, ArrayList<SimulationNode> alt) {
		this(line, column);
		probability = prob;
		pAlternatives.addAll(alt);
	}
	
	/**
	 * Constructor for the PaltNode
	 * @param line
	 * @param column
	 * @param prob probability of the transition
	 * @param alt first alternative that can be taken in the transition
	 */
	public PaltTransition(int line, int column, double prob, SimulationNode alt) {
		this(line, column);
		probability = prob;
		pAlternatives.add(alt);
	}
	
	/**
	 * Method for adding an alternative in a parallel setting.
	 * @param prob
	 * @param alt
	 */
	public void addAlternative(PaltTransition alt) {
		probability *= alt.probability;
		pAlternatives.addAll(alt.pAlternatives);
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void childIsFinished(SimulationNode child) {}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectParTransitions()
	 */
	@Override
	public List<ParTransition> collectParTransitions() {
		return null;
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		return null;
	}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	@Override
	public void generateAlphabet() {}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#notificationOfChildActivity(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void notificationOfChildActivity(SimulationNode child) {}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#releaseTransitions()
	 */
	@Override
	public void releaseTransitions() {}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition()
	 */
	@Override
	public void takeTransition() {
		for (int i = 0; i < pAlternatives.size(); i++) {
			pAlternatives.get(i).takeTransition();
		}
	}
	
	@Override
	public ArrayList<IMarker> createSimHighlighting(IResource resource, IDocument document, boolean isPalt) {
		ArrayList<IMarker> markers = new ArrayList<IMarker>();
		
		for (int i = 0; i < pAlternatives.size(); i++) {
			markers.addAll(pAlternatives.get(i).createSimHighlighting(resource, document, true));
		}
		
		return markers;
	}

	/**
	 * @return the pAlternatives
	 */
	public ArrayList<SimulationNode> getPAlternatives() {
		return pAlternatives;
	}

	/**
	 * @return the probability
	 */
	public double getProbability() {
		return probability;
	}
	
	@Override
	public String printTransition() {
		String toReturn = "";
		if (getProbability()!=1) toReturn = "Probability of the alternative " + String.valueOf(getProbability());
		String assignment = "";
		for (int i = 0; i < pAlternatives.size(); i++) {
			if (pAlternatives.get(i) instanceof Alternative) {
				if (assignment == "") 
					assignment = ((Alternative)pAlternatives.get(i)).getAssignment();
				else assignment += ", " + ((Alternative)pAlternatives.get(i)).getAssignment();
			}
			
		}
		if (assignment != "")
			toReturn += "\nAssignment(s): "+assignment;
		/*for (int i = 0; i < pAlternatives.size(); i++) {
			//toReturn += pAlternatives.get(i).printTransition();
			if (i>0) toReturn += "and ";
					
			toReturn += "line " + pAlternatives.get(i).getLine()+ " ";
		}*/
		return toReturn;
	}

}
