/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.text.IDocument;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * Class representing a set of parallel transitions.
 */
public class ParTransition extends SimulationNode {

	private String representingLetter;
	
	private ArrayList<SimulationNode> transitions;
	
	private ArrayList<PaltTransition> paltTransitions;
	
	private boolean enabled;
	
	private boolean noCommonParent;
	
	private boolean isThrowStatement;

	private String assignment;
	
	private boolean underADo;
	
	private boolean isCaught;
	
	/**
	 * Constructor which registers the parent and the representing letter
	 * @param parent
	 * @param letter
	 */
	/*public ParTransition(ParNode parent, String letter) {
		super();
		this.parent = parent;
		representingLetter = letter;
		enabled = true;
		transitions = new ArrayList<SimulationNode>();
		noCommonParent = false;
		isThrowStatement = false;
	}*/
	
	/**
	 * Constructor for the ParTransitions with only one transition per language construct.
	 * @param parent
	 * @param letter
	 * @param trans
	 */
	public ParTransition(ParNode parent, String letter, ArrayList<SimulationNode> trans) {
		this(parent, letter);
		noCommonParent = true;
		transitions.addAll(trans);
		for (int i = 0; i < trans.size(); i++) {
			SimulationNode actTrans = trans.get(i); 
			String transGuard = actTrans.getGuard();
			if (transGuard != "")
				guard += (guard=="") ? transGuard : " && "+ transGuard;
			if (actTrans instanceof ActionNode){
				String assi = ((ActionNode)actTrans).getAssignment();
				if (assi != "")
					assignment += (assignment=="") ? assi : ", " + assi;
			}
			if (actTrans instanceof BreakNode){
				String assi = ((BreakNode)actTrans).getAssignment();
				if (assi != "")
					assignment += (assignment=="") ? assi : ", " + assi;
			}
				
		}
		
	}
	
	/**
	 * Default constructor which registers the parent and the
	 * representing letter.
	 * @param parent
	 * @param letter
	 */
	public ParTransition(SimulationNode parent, String letter) {
		super();
		this.parent = parent;
		representingLetter = letter;
		enabled = true;
		transitions = new ArrayList<SimulationNode>();
		paltTransitions = new ArrayList<PaltTransition>();
		noCommonParent = false;
		assignment="";
		underADo = false;
		isCaught = false;
	}
	
	/**
	 * Adds a transition to the transition pool. If the actual letter of the 
	 * transition isn't the representing letter, the ParTransition is disabled.
	 * @param trans
	 */
	public void addTransition(SimulationNode trans) {
		
		if (!transitions.contains(trans)) {
			transitions.add(trans);
			guard += (guard=="") ? trans.getGuard() : " && "+ trans.getGuard();
			if (trans instanceof ActionNode){
				String assi = ((ActionNode)trans).getAssignment();
				if (assi != "")
					assignment += (assignment=="") ? assi : ", " + assi;
			}
			if (trans instanceof BreakNode){
				String assi = ((BreakNode)trans).getAssignment();
				if (assi != "")
					assignment += (assignment=="") ? assi : ", " + assi;
			}
			if (trans.getAlphabet().isEmpty() && !trans.getName().equals("tau") 
				&& !trans.getName().equals("stop") && !isThrowStatement) enabled=false;
			if (!trans.getAlphabet().isEmpty() && !representingLetter.equals("tau"))
				if (!trans.getAlphabet().get(0).equals(representingLetter)) 
					enabled=false;
			
		}
	}
	
	/**
	 * Adds a transition to the transition pool. If the transition is disabled 
	 * the ParTransition is disabled.
	 * @param trans
	 */
	public void addParTransition(ParTransition trans){
		if (trans.isEnabled()){
			List<SimulationNode> toAdd = trans.getTransitions();
			for (int i = 0; i < toAdd.size(); i++) {
				SimulationNode actTrans = toAdd.get(i);
				if (!transitions.contains(actTrans)){
					transitions.add(actTrans);
					guard += (guard=="") ? trans.getGuard() : " && "+ trans.getGuard();
					assignment += (assignment=="") ? trans.getAssignment() : ", " + trans.getAssignment();
				}
			}
		} else enabled=false;
	}
	
	/**
	 * Method adding a break Node to the parallel transition
	 * @param trans
	 */
	public void addBreakTransition(BreakNode trans){
		if (!transitions.contains(trans)){
			transitions.add(trans);
			guard += (guard=="") ? trans.getGuard() : " && "+ trans.getGuard();
			assignment += (assignment=="") ? trans.getAssignment() : ", " + trans.getAssignment();
		}
	}
	
	/**
	 * Is enabled exactly if not disabled earlier and the transition list is not empty.
	 * @return the enabledness
	 */
	public boolean isEnabled() {
		
		/*if(!enabled) System.err.println("disabled because of boolean");
		else if (getTransitions().isEmpty()) System.err.println("disabled because of empty trans list");
		else System.err.println("enabled because everything works");*/
		
		return (enabled && !getTransitions().isEmpty());
	}
	
	/**
	 * Method called to indicate that the transition is a thrown 
	 * exception and thus always enabled. 
	 *
	 */
	public void setThrowStatement() {
		isThrowStatement = true;
	}


	/**
	 * @return the letter of the action that is to be taken 
	 * parallely
	 */
	public String getRepresentingLetter() {
		return representingLetter;
	}
	
	/**
	 * Method relabels the transition in case of a relabeling.
	 * @param newLabel
	 */
	public void relabelBy(String newLabel){
		representingLetter = newLabel;
	}


	/**
	 * @return the transitions that are taken parallely
	 */
	public List<SimulationNode> getTransitions() {
		return transitions;
	}


	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#childIsFinished(modesteditor.core.stepsimulation.SimulationNode)
	 */
	@Override
	public void childIsFinished(SimulationNode child) {}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#collectTransitions()
	 */
	/**
	 * Not needed since the class is already a transition
	 */
	@Override
	public List<SimulationNode> collectTransitions() {
		return null;
	}

	/**
	 * Not needed since the class is already a transition
	 */
	@Override
	public List<ParTransition> collectParTransitions() {
		return null;
	}
	
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#generateAlphabet()
	 */
	@Override
	public void generateAlphabet() {}

	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#releaseTransitions()
	 */
	@Override
	public void releaseTransitions() {}

	/**
	 * If the transition is taken, this method ensures that all child transitions
	 * are taken parallely. 
	 */
	/* (non-Javadoc)
	 * @see modesteditor.core.stepsimulation.SimulationNode#takeTransition(java.util.List)
	 */
	@Override
	public void takeTransition() {
		for (Iterator iter = transitions.iterator(); iter.hasNext();) {
			SimulationNode trans = (SimulationNode) iter.next();
			if (trans instanceof Alternative){
				trans.takeTransition();
				List<SimulationNode> newTrans = trans.collectTransitions();
				newTrans.get(0).takeTransition();
			} else
				trans.takeTransition();
		}

	}

	@Override
	public void notificationOfChildActivity(SimulationNode child) {}
	
	@Override
	public String printTransition() {
		/*String toReturn = "";
		for (int i = 0; i < transitions.size(); i++) {
			SimulationNode trans = transitions.get(i);
			if (representingLetter == "tau") trans.hide();
			toReturn = toReturn + trans.printTransition();
		}
		return toReturn;*/
		String label = (underADo || isCaught)? "tau" : representingLetter;
		String toReturn = "["+guard+"] "+label;
		if (paltTransitions.size()==1) toReturn += " " + paltTransitions.get(0).printTransition();
		else if (!paltTransitions.isEmpty()) toReturn += " (probabilistic transition following)";
		if (!assignment.equals("")) toReturn += "\nAssignment(s): " + assignment;
		return toReturn; /** TODO do something with the hide ...  Maybe remove it ;D*/
	}
	
	@Override
	public ArrayList<IMarker> createSimHighlighting(IResource resource, IDocument document, boolean isPalt) {
		ArrayList<IMarker> markers = new ArrayList<IMarker>();
		
		for (int i = 0; i < transitions.size(); i++) {
			markers.addAll(transitions.get(i).createSimHighlighting(resource, document, isPalt));
		}
		
		return markers;
	}
	
	
	/**
	 * Adds a PaltTransition to the list of PaltTransitions
	 * @param trans transition to add
	 */
	public void addPaltTransition(PaltTransition trans) {
		paltTransitions.add(trans);
	}
	
	/**
	 * Adds a PaltTransition list to the list of PaltTransitions
	 * @param trans transitions to add
	 */
	public void addPaltTransitionList(ArrayList<PaltTransition> trans) {
		paltTransitions.addAll(trans);
	}
	
	
	
	/**
	 * @return the paltTransitions
	 */
	public ArrayList<PaltTransition> getPaltTransitions() {
		return paltTransitions;
	}
	
	

	/**
	 * @return the assignment
	 */
	public String getAssignment() {
		return assignment;
	}

	
	
	/**
	 * @param assignment the assignment to set
	 */
	public void setAssignment(String assignment) {
		this.assignment = assignment;
	}
	
	/**
	 * Method that signals that the break in the transition 
	 * is in a do loop
	 */
	public void inDoSubtree(){
		underADo = true;
	}
	
	/**
	 * Method that signals that the thrown exception will be 
	 * caught later on.
	 */
	public void willBeCaught() {
		isCaught = true;
	}

	public boolean isThrowStatement(){
		return isThrowStatement;
	}
	
}
