/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import modesteditor.core.ModestEditor;
import modesteditor.core.ModestEditorPlugin;
import modesteditor.core.antlrparser.ModestSimulationTreeWalker;
import modesteditor.core.model.ModestSimulation;
import modesteditor.core.modest.ModestEditorMessages;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.PopupDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.texteditor.TextEditorAction;

import antlr.collections.AST;

/**
 * TextEditorAction in the Action bar that starts the 
 * simulatin in the SimulationView.
 */
public class SimulationAction extends TextEditorAction {

	public SimulationAction() {
		super(ModestEditorMessages.getResourceBundle(), "Simulate.", null);
		this.setImageDescriptor(ModestEditorPlugin.getImageDescriptor("start.png"));
		this.setToolTipText("To start simulation click button, but first open Step Simulation view");
	}
	
	@Override
	public void run() {
		
		IEditorPart editor = getTextEditor();
		if (editor!=null){
			
			if (editor!=null && editor instanceof ModestEditor){
				ModestSimulation simulation = ((ModestEditor)editor).getSimulation();
				AST actAST = simulation.getCurrentAST();
				if (actAST==null) {
					String[] ar = {"OK"};
					MessageDialog finished = new MessageDialog(new Shell(), "Code is not valid", null, 
							"Cannot simulate, the code contains errors.", MessageDialog.NONE, ar, 0);
					finished.open();
					return;
				}
			
		
			
			try {
				ModestSimulationTreeWalker walker = new ModestSimulationTreeWalker();
				
				SequentialProcess proc = (SequentialProcess)walker.start(actAST);
				
				proc.setActive();
				SimulationRoot root = new SimulationRoot(proc); 
				SimulationView view = SimulationView.getView();
				if (view == null){
					String[] ar = {"OK"};
					MessageDialog finished = new MessageDialog(new Shell(), "No Step Simulation View", null, 
							"Open the Step Simulation view first", MessageDialog.NONE, ar, 0);
					finished.open();
				}
				else {
					//view.clearSimulationListeners(); replaced by reset.
					view.reset();
					view.addListener((ModestEditor)editor);
					root.removeEditorListeners();
					root.addListener((ModestEditor)editor);
					view.setRoot(root);
				}
			
			
			} catch (Exception e) {
				
					System.err.println("Error in SimulationAction: " + e.toString());
					e.printStackTrace();
				
			}
			}
		} else {
			System.err.println("got no editor");
		}
		
	}
	
	@Override
	public void update() {
		super.update();
		setEnabled(true);
	}
	

}
