/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import modesteditor.core.ModestEditorPlugin;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Label provider for the step simulation.
 *
 */
public class SimulationLabelProvider extends LabelProvider {//implements ILabelProvider {

	private Map imageCache;
	
	public SimulationLabelProvider() {
		imageCache = new HashMap(11);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
	 */
	public Image getImage(Object element) {
		ImageDescriptor descriptor = null;
		
		if (element instanceof PaltTransition) {
			PaltTransition paltTrans = (PaltTransition) element;
			double prob = paltTrans.getProbability();
			
			int finalProb = (new Double(prob * 100)).intValue();
			if (finalProb != 100)
				descriptor = ModestEditorPlugin.getImageDescriptor(finalProb+"percent.gif");
			
			/*if (prob < 0.05)
				descriptor = ModestEditorPlugin.getImageDescriptor("0percent.gif");
			else if (prob < 0.15)
				descriptor = ModestEditorPlugin.getImageDescriptor("10percent.gif");
			else if (prob < 0.25)
				descriptor = ModestEditorPlugin.getImageDescriptor("20percent.gif");
			else if (prob < 0.35)
				descriptor = ModestEditorPlugin.getImageDescriptor("30percent.gif");
			else if (prob < 0.45)
				descriptor = ModestEditorPlugin.getImageDescriptor("40percent.gif");
			else if (prob < 0.55)
				descriptor = ModestEditorPlugin.getImageDescriptor("50percent.gif");
			else if (prob < 0.65)
				descriptor = ModestEditorPlugin.getImageDescriptor("60percent.gif");
			else if (prob < 0.75)
				descriptor = ModestEditorPlugin.getImageDescriptor("70percent.gif");
			else if (prob < 0.85)
				descriptor = ModestEditorPlugin.getImageDescriptor("80percent.gif");
			else if (prob < 0.95)
				descriptor = ModestEditorPlugin.getImageDescriptor("90percent.gif");
			else 
				descriptor = ModestEditorPlugin.getImageDescriptor("100percent.gif");*/
		}
		
		if (descriptor==null) 
			   //descriptor = ModestEditorPlugin.getImageDescriptor("diamond_blue.gif");//
			return null;
		   //obtain the cached image corresponding to the descriptor
		Image image = (Image)imageCache.get(descriptor);
		if (image == null) {
			image = descriptor.createImage();
			imageCache.put(descriptor, image);
		}
		return image;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
	 */
	public String getText(Object element) {
		if (element instanceof SimulationNode)
			return ((SimulationNode)element).printTransition();
		else return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener(ILabelProviderListener listener) {}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose() {
		for (Iterator i = imageCache.values().iterator(); i.hasNext();) {
			((Image) i.next()).dispose();
		}
		imageCache.clear();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
	 */
	public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener(ILabelProviderListener listener) {}

}
