/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import modesteditor.core.ModestEditor;
import modesteditor.core.antlrparser.ModestSimulationTreeWalker;
import modesteditor.core.model.ModestSimulation;

import org.eclipse.ui.IEditorPart;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.List;

import antlr.RecognitionException;
import antlr.collections.AST;

/**
 * Thread for the step simulation, in order to stay non blocking.
 *
 */
public class SimulationThread extends Thread {


	private ModestSimulation simulation;

	private AST actAST;

	public SimulationThread(IEditorPart editor) {
		if (editor!=null && editor instanceof ModestEditor){
			simulation = ((ModestEditor)editor).getSimulation();
			actAST = simulation.getCurrentAST();
			if (actAST!=null) System.err.println("got AST");
			System.err.println("AST: " + actAST.toStringList());
		}
	}

	@Override
	public void run() {

		try {
			ModestSimulationTreeWalker walker = new ModestSimulationTreeWalker();
			System.err.println("after TreeWalker creation");
			SequentialProcess root = (SequentialProcess)walker.start(actAST);
			System.err.println("got root");
			root.setActive();

			if (!walker.OK()) System.err.println("Error in Walker: " + walker.error());
			else { 
				//System.err.println("walker run valid!");
				while (root.isActive()) {
					if (root.isActive()) System.out.println("root is active, simulating");
					List<SimulationNode> trans = root.collectTransitions(); 
					if (trans.size()==0) {
						System.out.println("Cannot do anything, aborting...");
						break;
					}
					if (trans!=null)
						System.err.println("Size of transitions: " + trans.size());
					else System.err.println("transitions are null");
					for (int i = 0; i < trans.size(); i++) {
						//System.out.println(i + " : " + trans.get(i).toString());
						System.out.print(i + ": ");
						trans.get(i).printTransition();
					}
					System.out.println("Choice: ");
					try {
						InputStreamReader stdin = new InputStreamReader(System.in);
						BufferedReader console = new BufferedReader(stdin);
						String choiceStr = console.readLine();
						int choice = Integer.parseInt(choiceStr);
						//System.out.println("Choice taken: " + choice);
						//Actually take the transition chosen
						if (choice < trans.size()) 
							trans.get(choice).takeTransition();
						else {
							System.err.println("Wrong choice, aborting");
							break;
						}

					} catch (Exception e) {
						System.err.println("could not read in SimulationThread run(): " + e.toString());
						e.printStackTrace();
					}
				}
			}
			//System.err.println("After simulation");
		} catch (RecognitionException e) {
			System.err.println("RecognitionException in SimulationTreeWalker.");
			e.printStackTrace();
		}
	}
}
