/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2007
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.stepsimulation;

import modesteditor.core.ModestEditorPlugin;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.part.ViewPart;

import java.util.ArrayList;



/**
 * View class where the available transitions of the step simulation are displayed.
 *
 */
public class SimulationView extends ViewPart implements ISelectionChangedListener, ISimulationListener{

	private /*ListViewer*/TreeViewer alternativeViewer;
	private SimulationRoot root;
	private static SimulationView simView;
	protected Action takeAlternative;
	protected Action back;
	protected Action forward;
	private Object selection;
	private ArrayList<ISimulationViewListener> listeners;
	private ArrayList<SimulationNode> choiceStack;
	private int choiceIndex;
	
	// Seems not to be needed
	public SimulationView(SimulationRoot root) {
		this.root=root;
		simView=this;
		selection = null;
		listeners = new ArrayList<ISimulationViewListener>();
		choiceStack = new ArrayList<SimulationNode>();
		choiceIndex = 0;
	}
	
	public SimulationView() {
		root = new SimulationRoot(new SequentialProcess());
		simView=this;
		selection = null;
		listeners = new ArrayList<ISimulationViewListener>();
		choiceStack = new ArrayList<SimulationNode>();
		choiceIndex = 0;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {		
		/* Create a grid layout object so the text and treeviewer
		 * are layed out the way I want. */
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		layout.verticalSpacing = 2;
		layout.marginWidth = 0;
		layout.marginHeight = 2;
		parent.setLayout(layout);
		
		
		// only alternativeViewer statements were there before the adding of the button.
		alternativeViewer = new TreeViewer(parent);//new ListViewer(parent);
		alternativeViewer.setContentProvider(new SimulationContentProvider());
		alternativeViewer.setLabelProvider(new SimulationLabelProvider());
		alternativeViewer.addSelectionChangedListener(this);
		alternativeViewer.setInput(root);
		// end of these statements
		
		GridData layoutData = new GridData();
		
		// layout the viewer 
		layoutData = new GridData();
		layoutData.grabExcessHorizontalSpace = true;
		layoutData.grabExcessVerticalSpace = true;
		layoutData.horizontalAlignment = GridData.FILL;
		layoutData.verticalAlignment = GridData.FILL;
		alternativeViewer.getControl().setLayoutData(layoutData);
		
		createActions();
		createToolbar(parent);
		
	}
	
	protected void createActions() {
		takeAlternative = new Action("Take the selected alternative") {
			public void run() {
				takeAlternative();
			}
		};
		
		back = new Action("", ModestEditorPlugin.getImageDescriptor("back.gif")) {
			public void run() {
				back();
			}
		};
		
		forward = new Action("", ModestEditorPlugin.getImageDescriptor("forward.gif")) {
			public void run() {
				forward();
			}
		};
	}
	
	protected void createToolbar(Composite parent) {
		IToolBarManager toolbarManager = getViewSite().getActionBars().getToolBarManager();
		//ToolBarManager myToolBar  = new ToolBarManager(new ToolBar(parent, SWT.LEFT));
		
		toolbarManager.add(takeAlternative);
		toolbarManager.add(back);
		toolbarManager.add(forward);
		/*myToolBar.add(takeAlternative);
		myToolBar.add(back);
		myToolBar.add(forward);*/
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		// TODO Auto-generated method stub

	}
	
	/**
	 * Method updates the selection.
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		selection = ((StructuredSelection)event.getSelection()).getFirstElement();
		//root.choiceTaken((SimulationNode)trans);
		for (int i = 0; i < listeners.size(); i++) {
			listeners.get(i).simulationViewChanged(selection);
		}
	}
	
	/**
	 * Method calls the choiceTaken method from the SimuluationRoot on 
	 * hand of the selection.
	 */
	public void takeAlternative() {
		if (selection == null){
			String[] ar = {"OK"};
			MessageDialog finished = new MessageDialog(new Shell(), "No selection", null, 
					"Select an alternative first", MessageDialog.NONE, ar, 0);
						
			finished.open();
		} else {
			int pos = choiceStack.size()-1;
	        while(pos >= choiceIndex)
	            choiceStack.remove(pos--);
	        choiceStack.add((SimulationNode)selection);
			choiceIndex++;
			root.choiceTaken((SimulationNode)selection, false);
			selection = null;
		}
	}
	
	public void back(){
		if (choiceIndex > 0){
			System.err.println("Back: Choice stack has size " + choiceStack.size() + " and index is " + choiceIndex);
			choiceIndex--;
			if (choiceIndex > 0)
				if (choiceStack.get(choiceIndex) instanceof PaltTransition) 
					choiceIndex--;
			root.clearState();
			int currentChoice = 0;
			while (currentChoice < choiceIndex) {
				System.err.println("Trans in back motion: " + choiceStack.get(currentChoice).printTransition());
				root.choiceTaken(choiceStack.get(currentChoice++), true);
			}
			update();
		} else {
			System.err.println("Back: Choice stack has size " + choiceStack.size() + " and index is " + choiceIndex);
			String[] ar = {"OK"};
			MessageDialog finished = new MessageDialog(new Shell(), "No choice left", null, 
					"Cannot got further back, already at the simulation start", MessageDialog.NONE, ar, 0);
			finished.open();
		}
	}
	
	public void forward(){
		if (choiceIndex < choiceStack.size()){
			System.err.println("Forward: Choice stack has size " + choiceStack.size() + " and index is " + choiceIndex);
			
			//System.err.println("Trans in forward motion: " + choiceStack.get(choiceIndex).printTransition());
			root.choiceTaken(choiceStack.get(choiceIndex++), true);
			
			if (choiceIndex < choiceStack.size()){
				if (choiceStack.get(choiceIndex) instanceof PaltTransition)
					root.choiceTaken(choiceStack.get(choiceIndex++), true);
				if (choiceStack.get(choiceIndex) instanceof ParTransition)
					if (!((ParTransition)choiceStack.get(choiceIndex)).getPaltTransitions().isEmpty()) 
						root.choiceTaken(choiceStack.get(choiceIndex++), true);
			}
			update();
		}
		else {
			System.err.println("Choice stack has size " + choiceStack.size() + " and index is " + choiceIndex);
			String[] ar = {"OK"};
			MessageDialog finished = new MessageDialog(new Shell(), "No choice left", null, 
					"Cannot got further forward, no choices left to repeat", MessageDialog.NONE, ar, 0);
			finished.open();
		}
	}
	
	/**
	 * Method gets called from the SimulationRoot after an update.
	 * It updates the View and checks if the simulation is finished 
	 * and thus opens a MessageDialog. 
	 */
	public void simulationChanged(SimulationRoot sim) {
/*		if (!getControl().isDisposed()) {
    		getControl().getDisplay().asyncExec(new Runnable () {
				public void run () {
					update();
				}
    		});
    	}*/
		
		update();
		if (root.isFinished()) {
			String[] ar = {"OK"};
			MessageDialog finished = new MessageDialog(new Shell(), "Simulation finished", null, 
					root.getFinishingMessage(), MessageDialog.NONE, ar, 0);
						
			finished.open();
		}
	}
	
	/** Refreshes the tree viewer when the simulation is updated. */
    private void update () {
		
		if (alternativeViewer != null) {
			Control control = alternativeViewer.getControl();
			if (control != null && !control.isDisposed()) {
		    	alternativeViewer.removeSelectionChangedListener(this);
				control.setRedraw(false);
				alternativeViewer.refresh();
				//viewer.expandToLevel(2);
				control.setRedraw(true);
		    	alternativeViewer.addSelectionChangedListener(this);
			}
		}
    }
	

    public static SimulationView getView() {
    	return simView; 
    }

	/**
	 * @param root the root to set
	 */
	public void setRoot(SimulationRoot root) {
		this.root = root;
		root.addListener(this);
		alternativeViewer.setInput(root);
		update();
		if (root.isFinished()) {
			String[] ar = {"OK"};
			MessageDialog finished = new MessageDialog(new Shell(), "Simulation finished", null, 
					root.getFinishingMessage(), MessageDialog.NONE, ar, 0);						
			finished.open();
			clearSimulationListeners();
		}
	}
    
	public void addListener(ISimulationViewListener listener) {
		listeners.add(listener);
	}
	
	public void removeListener(ISimulationViewListener listener) {
		if (listeners.contains(listener))
			listeners.remove(listener);
	}
	
	public void clearSimulationListeners() {
		listeners.clear();
	}
	
	public void reset(){
		clearSimulationListeners();
		choiceIndex = 0;
		choiceStack.clear();
	}
    
}
