/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */
package modesteditor.core.util;

import modesteditor.core.ModestEditorPlugin;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;

/**
 * A utility class to get colors and font styles for syntax highlighting. 
 */
public class ModestTextStyleProvider {

	private TextAttribute defaultStyle, distStyle, keywordStyle, typeStyle, multiLinecommentStyle, singleLinecommentStyle;
	
	/** Creates a new text style provider. Currently, a new provider is
	 * created for each MoDeST editor. */
	/* Colour preferences are only read in the constructor, which is why
	 * editors need to be closed and reopened after changing the colours. */
	public ModestTextStyleProvider () {
		IPreferenceStore store = ModestEditorPlugin.getDefault().getPreferenceStore();
		RGB keywordColor = PreferenceConverter.getColor(store, ModestEditorPlugin.KEYWORD);
		RGB typeColor = PreferenceConverter.getColor(store, ModestEditorPlugin.TYPE);
		RGB distColor = PreferenceConverter.getColor(store, ModestEditorPlugin.DIST);
		RGB singleLineCommentColor = PreferenceConverter.getColor(store, ModestEditorPlugin.SINGLE_LINE_COMMENT);
		RGB multiLineCommentColor = PreferenceConverter.getColor(store, ModestEditorPlugin.MULTI_LINE_COMMENT);
		keywordStyle = new TextAttribute(new Color(null,keywordColor), null, SWT.BOLD);
		typeStyle = new TextAttribute(new Color(null,typeColor), null, SWT.NORMAL);
		distStyle = new TextAttribute(new Color(null,distColor), null, SWT.ITALIC);
		singleLinecommentStyle = new TextAttribute(new Color(null,singleLineCommentColor), null, SWT.NORMAL);
		multiLinecommentStyle = new TextAttribute(new Color(null,multiLineCommentColor), null, SWT.NORMAL);
	}

	/** Disposes the colors allocated in the constructor. */
	/* "Applications must dispose of all colors which they allocate."
	 * Unfortunately, this method is never called. */
	public void dispose() {
		TextAttribute[] styles = {defaultStyle, distStyle, keywordStyle, typeStyle, multiLinecommentStyle, singleLinecommentStyle};
		for (int i = 0; i < styles.length; i++) {
			if (styles[i].getBackground() != null)
				styles[i].getBackground().dispose();
			if (styles[i].getForeground() != null)
				styles[i].getForeground().dispose();
		}
	}
	
	/** Returns the colour and font style for comments in the editor. */
	public TextAttribute getMultiLineCommentStyle () {return multiLinecommentStyle;}
	public TextAttribute getSingleLineCommentStyle () {return singleLinecommentStyle;}
	/** Returns the colour and font style for keywords in the editor. */
	public TextAttribute getKeywordStyle () {return keywordStyle;}
	/** Returns the colour and font style for typedefs in the editor. */
	public TextAttribute getTypeStyle () {return typeStyle;}
	/** Returns the colour and font style for dists in the editor. */
	public TextAttribute getDistStyle () {return distStyle;}
	/** Returns the colour and font style for dists in the editor. */
	public TextAttribute getDefaultStyle () {return defaultStyle;}
	
}
