/**
 *
 * This is part of the MoDeST-Plugin
 * Saarland University 2006
 * Author: Christophe Boutter (boutter@ps.uni-sb.de)
 *
 */

package modesteditor.launch;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class. Generated by Eclipse and slightly modified.
 * Contains the string constants used as identifiers by other classes.
 */
public class ModestLaunchPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "modesteditor.launch";

	// The shared instance
	private static ModestLaunchPlugin plugin;
	// Resource bundle.
	private ResourceBundle resourceBundle;
	
	/** String attribute for the path to the momodest binary */
	public static final String COMPILER_LOCATION = "modestCompilerLocation";
	/** String attribute for the path to the fsns binary */
	public static final String FSNS_LOCATION = "modestFSNSLocation";
	/** String attribute for the path to the output folder of the compiler */
	public static final String OUTPUT_FOLDER_LOCATION = "modestOutputFolderLocation";
	/** String attribute for the dot file creation flag */
	public static final String DOT_FILE_CREATION = "dotFileCreationFlag";
	/** String attribute for the fsns run flag */
	public static final String FSNS_RUN = "fsnsRunFlag";
	/** String attribute for the project name in a launch configuration. */
	public static final String MODEST_CONFIG_PROJECT = "modestConfigProject";
	/** String attribute for the file path in a launch configuration. */
	public static final String MODEST_COMPILE_FILE = "modestCompileFile";
	
	
	/**
	 * The constructor
	 */
	public ModestLaunchPlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle("modesteditor.launch.ModestLaunchPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		initializeDefaultPluginPreferences();
	}
	
	protected void initializeDefaultPreferences (IPreferenceStore store) {
		store.setDefault(COMPILER_LOCATION, "");
		store.setDefault(FSNS_LOCATION, "");
		store.setDefault(OUTPUT_FOLDER_LOCATION, "");
		store.setDefault(DOT_FILE_CREATION, false);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static ModestLaunchPlugin getDefault() {
		return plugin;
	}
	
	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = ModestLaunchPlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

}
